#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>
#include	"logUtil.h"

/* ======================================================================
[infile]
1. MathStatus.txt (Format: {chemi|MM} {DIFF|PARAM|RESULT})   <MathStatus>
2. MathNCE.xml                                               <MathNCE>
3. MathNCEphp.xml                                            <MathNCEphp>
4. MathMM.xml                                                <MathMM>
5. MathMMphp.xml                                             <MathMMphp>
6. MathUsage.txt                                             <MathUsage>
7. MathDAE.txt                                               <MathDAE>
8. MathParam.txt                                             <MathParam>
9. MathUserFunc.txt _____> DIFF,PARAM                        <MathUserFunc>
10.MathCtrl.txt                                              <MathCtrl>
11.MathResult.txt                                            <MathResult>
12.MathResult.svg _____> RESULT                              <MathResultsvg>
[outfile]
1. DBMathModel.xml

case RESULT: MathStatus.txt ({chemi|MM} RESULT)
 {MathNCE.xml|MathMM.xml}
 {MathNCEphp.xml|MathMMphp.xml}
 MathUsage.txt
 MathDAE.txt
 MathParam.txt
 MathUserFunc.txt
 MathCtrl.txt
 MathResult.txt
 MathResult.svg
case DIFF,PARAM: MathStatus.txt ({chemi|MM} {DIFF|PARAM})
 {MathNCE.xml|MathMM.xml}
 {MathNCEphp.xml|MathMMphp.xml}
 MathUsage.txt
 MathDAE.txt
 MathParam.txt
 MathUserFunc.txt
 ====================================================================== */

#define	LC_DIFF_PARAM	7
#define	LC_RESULT		10

FILE	* InFp;
FILE	* OutFp;

static int initialize(int argc, char * argv[]);
static int integrate();
static int copyfile(char * tag, char * fname);
static void finalize();

int main(int argc, char * argv[])
{

#ifdef DBG
	// open debug-log file.
	if (log_dbginit(argc, argv) < 0) {
		printf("Cannot open dbgLog file!!");
		exit(-2);
	}
#endif

	// open sys-log file.
	if(log_sysinit(argc, argv) < 0) {
		printf("Cannot open sysLog file!!");
		exit(-1);
	}

	// Initialize argument, file
	if(initialize(argc, argv) < 0) {
		goto err;
	}

	//
	// intergarate XML file
	//
	if(integrate() < 0) {
		goto err;
	}

	// file finalize
	finalize();

#ifdef DBG
	log_dbgfinalize();
#endif
	log_sysfinalize();
	exit(0);
err:;
#ifdef DBG
	log_dbgfinalize();
#endif
	log_sysfinalize();
	exit(-1);
}

static int initialize(int argc, char * argv[])
{
	int	i, fcount = 0;
	char infname[FILENAME_MAX], outfname[FILENAME_MAX];

	memset(infname, 0, sizeof(infname));
	memset(outfname, 0, sizeof(outfname));

#ifdef DBG
	DBGW(10, ("===> initialize enter\n"));
#endif

	for (i = 0; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
			case 'I':
				fcount++;
				strcpy(infname, &argv[i][2]);
				break;
			case 'O':
				fcount++;
				strcpy(outfname, &argv[i][2]);
				break;
			case 'd':
				break;
			default:
				log_sys("initialize: invalid argument[%s] !!\n", &argv[i][0]);
				goto	err;
			}
		}
	}

	if (fcount != 2) {
		log_sys("initialize: invalid file name!!\n");
		goto	err;
	}

	if (!(InFp = fopen(infname, "r"))) {
		log_sys("initialize: MathStaus[%s] open err!!\n", infname);
		goto	err;
	}
	if (!(OutFp = fopen(outfname, "w"))) {
		log_sys("initialize: DBMathModel[%s] open err!!\n", outfname);
		goto	err;
	}

#ifdef DBG
	DBGW(10, ("===> initialize exit(0)\n"));
#endif
	return(0);
err:;
#ifdef DBG
	DBGW(10, ("===> initialize exit(-1)\n"));
#endif
	return(-1);
}

static int integrate()
{
	int	i, loopcount;
	char	eqtype[FILENAME_MAX], status[FILENAME_MAX];
	char * chemi_name[LC_RESULT][2] = {
						{"MathStatus", "MathStatus.txt"},
						{"MathNCE", "MathNCE.xml"},
						{"MathNCEphp", "MathNCEphp.xml"},
						{"MathUsage", "MathUsage.txt"},
						{"MathDAE", "MathDAE.txt"},
						{"MathParam", "MathParam.txt"},
						{"MathUserFunc", "MathUserFunc.txt"},
						{"MathCtrl", "MathCtrl.txt"},
						{"MathResult", "MathResult.txt"},
						{"MathResult", "MathResult.svg"} };
	char * mm_name[LC_RESULT][2] = { 
						{"MathStatus", "MathStatus.txt"},
						{"MathMM", "MathMM.xml"},
						{"MathMMphp", "MathMMphp.xml"},
						{"MathUsage", "MathUsage.txt"},
						{"MathDAE", "MathDAE.txt"},
						{"MathParam", "MathParam.txt"},
						{"MathUserFunc", "MathUserFunc.txt"},
						{"MathCtrl", "MathCtrl.txt"},
						{"MathResult", "MathResult.txt"},
						{"MathResult", "MathResult.svg"} };
	char head[] = "<\?xml version=\"1.0\" encoding=\"Shift_JIS\"\?>\n<DBMathModel>";
	char tail[] = "</DBMathModel>";

#ifdef DBG
	DBGW(10, ("===> integrate enter\n"));
#endif

	if (fscanf(InFp, "%s %s", eqtype, status) < 0) {
		log_sys("integrate: MathStatus.xml read err!!\n");
		goto	err;
	}
#ifdef DBG
	DBGW(10, ("eqtype[%s], status[%s]\n", eqtype, status));
#endif

	if (fprintf(OutFp, "%s\n", head) < 0) {
		log_sys("integrate: DBMathModel.xml write err!!\n");
		goto	err;
	}
	if (strcmp(eqtype,"chemi") == 0) {
		loopcount = (strcmp(status,"RESULT") == 0) ? LC_RESULT: LC_DIFF_PARAM;
		for (i = 0; i < loopcount; i++) {
			if (copyfile(chemi_name[i][0], chemi_name[i][1]) < 0) {
				goto	err;
			}
		}
	} else if (strcmp(eqtype,"MM") == 0) {
		loopcount = (strcmp(status,"RESULT") == 0) ? LC_RESULT: LC_DIFF_PARAM;
		for (i = 0; i < loopcount; i++) {
			if (copyfile(mm_name[i][0], mm_name[i][1]) < 0) {
				goto	err;
			}
		}
	} else {
		log_sys("integrate: MathStatus.xml eqtype err!! [%s]\n", eqtype);
		goto	err;
	}
	if (fprintf(OutFp, "%s\n", tail) < 0) {
		log_sys("integrate: DBMathModel.xml write err!!\n");
		goto	err;
	}

#ifdef DBG
	DBGW(10, ("===> integrate exit(0)\n"));
#endif
	return(0);
err:;
#ifdef DBG
	DBGW(10, ("===> integrate exit(-1)\n"));
#endif
	return(-1);
}

static int copyfile(char * tag, char * fname)
{
	char	linebuff[BUFSIZ];
	FILE	* fp;
#ifdef DBG
	DBGW(10, ("===> copyfile enter\n"));
#endif

	if (!(fp = fopen(fname, "r"))){
		log_sys("copyfile: file[%s] open err!!\n", fname);
		goto	err;
	}
	if (fprintf(OutFp, "<%s>\n", tag) < 0) {
		log_sys("copyfile: DBMathModel.xml write err!!\n");
		goto	err;
	}
	while (fgets(linebuff, sizeof(linebuff), fp) != NULL) {
		if (fwrite(linebuff, strlen(linebuff), 1, OutFp) != 1) {
			log_sys("copyfile: DBMathModel.xml write err!!\n");
			goto	err;
		}
	}
	if (fprintf(OutFp, "</%s>\n", tag) < 0) {
		log_sys("copyfile: DBMathModel.xml write err!!\n");
		goto	err;
	}
	fclose(fp);

#ifdef DBG
	DBGW(10, ("===> copyfile exit(0)\n"));
#endif
	return(0);
err:;
#ifdef DBG
	DBGW(10, ("===> copyfile exit(-1)\n"));
#endif
	return(-1);
}

static void finalize()
{
	fclose(InFp);
	fclose(OutFp);
}
