/*
 * "R & B": Real-coded GAs & Bit-string GAs
 *
 * crossover module
 */
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "randb.h"
#include "logUtil.h"
#include "utl_alloc.h"
#include "utl_alge.h"
#include "utl_rand.h"

/*
 * Macros
 */
#define MAX(a, b) (a > b ? a : b)
#define MIN(a, b) (a < b ? a : b)

#define PRINT_LOOP_COUNT	10000

/* check_child_region return type */
#define IN_REGION	0
#define	OUT_OF_REGION	1

/*
 * Prottype declaretions of a static function
 */

/*-----
 * blxؿ
 * 	ꤵ줿ƤBLX򺵤Ԥ줿ҸΤ֤
 */
static int blx(const CrossoverParam_T *crossover_param, const GaGene_T *island, int num_variable, const int *parents_no, GaGene_T *child);

/*-----
 * undxmؿ
 * 	ꤵ줿ƤUNDXm򺵤Ԥ줿ҸΤ֤
 */
static int undxm(const CrossoverParam_T *crossover_param, const GaGene_T *island, int num_variable, const int *parents_no, GaGene_T *child);

/*-----
 * spxؿ
 * 	ꤵ줿ƤSPX򺵤Ԥ줿ҸΤ֤
 */
static int spx(const CrossoverParam_T *crossover_param, const GaGene_T *island, int num_variable, const int *parents_no, GaGene_T *child);

/*-----
 * npoints_crossover
 */
static int npoints_crossover(const CrossoverParam_T *crossover_param, const GaGene_T *island, int gene_length, const int *parents_no, GaGene_T *child);	

/*-----
 * check_child_regionؿ
 * 	θΤΰ˼ޤäƤ뤫å
 */
static int check_child_region(const VariableRegion_T *variable_region, int num_variable, const GaGene_T *ga_gene);

/*-----
 * func_comp
 */
int func_comp(const void *a, const void *b);

/***********************
 * Public functions    *
 ***********************/

/*-----
 * Crossoverؿ
 *		GaParam_T˻ꤵ줿ˡ3ǻꤵ줿
 *   	Ƥǡ򺹤ԤҸΤ֤
 */
int Crossover(const GaParam_T *ga_param, const GaGene_T *island, const int *parents_no, GaGene_T *child)
{
	int			num_var;
	int			gene_length;
	int			rv = EXIT_ERROR;
	int			ecount = 0;
	
#ifdef DBG
	DBGW(10, ("Crossover enter.\n"));
#endif /* DBG */
	
	num_var = ga_param->gaEncodeParam.numVariable;

	for(;;){
		switch(ga_param->crossoverParam.crossoverType){
		case BLX:
			rv = blx(&(ga_param->crossoverParam), island, num_var, parents_no, child);
			break;
		case UNDX:
		case UNDXM:
			rv =  undxm(&(ga_param->crossoverParam),island, num_var, parents_no, child);
			break;
		case SPX:
			rv = spx(&(ga_param->crossoverParam), island, num_var, parents_no, child);
			break;
		case NPOINTS:
			gene_length=ga_param->gaEncodeParam.geneLength;
			rv = npoints_crossover(&(ga_param->crossoverParam), island, gene_length, parents_no, child);
			goto out;
		default:
			log_sys(" ## System Error: In Crossover.\n");
			goto out;
		}

		
		/* 줿ҸΤΰ賰äľ */
		if(ga_param->gaEncodeParam.encodeType == REAL){
			if(check_child_region(&(ga_param->variableRegion), num_var, child)
					== IN_REGION){
				if(ecount > PRINT_LOOP_COUNT) printf("LOOP OUT\n");
				break;
			} else {
				ecount++;
				if(ecount % PRINT_LOOP_COUNT == 0) {
					printf("CROSSOVER OUT OF REGION(%d times)\n", ecount);
				}
			}
		}
	}

out:
#ifdef DBG
	DBGW(10, ("Crossover exit(%d).\n", rv));
#endif /* DBG */
	
	return rv;
}
			


/***********************
 * Static functions    *
 ***********************/

/*-----
 * blxؿ
 * 	ꤵ줿ƤBLX򺵤Ԥ줿ҸΤ֤
 */
static int blx(const CrossoverParam_T *crossover_param, const GaGene_T *island, int num_variable, const int *parents_no, GaGene_T *child)
{
	int					i;
	int					rv = EXIT_ERROR;
	double				I;		/*  */
	double				alpha;
	const double		*p1;    /* 1 */
	const double		*p2; 	/* 2 */
#ifdef DBG
	DBGW(10, ("blx enter.\n"));
#endif /* DBG */
	
	alpha = crossover_param->alpha;	

	p1 = island[parents_no[0]].gene_r;
	p2 = island[parents_no[1]].gene_r;

	for(i = 0; i < num_variable; i++){
		I = fabs(p1[i] - p2[i]);
		child->gene_r[i] = GenUniformRand(
						MIN(p1[i], p2[i]) - alpha * I,
						MAX(p1[i], p2[i]) + alpha * I);
	}
	
	child->fitness = -DBL_MAX;

	rv = EXIT_NORMAL;
#ifdef DBG
	DBGW(10, ("blx exit(%d)\n", rv));
#endif /* DBG */
	
	return rv;
}

/*-----
 * undxmؿ
 * 	ꤵ줿ƤUNDXm򺵤Ԥ줿ҸΤ֤
 * 	)Notationϡ
 * 	ִ¿(2000)¿GAΤʬ۸򺵤¿
 * 	ƤѤĥˡơ¬ưزʸVol.36, No.10
 * 	򻲹ͤˤ
 */
static int undxm(const CrossoverParam_T *crossover_param, const GaGene_T *island, int num_variable, const int *parents_no, GaGene_T *child)
{
	int		i, j;
	int		rv = EXIT_ERROR;
	int		m;				/* UNDXmm */
	int		n;				/* õ֤μ */
	int		rank_d;			/* mĤκ٥ȥ뤬ĥʬ֤γ */
	
	double	alpha;				/* õʬɸкĴ */
	double	beta;				/* õʬɸкĴ */
	double	sigma_xi;			/* õʬɸк */
	double	sigma_eta;			/* õʬɸк */
	double	w;					/* õʬ */
	double	D;					/* õʬ礭뷸 */
	double	*p = NULL;			/* ſ */
	double	*random_vec = NULL;	/* õʬ뤿٥ȥ */
	double	*main_search = NULL;/* õʬ */
	double	*sub_search = NULL;	/* õʬ */
	double	*temp_D = NULL;		/* D뤿Υ٥ȥ */
	double	**parents = NULL;	/* (m + 2) */
	double	**dif_vec = NULL;	/* ƤȽſκ٥ȥ */
	double	**red_d = NULL;		/* mĤκ٥ȥ뤬ĥʬ֤ΰΩʬ */
	double	**basis_d = NULL;	/* mĤκ٥ȥ뤬ĥ֤ľ */
	double 	**basis_dt = NULL;	/* 嵭ιž */
	double	**proj_mat = NULL;	/* õʬ뤿õ */
#ifdef DBG
	DBGW(10, ("undxm enter.\n"));
#endif /* DBG */

	alpha = crossover_param->alpha;
	beta = crossover_param->beta;
	m = crossover_param->m;
	n = num_variable;
	
	/* UNDXUNDXMȤǡʬεۤ */
	switch(crossover_param->crossoverType){
	case UNDX:
		sigma_xi = alpha;
		sigma_eta = beta/sqrt(n);
		break;
	case UNDXM:
		sigma_xi = alpha/sqrt(m);
		sigma_eta = (beta * sqrt(m + 1) * sqrt(3))/
				(sqrt(n - m) * sqrt(m + 2) * sqrt(2));
		break;
	default:
		log_sys(" ## System Error: In undxm.\n");
		goto out;
	}

	/* Ѥΰγ */
	if((p = DoubleVectorAlloc(n)) == NULL) goto out;
	if((random_vec = DoubleVectorAlloc(n)) == NULL) goto out;
	if((main_search = DoubleVectorAlloc(n)) == NULL) goto out;
	if((sub_search = DoubleVectorAlloc(n)) == NULL) goto out;
	if((temp_D = DoubleVectorAlloc(n)) == NULL) goto out;
	
	if((parents = DoubleMatrixAlloc(m + 2, n)) == NULL) goto out;
	if((dif_vec = DoubleMatrixAlloc(m + 2, n)) == NULL) goto out;
	if((red_d = DoubleMatrixAlloc(m, n)) == NULL) goto out;
	if((basis_d = DoubleMatrixAlloc(m, n)) == NULL) goto out;
	if((basis_dt = DoubleMatrixAlloc(n, m)) == NULL) goto out;
	if((proj_mat = DoubleMatrixAlloc(n, n)) == NULL) goto out;

	/* Ƥξ򥳥ԡ */
	for(i = 0; i < m + 2; i++){
		memcpy(parents[i], island[parents_no[i]].gene_r, 
				num_variable * sizeof(double));
	}
	
	/* 0mޤǤοƤνſ */
	CalculateCenterOfGravity(parents, m + 1, n, p);

	/* ƿƤνſκ٥ȥ */
	for(i = 0; i < m + 2; i++){
		for(j = 0; j < n; j++){
			dif_vec[i][j] = parents[i][j] - p[j];
		}
	}

	/*-----õʬ-----*/
	memset(main_search, 0, n * sizeof(double));
	for(i = 0; i < m; i++){
		w = GenNormalRand(0, sigma_xi * sigma_xi);
		for(j = 0; j < n; j++){
			main_search[j] += w * dif_vec[i][j];
		}
	}
	
	/*-----õʬ-----*/
	if(n == m){
		/* mõ֤μϡõʬ0 */
		for(i = 0; i < n; i++){
			sub_search[i] = 0;
		}
	} else { /* m < n */	
		/* 0m-1ޤǤοƤκ٥ȥ뤬ĥʬ֤δ */
		/* Ωʥ٥ȥȳ */
		rank_d = ReductMatrix(dif_vec, m, n, red_d);
		
		/* X(basid_d)ľ٥ȥ(٥ȥ)Ĥ¤٤Ȥ */
		/* Gramm-Shimidtľˡˤľ */
		if(rank_d != 0){
			GrammSchmidtOrthogonalize(red_d, rank_d, n, basis_d);
		}

		/* y(random_vec)ʬN(0, sigma_eta^2)˽Ȥȯ */
		for(i = 0; i < n; i++){
				random_vec[i] = GenNormalRand(0, sigma_eta * sigma_eta);
		}
	
		/* y(E - XT * X)õʬȤʤ 
	   	E:(n x n)ñ̹ XT:Xžֹ n:õ֤μ */ 
		if(rank_d != 0){
			TransposedMatrix(basis_d, rank_d, n, basis_dt);
			ProductMatrix(basis_dt, n, rank_d, basis_d, rank_d, n, proj_mat);
		}
			
		/* ͱƹ(E - XT *X) */
		for(i = 0; i < n; i++){
			for(j = 0; j < n; j++){
				if(i == j){
					proj_mat[i][j] = 1.0 - proj_mat[i][j];
				} else {
					proj_mat[i][j] *= -1.0;
				}
			}
		}

		if(rank_d != 0){
			ProductMatrix(&random_vec, 1, n, proj_mat, n, n, &sub_search);
			ProductMatrix(&(dif_vec[m + 1]), 1, n, proj_mat, n, n, &temp_D);
			D = VectorNorm(temp_D, n);
		} else {
			memcpy(sub_search, random_vec, n * sizeof(double));
			D = VectorNorm(dif_vec[m + 1], n);
		}
	}

	/* ҸΤ */
	for(i = 0; i < n; i++){
		child->gene_r[i] = p[i] + main_search[i] + D * sub_search[i];
	}
	
	rv = EXIT_NORMAL;
out:
	if(p != NULL) FreeDoubleVector(p);
	if(random_vec != NULL) FreeDoubleVector(random_vec);
	if(main_search != NULL) FreeDoubleVector(main_search);
	if(sub_search != NULL) FreeDoubleVector(sub_search);
	if(temp_D != NULL) FreeDoubleVector(temp_D);
		
	if(parents != NULL) FreeDoubleMatrix(parents);
	if(dif_vec != NULL) FreeDoubleMatrix(dif_vec);
	if(red_d != NULL) FreeDoubleMatrix(red_d);
	if(basis_d != NULL) FreeDoubleMatrix(basis_d);
	if(basis_dt != NULL) FreeDoubleMatrix(basis_dt);
	if(proj_mat != NULL) FreeDoubleMatrix(proj_mat);
#ifdef DBG
	DBGW(10, ("undxm exit(%d)\n", rv));
#endif /* DBG */
	
	return rv;
}
	
/*-----
 * spxؿ
 * 	ꤵ줿ƤSPX򺵤Ԥ줿ҸΤ֤
 * 	)Notationϡ
 * 	(2001)¿GAˤ륷ץå򺵤
 * 	͹ǽزʸ161Qפ򻲹ͤˤ
 */
static int spx(const CrossoverParam_T *crossover_param, const GaGene_T *island, int num_variable, const int *parents_no, GaGene_T *child)
{
	int			i, j, k;
	int 		num_parent;				/* Ƥο */
	int			rv = EXIT_ERROR;		/* ؿ */
	double		epsilon;				/* ĥΨ */
	double		rk;						/* u(0,1)^(1/(k+1)) */
	double		**parents = NULL;		/* ƤΰҤ */
	double		*child_gene = NULL;		/* ҸΤΰ */
	double		*g = NULL;				/* ſ */
	double		**xk = NULL;			
	double		**ck = NULL;			
#ifdef DBG
	DBGW(10, ("spx enter.\n"));
#endif /* DBG */
	

	epsilon    = crossover_param->alpha;
	num_parent = crossover_param->numParent;
	
	/* Ѥΰγ */
	if((parents	= DoubleMatrixAlloc(num_parent, num_variable)) == NULL){
		goto out;
	}
	if((child_gene = DoubleVectorAlloc(num_variable)) == NULL){
		goto out;
	}
	if((ck	= DoubleMatrixAlloc(num_parent, num_variable)) == NULL){
		goto out;
	}
	if((g	= DoubleVectorAlloc(num_variable)) == NULL){
		goto out;
	}
	if((xk	= DoubleMatrixAlloc(num_parent, num_variable)) == NULL){
		goto out;
	}

	for(i = 0; i < num_parent; i++){
		memcpy(parents[i], island[parents_no[i]].gene_r, 
				num_variable * sizeof(double));
	}

	/* Ƥθηνſ */
	CalculateCenterOfGravity(parents, num_parent, num_variable, g);

	/* xk[k] = g + epsilon * (parents[k] - g) ʸ(3)*/
	for(k = 0; k < num_parent; k++){
		for(j = 0; j < num_variable; j++){
			xk[k][j] = g[j] + epsilon * (parents[k][j] - g[j]);
		}
	}

	/* ck[0] = 0 ʸ(4)*/
	memset(ck[0], 0, num_variable * sizeof(double));

	/* xk[k] = rk[k] * (xk[k - 1] - xk[k] + ck[k-1]) ʸ(4)*/
	for(k = 1; k < num_parent; k++){
		rk = pow(GenUniformRand(0, 1), 1/((double)k+1)); /* ʸ(5) */
		for(j = 0; j < num_variable; j++){
			ck[k][j] = rk * (xk[k-1][j] - xk[k][j] + ck[k-1][j]);
		}
	}

	/* child_gene = xk[num_parent - 1] + ck[num_parent - 1] ʸ(7) */
	for(i = 0; i < num_variable; i++){
		child_gene[i] = xk[num_parent - 1][i] + ck[num_parent - 1][i];
	}

	memcpy(child->gene_r, child_gene, num_variable * sizeof(double));
	child->fitness = -DBL_MAX;

	rv = EXIT_NORMAL;
	
out:
	if(parents != NULL) FreeDoubleMatrix(parents);
	if(child_gene != NULL) FreeDoubleVector(child_gene);
	if(ck != NULL) FreeDoubleMatrix(ck);
	if(g != NULL) FreeDoubleVector(g);
	if(xk != NULL) FreeDoubleMatrix(xk);
#ifdef DBG
	DBGW(10, ("spx exit(%d)\n", rv));
#endif /* DBG */
	
	return rv;
}

/*-----
 * npoints_crossover
 */
static int npoints_crossover(const CrossoverParam_T *crossover_param, const GaGene_T *island, int gene_length, const int *parents_no, GaGene_T *child)
{
	int	i, j;
	int	n;
	int crossover_table[BGA_GENE_MAX]; 
#ifdef DBG
	DBGW(10, ("npoints_crossover enter.\n"));
#endif /* DBG */
	
#ifdef XOVR_DBG
	for(i = 0; i < gene_length; i++){
		printf("%d, ", island[parents_no[0]].gene_b[i]);
	}
	printf("\n");
	for(i = 0; i < gene_length; i++){
		printf("%d, ", island[parents_no[1]].gene_b[i]);
	}
	printf("\n");
#endif /* XOVR_DBG */
			
	n = crossover_param->m;

	for (i = 0; i < n;){
		crossover_table[i] = GenIntRand(1, gene_length);
		for(j = 0; j < i; j++){
			if(crossover_table[i] == crossover_table[j]){
				continue;
			}
		}
		i++;
	}
	
	qsort(crossover_table, n, sizeof(int), func_comp);
	
	memcpy(&(child[0]), &(island[parents_no[0]]), sizeof(GaGene_T));
	memcpy(&(child[1]), &(island[parents_no[1]]), sizeof(GaGene_T));
	for (j = 0; j < n; j++){
		for(i = crossover_table[j]; i < gene_length; i++){
			if(j % 2 == 0){
				child[0].gene_b[i] = island[parents_no[1]].gene_b[i];
				child[1].gene_b[i] = island[parents_no[0]].gene_b[i];
			} else {
				child[0].gene_b[i] = island[parents_no[0]].gene_b[i];
				child[1].gene_b[i] = island[parents_no[1]].gene_b[i];
			}
		}
	}
#ifdef XOVR_DBG
	for(i = 0; i < gene_length; i++){
		printf("%d, ", child[0].gene_b[i]);
	}
	printf("\n");
	for(i = 0; i < gene_length; i++){
		printf("%d, ", child[1].gene_b[i]);
	}
	printf("\n");
#endif /* XOVR_DBG */

#ifdef DBG	
	DBGW(10, ("npoints_crossover exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}	





	
/*-----
 * check_child_regionؿ
 * 	θΤΰ˼ޤäƤ뤫å
 * 	͡IN_REGION, OUT_OF_REGION
 * 	ΰVariableRegion_T*(I), ѿοint(i), åGaGene_T*(I)
 */
static int check_child_region(const VariableRegion_T *variable_region, int num_variable, const GaGene_T *ga_gene)
{
	int	i;
	int rv = OUT_OF_REGION;
#ifdef DBG
	DBGW(10, ("check_child_region enter.\n"));

	/*
	for(i = 0; i < num_variable; i++){
		printf("%e,%e,%e\n", variable_region->minArray[i], 
						variable_region->maxArray[i], ga_gene->gene_r[i]);
	}*/
#endif /* DBG */
			
	for(i = 0; i < num_variable; i++){
#ifndef WIN32
		if(isnan(ga_gene->gene_r[i])) exit(0);
#endif
		if(ga_gene->gene_r[i] < variable_region->minArray[i] ||
			ga_gene->gene_r[i] > variable_region->maxArray[i]){
			goto out;
		}
	}

	rv = IN_REGION;
out:	
#ifdef DBG
	DBGW(10, ("check_child_region exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}

/*-----
 * func_comp
 */
int func_comp(const void *a, const void *b){
	int rv;
#ifdef DBG
	DBGW(10, ("func_comp enter.\n"));
#endif /* DBG */
	
	rv = (*(int*)a - *(int*)b);
	
#ifdef DBG
	DBGW(10, ("func_comp exit(%d)\n", rv));
#endif /* DBG */
	return rv;
}	
