/*
 * "R & B": Real-coded GAs & Bit-string GAs
 *
 * Island operation modele
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "randb.h"
#include "logUtil.h"
#include "utl_alloc.h"
#include "utl_rand.h"

/*
 * Prottype declarations of a static function
 */

/*-----
 * generate_gene_r_random
 */
static int generate_gene_r_random(const VariableRegion_T *variable_region, GaGene_T *out_gene);

/*-----
 * generate_gene_b_random
 */
static int generate_gene_b_random(int geneLength, GaGene_T *out_gene);

/***********************
 * Public functions    *
 ***********************/

/*-----
 * GenerateInitialPopulation
 */
int GenerateInitialPopulation(GaIsland_T *island, GaEncodeParam_T *ga_encode_param, VariableRegion_T *variable_region)
{
	int i;

#ifdef DBG
	DBGW(10, ("GenerateInitialPopulation enter.\n"));
#endif /* DBG */
	
	switch(ga_encode_param->encodeType){
	case REAL:
		for(i = 0; i < island->population; i++){
			generate_gene_r_random(variable_region, 
									&(island->residents[i]));
		}
		break;
	case BINARY:
	case GRAY:
		for(i = 0; i < island->population; i++){
			generate_gene_b_random(ga_encode_param->geneLength,
									&(island->residents[i]));
		}
	default: break;
	}

	for(i = 0; i < island->population; i++){
		island->sortedIndex[i] = i;
	}

#ifdef DBG
	DBGW(10, ("GenerateInitialPopulation exit(0).\n"));
#endif /* DBG */
	
	return EXIT_NORMAL;
}

/*-----
 * IslandFitness
 */
int IslandFitness(const GaParam_T *ga_param, GaIsland_T *island, int begin, int end)
{
	int		i;
	int		*sorted_index;
	int		rv = EXIT_ERROR;

#ifdef DBG
	DBGW(10, ("IslandFitness enter.\n"));
#endif /* DBG */
	
	sorted_index = island->sortedIndex;
	
	for(i = begin; i < end; i++){
		if(Fitness(ga_param, &(island->residents[sorted_index[i]]), 
				&(island->residents[sorted_index[i]].fitness)) == EXIT_ERROR){
			goto out;
		}
	}

	rv = EXIT_NORMAL;
out:
#ifdef DBG
	DBGW(10, ("IslandFitness exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}

/*-----
 * IslandCrossoverNormal
 */
int IslandCrossoverNormal(GaParam_T *ga_param, GaIsland_T *island, int begin, int end) 
{
	int		i;
	int		island_pop;
	int		rv = EXIT_ERROR;
	int		parents[NUM_PARENT_MAX];
	GaGene_T temp[ISLAND_POP_MAX];

#ifdef DBG
	DBGW(10, ("IslandCrossoverNormal enter.\n"));
#endif /* DBG */

	island_pop = island->population;
	
	if(ga_param->gaEncodeParam.encodeType == REAL){
		for(i = begin; i < end; i++){
			if(SelectParent(ga_param, island, parents) == EXIT_ERROR){
				goto out;
			}
			if(Crossover(ga_param, island->residents, parents, &(temp[i])) == EXIT_ERROR){
				goto out;
			}
		}
	} else {
		for(i = begin; i < end; i += 2){
			if(SelectParent(ga_param, island, parents) == EXIT_ERROR){
				goto out;
			}
			if(Crossover(ga_param, island->residents, parents, &(temp[i])) == EXIT_ERROR){
				goto out;
			}
		}
	}

	for(i = begin; i < end; i++){
		island->residents[island->sortedIndex[i]] = temp[i];
	}	
		
	
	rv = EXIT_NORMAL;
out:
#ifdef DBG
	DBGW(10, ("IslandCrossoverNormal exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}

/*-----
 * IslandMggDigenesis
 */
int IslandMggDigenesis(GaParam_T *ga_param, GaIsland_T *island)
{
	int		i;
	int		num_parent;
	int		num_children;
	int 	mgg_size;
	int 	survive_no;
	int		exchange[2];
	int		rv = EXIT_ERROR;
	int		parents[NUM_PARENT_MAX];
	GaIsland_T 	temp_island;

#ifdef DBG
	DBGW(10, ("IslandMggDigenesis enter.\n"));
#endif /* DBG */

	num_parent = ga_param->crossoverParam.numParent;
	num_children = ga_param->digenesisParam.numMGGChildren;
	mgg_size = num_children + 2;
	temp_island.population = mgg_size;

	if(SelectParent(ga_param, island, parents) == EXIT_ERROR){
		goto out;
	}	

	for(i = 0; i < temp_island.population; i++){
		temp_island.sortedIndex[i] = i;
	}
	
	if(ga_param->gaEncodeParam.encodeType == REAL){
		for(i = 0; i < num_children; i++){
			if(Crossover(ga_param, island->residents, parents, 
								&(temp_island.residents[i])) == EXIT_ERROR){
				goto out;
			}
		}
	} else {
		for(i = 0; i < num_children; i += 2){
			if(Crossover(ga_param, island->residents, parents, 
								&(temp_island.residents[i])) == EXIT_ERROR){
				goto out;
			}
		}
	}

	if(IslandFitness(ga_param, &temp_island, 0, num_children) == EXIT_ERROR){
		goto out;
	}

	exchange[0] = GenIntRand(0, num_parent);
	for( ;; ){
		exchange[1] = GenIntRand(0, num_parent);
		if(exchange[0] != exchange[1]) break;
	}

	temp_island.residents[num_children] = 
			island->residents[parents[exchange[0]]];
	temp_island.residents[num_children + 1] = 
			island->residents[parents[exchange[1]]];

	SortByFitness(&temp_island);

	for( ;; ){
		if(SelectRoulette(&temp_island, 1, &survive_no) == EXIT_ERROR){
			goto out;
		}
		if(survive_no != temp_island.sortedIndex[0]){
			break;
		}
	}

	island->residents[parents[exchange[0]]]
			= temp_island.residents[temp_island.sortedIndex[0]];
	island->residents[parents[exchange[1]]]
			= temp_island.residents[survive_no];

	rv = EXIT_NORMAL;
out:
#ifdef DBG
	DBGW(10, ("IslandMggDigenesis exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}
	
/*-----
 * IslandMutation
 */
int IslandMutation(GaParam_T *ga_param, GaIsland_T *island, int begin, int end)
{
	int i;
	int	*sorted_index;
	int rv = EXIT_ERROR;
	GaGene_T 	best_gene;
	GaGene_T	temp[ISLAND_POP_MAX];

#ifdef DBG
	DBGW(10, ("IslandMutation enter.\n"));
#endif /* DBG */
	
	sorted_index = island->sortedIndex;
	
	best_gene = island->residents[sorted_index[0]];
	for(i = begin; i < end; i++){
		if(Mutation(ga_param, &(island->residents[sorted_index[i]]), &best_gene, &(temp[i])) == EXIT_ERROR){
			goto out;
		}
	}

	for(i = begin; i < end; i++){
		island->residents[sorted_index[i]] = temp[i];
	}	

	rv = EXIT_NORMAL;
out:
#ifdef DBG
	DBGW(10, ("IslandMutation exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}
	
		
		
/***********************
 * Static functions    *
 ***********************/

/*-----
 * generate_gene_r_random
 */
static int generate_gene_r_random(const VariableRegion_T *variable_region, GaGene_T *out_gene)
{
	int i;

#ifdef DBG
	DBGW(10, ("generate_gene_r_random enter.\n"));
#endif /* DBG */

	for(i = 0; i < variable_region->arraySize; i++){
		out_gene->gene_r[i] = 
			GenUniformRand(variable_region->minArray[i],
							variable_region->maxArray[i]);
	}

	out_gene->fitness = -DBL_MAX;

#ifdef DBG
	DBGW(10, ("generate_gene_r_random exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}

/*-----
 * generate_gene_b_random
 */
static int generate_gene_b_random(int geneLength, GaGene_T *out_gene)
{
	int i;

#ifdef DBG
	DBGW(10, ("generate_gene_b_random enter.\n"));
#endif /* DBG */

	for(i = 0; i < geneLength; i++){
		out_gene->gene_b[i] = GenIntRand(0,2);
	}

	out_gene->fitness = -DBL_MAX;

#ifdef DBG
	DBGW(10, ("generate_gene_b_random exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}
