#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define _HAVE_MAIN_ROUTINE_

#include "logUtil.h"
#include "randb.h"

#ifndef NFS
#include "utl_mpi_ft.h"
#endif /* NFS */

#ifdef MPI
#include <libgen.h>
#include "gaMpi.h"
#endif
#ifdef LIFE

#include "life.h"
#include "ga2ls.h"
#include "lsInterface.h"


static char *fName[FILE_NUM];
static int adm_argc_argv(GaParam_T *ga_param, int argc, char *argv[]);
#endif /* LIFE */

static int my_init(GaParam_T *ga_param, int argc, char *argv[]);
static void my_final();

int main(int argc, char **argv)
{
	int			rv = EXIT_ERROR;
	GaParam_T	ga_param;

#ifdef MPI
	MPI_Init(&argc, &argv);
#endif

	if(my_init(&ga_param, argc, argv) == EXIT_ERROR){
		goto out;
	}

#ifdef DBG
	DBGW(10, ("main enter(after my_init).\n"));
#endif /* DBG */

#ifdef MPI	
	switch(ga_param.digenesisParam.digenesisType){
	case NORMAL_DIGENESIS:
		if(ga_param.mpiInfo.numProcs == 1){
			if(NormalExecute(&ga_param) == EXIT_ERROR){
				goto out;
			}
		} else { 
			if(NormalExecuteMpi(&ga_param) == EXIT_ERROR){
				goto out;
			}
		}
		break;
	case MGG:
		if(ga_param.mpiInfo.numProcs == 1){
			if(MGGExecute(&ga_param) == EXIT_ERROR){
				goto out;
			}
		} else {
			if(MGGExecuteMpi(&ga_param) == EXIT_ERROR){
			 goto out;
			}
		}
		break;
	default:
		log_sys(" ## System Error: In main.\n");
		goto out;
	}
#else /* MPI */

	switch(ga_param.digenesisParam.digenesisType){
	case NORMAL_DIGENESIS:
		if(NormalExecute(&ga_param) == EXIT_ERROR){
			goto out;
		}
		break;
	case MGG:
		if(MGGExecute(&ga_param) == EXIT_ERROR){
			goto out;
		}
		break;
	default:
		log_sys(" ## System Error: In main.\n");
		goto out;
	}

#endif /* MPI */
	

	rv = EXIT_NORMAL;
out:	

#ifdef DBG
	DBGW(10, ("main exit(%d)(before my_finale)\n", rv));
#endif /* DBG */

	my_final();
	
#ifdef MPI
	if(rv == EXIT_ERROR){
		MPI_Abort(MPI_COMM_WORLD, EXIT_ERROR);
	}

	MPI_Finalize();
#endif /* MPI */

	return rv;
}

static int my_init(GaParam_T *ga_param, int argc, char *argv[])
{
	int	rv = EXIT_ERROR;
#ifdef LIFE
	int i;
	double		t_start, t_end, h;
	double		dt_save;
	double		y[MAX_VAR_NUM];
#endif /* LIFE */

#ifdef MPI
	MPI_Datatype	mpi_param_t;
#endif /* MPI */

	
#ifdef DBG
	// open debug-log file.
	if(log_dbginit(argc, argv)) {
		printf("Cannot open dbgLog file!!");
		goto	out;
	}

	DBGW(10, ("my_init enter\n", rv));
#endif

	// open sys-log file.
	if(log_sysinit(argc, argv)) {
		printf("Cannot open sysLog file!!");
		goto	out;
	}
	InitializeParam(ga_param);
	SetCommandLineInfo(&(ga_param->comLine), argc, argv);
	
#ifdef MPI
	GetMpiProcInfo(&(ga_param->mpiInfo));
	TypeParamStruct(ga_param, &mpi_param_t);
	
	if(ga_param->mpiInfo.myProcRank == 0){ /* master */

#ifdef LIFE
		if(adm_argc_argv(ga_param, argc, argv) == EXIT_ERROR){
			goto out;
		}
		if(ga_param->mpiInfo.numProcs > 1){		
			for(i = 1; i < ga_param->mpiInfo.numProcs; i++){
				if(MPI_Send(fName[FILE_IN], strlen(fName[FILE_IN]) + 1, 
					MPI_CHAR, i, 0, MPI_COMM_WORLD) != MPI_SUCCESS){
					log_sys(" ## Error: MPI_Send(In my_init).\n");
					goto out;
				}
			}

#ifndef NFS
			if(BcastFileSend(fName[FILE_IN]) == EXIT_ERROR){
				goto out;
			}
#endif /* NFS */
		}
		
#else /* LIFE */
		if(AdminCommandLine(&(ga_param->ioParam), argc, argv) == EXIT_ERROR){
			goto out;
		}
		if(SetRegionFromFile(ga_param) == EXIT_ERROR){
			goto out;
		}
#endif /* LIFE */
	} else { /* slave */

#ifdef LIFE
		char buf[FILENAME_MAX];	
		char in_file_name[FILENAME_MAX];
		MPI_Status stat;
		if(MPI_Recv(buf, FILENAME_MAX, MPI_CHAR, 0, 
					0, MPI_COMM_WORLD, &stat) != MPI_SUCCESS){
			log_sys(" ## Error: MPI_Recv(In my_init).\n");
			goto out;
		}
#ifdef NFS
		strcpy(in_file_name, buf);
#else /* NFS */
		sprintf(in_file_name, "%s_%d", basename(buf), ga_param->mpiInfo.myProcRank);
#endif /* NFS */	
		if((fName[FILE_IN] = strdup(in_file_name)) == NULL){
			log_sys(" ## Error: strdup(In my_init).\n");
			goto out;
		}

#ifndef NFS
		if(BcastFileRecv(fName[FILE_IN]) == EXIT_ERROR){
			goto out;
		}
#endif /* NFS */
		
#endif /* LIFE */
	}
		
#ifdef LIFE		
	if(Set_data(y, &t_start, &t_end, &h, &dt_save, fName) < 0){
		goto out;
	}
	PreserveDefaultLsParam(t_start, t_end, h, dt_save, y);
	if(AllocSolutionMatrix() == EXIT_ERROR){
		goto out;
	}
	if(ReadRegionFromLS(ga_param) == EXIT_ERROR){
		goto out;
	}
	
	for(i = 0; i < P_GaNum; i++){
		if(!strcmp("constantPlayer", Pno2Name(P_Ga[i].p_no))){
			log_sys(" ## Error: constantPlayer cannot be set as subject to search.\n");
			goto out;
		}
	}
#endif  /* LIFE */

	if(ga_param->mpiInfo.myProcRank == 0){ /* master */			
		if(SetParamsFromFile(ga_param) == EXIT_ERROR){
			goto out;
		}

		SetDelta(ga_param);
		
		DistributeParams(ga_param, mpi_param_t);

#ifndef LIFE		
		DistributeVariableRegion(ga_param);
#endif /* LIFE */
		
		PrintExecuteParameters(ga_param, stdout);
		if(ga_param->ioParam.outputFile != NULL){
			PrintExecuteParameters(ga_param, ga_param->ioParam.outputFile);
		}
		
	} else { /* slave */
		AcceptParams(ga_param, mpi_param_t);
#ifdef LIFE
		SetDelta(ga_param);

#ifndef NFS
#ifndef DBG
		if(unlink(fName[FILE_IN]) < 0){
			log_sys(" ## Error: unlink(In my_init).\n");
			goto out;
		}
#endif /* DBG */
#endif /* NFS */
		
#else /* LIFE */
		AcceptVariableRegion(ga_param);

#endif /* LIFE */
	}


	
#else /* MPI */

#ifdef LIFE
	if(adm_argc_argv(ga_param, argc, argv) == EXIT_ERROR){
		goto out;
	}
	if(Set_data(y, &t_start, &t_end, &h, &dt_save, fName) < 0){
		goto out;
	}
	PreserveDefaultLsParam(t_start, t_end, h, dt_save, y);
	if(AllocSolutionMatrix() == EXIT_ERROR){
		goto out;
	}
	if(ReadRegionFromLS(ga_param) == EXIT_ERROR){
		goto out;
	}		

#else /* LIFE */
	
	if(AdminCommandLine(&(ga_param->ioParam), argc, argv) == EXIT_ERROR){
		goto out;
	}
	if(SetRegionFromFile(ga_param) == EXIT_ERROR){
		goto out;
	}

#endif /* LIFE */

	if(SetParamsFromFile(ga_param) == EXIT_ERROR){
		goto out;
	}

	SetDelta(ga_param);
		
	PrintExecuteParameters(ga_param, stdout);
	if(ga_param->ioParam.outputFile != NULL){
		PrintExecuteParameters(ga_param, ga_param->ioParam.outputFile);
	}

#endif /* MPI */	
	
	rv = EXIT_NORMAL;
out:
#ifdef DBG
	DBGW(10, ("my_init exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}

#ifdef LIFE
#include "utl_arg.h"
static int adm_argc_argv(GaParam_T *ga_param, int argc, char *argv[])
{
#define LFLAG_I	(1 << 0)
#define LFLAG_p (1 << 1)
#define LFLAG_f	(1 << 2)
#define LFLAG_s (1 << 3)
		
	int	ch, foptions = 0;
	int rv = EXIT_ERROR;
	char *cargv[NUM_OPTION_MAX];

	if(argc > NUM_OPTION_MAX){
		log_sys(" ## Error: Too many options!\n");
		goto out;
	}

	if(CopyArgv(argc, argv, cargv) == EXIT_ERROR){
		goto out;
	}

	ga_param->ioParam.individualFileName[0] = '\0';
	
	if(opterr != 0) opterr = 0;
	while((ch = getopt(argc, cargv, "I:O:p:f:s:")) != EOF){
		switch(ch){
		case 'I':
			if (foptions & LFLAG_I) {
				log_sys(" ## Error: Duplicated option: \'%c\'\n", ch);
				goto out;
			}
			fName[FILE_IN] = strdup(optarg);
			foptions |= LFLAG_I;
			break;
		case 'O':
			log_sys(" ## Warning: -O option has no effect to GA.\n");
			break;
		case 'p':
			if (foptions & LFLAG_p) {
				log_sys(" ## Error: Duplicated option: \'%c\'\n", ch);
				goto out;
			}
			strcpy(ga_param->ioParam.paramFileName, optarg);
			foptions |= LFLAG_p;
			break;
		case 'f':
			if (foptions & LFLAG_f) {
				log_sys(" ## Error: Duplicated option: \'%c\'\n", ch);
				goto out;
			}
			strcpy(ga_param->ioParam.outputFileName, optarg);
			if ((ga_param->ioParam.outputFile
				=fopen(optarg, "w")) == NULL) {
				log_sys(" ## Error: The output file cannot open: \'%s\'\n", optarg);
				goto out;
			}
			foptions |= LFLAG_f;
			break;
		case 's':
			if (foptions & LFLAG_s) {
				log_sys(" ## Error: Duplicated option: \'%c\'\n", ch);
				goto out;
			}
			strcpy(ga_param->ioParam.individualFileName, optarg);
			foptions |= LFLAG_s;
			break;	
		default:
				break;
		}
	}

	if (!(foptions & LFLAG_I)){
		log_sys(" ## ERROR: Option -I cannot be omitted.\n");
		goto out;
	}
	
	if (!(foptions & LFLAG_p)){
		log_sys(" ## ERROR: Option -p cannot be omitted.\n");
		goto out;
	}

	rv = EXIT_NORMAL;
out:
	optind = 1;
	FreeCargv(argc, cargv);
	
	return rv;	
}
#endif /* LIFE */

static void my_final()
{
#ifdef DBG
	DBGW(10, ("my_final enter.\n"));
#endif /* DBG */

#ifdef LIFE
	if(SolutionMatrix != NULL) FreeSolutionMatrix();

	/*
	 * ѥ᡼γǼΰγ
	 */
	freeParamTag();
#endif /* LIFE */
	
	/*
	 * λ
	 */
	log_sysfinalize();
#ifdef DBG
	DBGW(10, ("my_final exit\n"));
	log_dbgfinalize();
#endif /* DBG */
}

