/*
 * "R & B": Real-coded GAs & Bit-string GAs
 *
 * digenesis module
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mpi.h>

#include "randb.h"
#include "gaMpi.h"
#include "logUtil.h"
#include "utl_alloc.h"
#include "utl_rand.h"

static void copy_island(GaIsland_T *to, GaIsland_T *from)
{
	int		copy_size;
#ifdef DBG
	DBGW(10, ("copy_island enter.\n"));
#endif /* DBG */

	copy_size = sizeof(GaIsland_T) - sizeof(GaIsland_T*) - sizeof(int*);
	
	memcpy(to, from, copy_size);
#ifdef DBG
	DBGW(10, ("copy_island exit\n"));
#endif /* DBG */

}

/*-----
 * NormalExecuteMpiؿ
 * 	NormalExecuteؿMPI
 */
int NormalExecuteMpi(GaParam_T *ga_param)
{
	int		i;
	int		proc_rank;	/* ΥץΥ */
	int		num_procs;	/* MPI󤹤ץο */ 
	int		rv = EXIT_ERROR;
	GaIsland_T		temp_island;
	MPI_Datatype	mpi_island_t;
	MPI_Status		stat;

#ifdef DBG
	DBGW(10, ("NormalExecuteMpi enter.\n"));
#endif /* DBG */

	proc_rank  = ga_param->mpiInfo.myProcRank;
	num_procs  = ga_param->mpiInfo.numProcs;

	if(TypeIslandStruct(ga_param, &temp_island, &mpi_island_t) == EXIT_ERROR){
		goto out;
	}

#ifndef DBG	
	SetSeedByTime();
#endif /* DBG */
	if(proc_rank == 0){
		int		generation;	/* generation */
		int		num_island;
		int		tag;		/* MPIԤݤΥ */
		int		num_send;	/* ο */
		int		num_recv;	/* ο */
		int		*proc_adm = NULL;
		GaIsland_T	*islands = NULL;
		
		num_island = ga_param->gaTypeParam.numIsland;
		
		if((proc_adm = IntVectorAlloc(num_procs)) == NULL){
			goto master_out;
		}
		if((islands = GaIslandVectorAlloc(num_island)) == NULL){
			goto master_out;
		}
		if(MakeImmigrationBuffer(ga_param, islands) == EXIT_ERROR){
			goto master_out;
		}
		
		/* Τȯ */
		for(i = 0; i < num_island; i++){
			islands[i].population = ga_param->gaTypeParam.islandPopulation;
			GenerateInitialPopulation(&(islands[i]),
					&(ga_param->gaEncodeParam), &(ga_param->variableRegion));
		}

		/* Υե뤬ꤵƤϡɤ߹ */ 
		if(strlen(ga_param->ioParam.individualFileName) > 0){
			if(SetIndividualFromFile(ga_param, islands) == EXIT_ERROR){
				goto out;
			}
		}

		tag = FITNESS_ONLY; /* Ŭٷ׻Τ */
		for(generation = 0; 
			generation < ga_param->gaTypeParam.maxGeneration; generation++){
			/* ܽäܽ */
			if(generation != 0
				&&ga_param->gaTypeParam.gaType == DGA 
				&& ga_param->gaTypeParam.immigrationSwitch != IMMIGRATION_OFF
				&& generation%(ga_param->gaTypeParam.immigrationInterval) == 0){
				if(Immigrate(ga_param, islands) == EXIT_ERROR){
					goto master_out;
				}
			}
			/* ä */
			if(ga_param->gaTypeParam.gaType == DIGA
				&& generation == ga_param->gaTypeParam.integrateGeneration){
				IntegrateIslands(ga_param, islands);
				AjustParamsAfterIntegrate(ga_param);
				num_island = 1;
			}

			/* ܽʳνϥ졼֤˰ */
			for(num_send = 0; num_send < num_procs - 1; num_send++){
				if(num_send == num_island){
					break;
				}
				if(SendIsland(&(islands[num_send]), mpi_island_t, num_send+1,
							tag, MPI_COMM_WORLD) == EXIT_ERROR){
					goto master_out;
				}
				proc_adm[num_send + 1] = num_send;
			}
			num_recv = 0;
			for(;;){
				int copyto; /* 졼֤γǼ */
				if(RecvIsland(&temp_island, mpi_island_t, MPI_ANY_SOURCE,
						MPI_ANY_TAG, MPI_COMM_WORLD, &stat) == EXIT_ERROR){
					goto master_out;
				}
				copyto = proc_adm[stat.MPI_SOURCE];
				copy_island(&(islands[copyto]), &temp_island);
				num_recv++;
				if(num_send < num_island){
					if(SendIsland(&(islands[num_send]), mpi_island_t,
						stat.MPI_SOURCE, tag, MPI_COMM_WORLD) == EXIT_ERROR){
						goto master_out;
					}
					proc_adm[stat.MPI_SOURCE] = num_send;
					num_send++;
				}
				if(num_recv == num_island){
					break;
				}
			}
			/* ¹ԤˤɸΤ¸ */
			RecordBestGene(generation, islands, num_island, 
							&(ga_param->bestGene));

			/* ɸϤӥեؤν */
			PrintBestGeneAtPresent(generation, ga_param);
			if(ga_param->ioParam.outputFile != NULL){
				PrintFileAll(generation, ga_param, islands);
			}

			/* λȽ */
			if(ga_param->bestGene.gaGene.fitness > 
				ga_param->gaTypeParam.finValue){
				break;
			}	
			tag = NORMAL_TAG;
		}

		/* ǽã顢졼֤˽λ̿ */
		for(i = 0; i < num_procs -1; i++){
			if(SendIsland(&temp_island, mpi_island_t, i + 1,
					END_TAG, MPI_COMM_WORLD) == EXIT_ERROR){
				goto master_out;
			}
		}
		
		PrintResult(ga_param);
		rv = EXIT_NORMAL;
		
master_out:
		FreeImmigrationBuffer(ga_param, islands);

		if(proc_adm != NULL) FreeIntVector(proc_adm);
		if(islands != NULL) FreeGaIslandVector(islands);
		if(ga_param->ioParam.outputFile != NULL)
			fclose(ga_param->ioParam.outputFile);
		
	} else { /* slave */
		int		island_pop;
		int		num_parent;	/* Ƥο */
		int		num_elite;	/* ꡼ȿ */
		GaIsland_T 	island;

		island_pop = ga_param->gaTypeParam.islandPopulation;		
		num_parent = ga_param->crossoverParam.numParent;
		num_elite  = ga_param->digenesisParam.numElite;

		for(;;){
			if(RecvIsland(&island, mpi_island_t, MPI_ANY_SOURCE,
					MPI_ANY_TAG, MPI_COMM_WORLD, &stat) == EXIT_ERROR){
				goto out;
			}
			switch(stat.MPI_TAG){
			case END_TAG:
				rv = EXIT_NORMAL;
				goto out;
			case NORMAL_TAG:
				/* ȸ */
				if(ga_param->crossoverParam.crossoverType != NO_CROSSOVER){
					if(IslandCrossoverNormal(ga_param, &island,
								num_elite, island_pop) == EXIT_ERROR){
						goto out;
					}
				}
				/* Ѱ */
				if(ga_param->mutationParam.mutationType != NO_MUTATION){
					if(IslandMutation(ga_param, &island,
								num_elite, island_pop) == EXIT_ERROR){
						goto out;
					}
				}
				/* Ŭ٤׻Ŭ٤ǥ */
				if(IslandFitness(ga_param, &island, num_elite, island_pop)
						== EXIT_ERROR){
					goto out;
				}
				SortByFitness(&island);
				break;
			case FITNESS_ONLY:
				/* Ŭ٤׻Ŭ٤ǥ */
				if(IslandFitness(ga_param, &island, 0, island_pop) == EXIT_ERROR){
					goto out;
				}
				SortByFitness(&island);
				break;
			default:
				log_sys(" ## System Error: In NormalExecuteMpi.\n");
				goto out;
			}
			if(SendIsland(&island, mpi_island_t, 0, 
							NORMAL_TAG, MPI_COMM_WORLD) == EXIT_ERROR){
				goto out;
			}
		}
	}
	
out:
#ifdef DBG
	DBGW(10, ("NormalExecuteMpi exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}
			

/*-----
 * MGGExecuteMpiؿ
 * 	MGGExecuteؿMPI
 */
int MGGExecuteMpi(GaParam_T *ga_param)
{
	int		i;
	int		proc_rank;	/* ΥץΥ */
	int		num_procs;	/* MPI󤹤ץο */ 
	int		rv = EXIT_ERROR;
	GaIsland_T		temp_island;
	MPI_Datatype	mpi_island_t;
	MPI_Status		stat;
#ifdef DBG
	DBGW(10, ("MGGExecuteMpi enter.\n"));
#endif /* DBG */

	proc_rank  = ga_param->mpiInfo.myProcRank;
	num_procs  = ga_param->mpiInfo.numProcs;
	
	if(TypeIslandStruct(ga_param, &temp_island, &mpi_island_t) == EXIT_ERROR){
		goto out;
	}
	
#ifndef DBG
	SetSeedByTime();
#endif /* DBG */
	if(proc_rank == 0){
		int		generation;	/* generation */
		int		num_island;
		int		tag;		/* MPIԤݤΥ */
		int		num_send;	/* ο */
		int		num_recv;	/* ο */
		int		*proc_adm = NULL;
		GaIsland_T	*islands = NULL;

		num_island = ga_param->gaTypeParam.numIsland;
		
		if((proc_adm = IntVectorAlloc(num_procs)) == NULL){
			goto master_out;
		}
		if((islands = GaIslandVectorAlloc(num_island)) == NULL){
			goto master_out;
		}
		if(MakeImmigrationBuffer(ga_param, islands) == EXIT_ERROR){
			goto master_out;
		}
		
		/* Τȯ */
		for(i = 0; i < num_island; i++){
			islands[i].population = ga_param->gaTypeParam.islandPopulation;
			GenerateInitialPopulation(&(islands[i]), 
					&(ga_param->gaEncodeParam), &(ga_param->variableRegion));
		}
		
		/* Υե뤬ꤵƤϡɤ߹ */ 
		if(strlen(ga_param->ioParam.individualFileName) > 0){
			if(SetIndividualFromFile(ga_param, islands) == EXIT_ERROR){
				goto out;
			}
		}

		tag = FITNESS_ONLY; /* Ŭٷ׻Τ */
		for(generation = 0; 
			generation < ga_param->gaTypeParam.maxGeneration; generation++){
			/* ܽäܽ */
			if(generation != 0
				&&ga_param->gaTypeParam.gaType == DGA 
				&& ga_param->gaTypeParam.immigrationSwitch != IMMIGRATION_OFF
				&& generation%(ga_param->gaTypeParam.immigrationInterval) == 0){
				if(Immigrate(ga_param, islands) == EXIT_ERROR){
					goto master_out;
				}
			}
			/* ä */
			if(ga_param->gaTypeParam.gaType == DIGA
				&& generation == ga_param->gaTypeParam.integrateGeneration){
				IntegrateIslands(ga_param, islands);
				AjustParamsAfterIntegrate(ga_param);
				num_island = 1;
			}
			/* ܽʳνϥ졼֤˰ */
			for(num_send = 0; num_send < num_procs - 1; num_send++){
				if(num_send == num_island){
					break;
				}
				if(SendIsland(&(islands[num_send]), mpi_island_t, num_send+1,
							tag, MPI_COMM_WORLD) == EXIT_ERROR){
					goto master_out;
				}
				proc_adm[num_send + 1] = num_send;
			}
			num_recv = 0;
			for(;;){
				int copyto; /* 졼֤γǼ */
				if(RecvIsland(&temp_island, mpi_island_t, MPI_ANY_SOURCE,
						MPI_ANY_TAG, MPI_COMM_WORLD, &stat) == EXIT_ERROR){
					goto master_out;
				}
				copyto = proc_adm[stat.MPI_SOURCE];
				copy_island(&(islands[copyto]), &temp_island);
				num_recv++;
				if(num_send < num_island){
					if(SendIsland(&(islands[num_send]), mpi_island_t,
						stat.MPI_SOURCE, tag, MPI_COMM_WORLD) == EXIT_ERROR){
						goto master_out;
					}
					proc_adm[stat.MPI_SOURCE] = num_send;
					num_send++;
				}
				if(num_recv == num_island){
					break;
				}
			}

			/* ¹ԤˤɸΤ¸ */
			RecordBestGene(generation, islands, num_island, 
							&(ga_param->bestGene));

			/* ɸϤӥեؤν */
			PrintBestGeneAtPresent(generation, ga_param);
			if(ga_param->ioParam.outputFile != NULL){
				PrintFileAll(generation, ga_param, islands);
			}

			/* λȽ */
			if(ga_param->bestGene.gaGene.fitness > 
				ga_param->gaTypeParam.finValue){
				break;
			}	
			tag = NORMAL_TAG;
		}
		/* ǽã顢졼֤˽λ̿ */
		for(i = 0; i < num_procs -1; i++){
			if(SendIsland(&temp_island, mpi_island_t, i + 1,
					END_TAG, MPI_COMM_WORLD) == EXIT_ERROR){
				goto master_out;
			}
		}
			
		PrintResult(ga_param);
		rv = EXIT_NORMAL;
master_out:
		FreeImmigrationBuffer(ga_param, islands);

		if(proc_adm != NULL) FreeIntVector(proc_adm);
		if(islands != NULL) FreeGaIslandVector(islands);
		if(ga_param->ioParam.outputFile != NULL)
			fclose(ga_param->ioParam.outputFile);

		
	} else { /* slave */
		int		island_pop;
		GaIsland_T	island;
		
		island_pop = ga_param->gaTypeParam.islandPopulation;
		
		for(;;){
			if(RecvIsland(&island, mpi_island_t, MPI_ANY_SOURCE,
					MPI_ANY_TAG, MPI_COMM_WORLD, &stat) == EXIT_ERROR){
				goto out;
			}
			switch(stat.MPI_TAG){
			case END_TAG:
				rv = EXIT_NORMAL;
				goto out;
			case NORMAL_TAG:
				/* MGGˤ */
				if(ga_param->crossoverParam.crossoverType != NO_CROSSOVER){
					if(IslandMggDigenesis(ga_param, &island) == EXIT_ERROR){
						goto out;
					}
				}
				/* Ѱ */
				if(ga_param->mutationParam.mutationType != NO_MUTATION){
					if(IslandMutation(ga_param, &island, 0, island_pop) == EXIT_ERROR){
						goto out;
					}
					if(IslandFitness(ga_param, &island, 0, island_pop) == EXIT_ERROR){
						goto out;
					}
				}
				SortByFitness(&island);
				break;	
			case FITNESS_ONLY:
				/* Ŭ٤׻Ŭ٤ǥ */
				if(IslandFitness(ga_param, &island, 0, island_pop) == EXIT_ERROR){
					goto out;
				}
				SortByFitness(&island);
				break;
			default:
				log_sys(" ## System Error: In NormalExecuteMpi.\n");
				goto out;
			}
			if(SendIsland(&island, mpi_island_t, 0, 
							NORMAL_TAG, MPI_COMM_WORLD) == EXIT_ERROR){
				goto out;
			}
		}
	}
	
out:
#ifdef DBG
	DBGW(10, ("MGGExecuteMpi exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}
