/*
 * "R & B": Real-coded GAs & Bit-string GAs
 *
 * mutation module
 */

#include <assert.h>
#include <math.h>

#include "randb.h"
#include "logUtil.h"
#include "utl_rand.h"

/*
 * Prottype declations of a static function
 */

/*-----
 * region_uniؿ
 *	ΰѰۤԤˤѰۤΤ֤
 */
static int region_uni(const GaGene_T *ga_gene, double mutation_rate, const VariableRegion_T *variable_region, GaGene_T *out_gene);

/*-----
 * fixed_uniؿ
 * 	ѰۤԤˤѰۤΤ֤
 *	ΰۤͤäϡͤդ
 *	ΰ˼ޤޤǷ֤
 */
static int fixed_uni(const GaGene_T *ga_gene, double mutation_rate, double fixed_width, const VariableRegion_T *variable_region, GaGene_T *out_gene);

/*-----
 * fixed_normalؿ
 * 	ѰۤԤˤѰۤΤ֤
 *	ΰۤͤäϡͤդ
 *	ΰ˼ޤޤǷ֤
 */
static int fixed_normal(const GaGene_T *ga_gene, double mutation_rate, double sd, const VariableRegion_T *variable_region, GaGene_T *out_gene);

/*-----
 * variable_uniؿ
 * 	ѰۤԤˤѰۤΤ֤
 *	ΰۤͤäϡͤդ
 *	ΰ˼ޤޤǷ֤
 */
static int variable_uni(const GaGene_T *ga_gene, const GaGene_T *island_bestgene, double mutation_rate, const VariableRegion_T *variable_region, GaGene_T *out_gene);

/*-----
 * variable_normalؿ
 * 	ѰۤԤˤѰۤΤ֤
 *	ΰۤͤäϡͤդ
 *	ΰ˼ޤޤǷ֤
 */
static int variable_normal(const GaGene_T *ga_gene, const GaGene_T *island_bestgene, double mutation_rate, const VariableRegion_T *variable_region, GaGene_T *out_gene);

/*-----
 * bit_reverse
 */
static int bit_reverse(GaGene_T *ga_gene, double mutation_rate, int gene_length);


/***********************
 * Public functions    *
 ***********************/

/*-----
 * Mutationؿ
 *      ꤵ줿ˡǡѰۤԤ줿Τ֤
 */
int Mutation(const GaParam_T *ga_param, GaGene_T *ga_gene, const GaGene_T *island_bestgene, GaGene_T *out_gene){
	int			i;
	int			rv = EXIT_ERROR;
	double		mutation_rate;
	double		width;
	double 		num_var;

#ifdef DBG
	DBGW(10, ("Mutation enter.\n"));
#endif /* DBG */
	
	mutation_rate = ga_param->mutationParam.mutationRate;
	num_var = ga_param->gaEncodeParam.numVariable;

#ifdef MUT_DBG
	printf(" ** Input gene\n");
	for(i = 0; i < ga_param->gaEncodeParam.geneLength; i++){
		if(ga_param->gaEncodeParam.encodeType == REAL){
			printf("%f, ", ga_gene->gene_r[i]);
		} else {
			printf("%d, ", ga_gene->gene_b[i]);
		}
	}
	printf("\n");
#endif /* MUT_DBG */	
	
	switch(ga_param->mutationParam.mutationType){
	case REGION_UNI:
		rv = region_uni(ga_gene, mutation_rate,
				&(ga_param->variableRegion), out_gene);
		break;
	case FIXED_UNI:
		width = ga_param->mutationParam.fixedWidth;
		rv = fixed_uni(ga_gene, mutation_rate, width, 
				&(ga_param->variableRegion), out_gene);
		break;
	case FIXED_NORMAL:
		width = ga_param->mutationParam.fixedWidth;
		rv = fixed_normal(ga_gene, mutation_rate, width, 
				&(ga_param->variableRegion), out_gene);
		break;
	case VARIABLE_UNI:
		rv = variable_uni(ga_gene, island_bestgene, mutation_rate,
				&(ga_param->variableRegion), out_gene);
		break;
	case VARIABLE_NORMAL:
		rv = variable_normal(ga_gene, island_bestgene, 
			mutation_rate, &(ga_param->variableRegion), out_gene);
		break;
	case BIT_REVERSE:
		for(i = 0; i < ga_param->gaEncodeParam.geneLength; i++){
			out_gene->gene_b[i] = ga_gene->gene_b[i];
		}
		rv = bit_reverse(out_gene, mutation_rate, 
						ga_param->gaEncodeParam.geneLength);
		break;
	default:
		log_sys(" ## System Error: In Mutation.\n");
		goto out;
	}

#ifdef MUT_DBG
	printf(" ** Output gene\n");
	for(i = 0; i < ga_param->gaEncodeParam.geneLength; i++){
		if(ga_param->gaEncodeParam.encodeType == REAL){
			printf("%f, ", out_gene->gene_r[i]);
		} else {
			printf("%d, ", out_gene->gene_b[i]);
		}
	}
	printf("\n");
#endif /* MUT_DBG */
	
out:
#ifdef DBG
	DBGW(10, ("Mutation exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}	
		
		
/***********************
 * Static functions    *
 ***********************/

/*-----
 * region_uniؿ
 *	ΰѰۤԤˤѰۤΤ֤
 *	͡GaGene_T
 *	θGaGene_T*(I), ѰΨdouble(I),
 *	      ѿΰVariableRegion_T*(I), ѿο(I)
 */
static int region_uni(const GaGene_T *ga_gene, double mutation_rate, const VariableRegion_T *variable_region, GaGene_T *out_gene)
{
	int		i;
	int 	num_var;

#ifdef DBG
	DBGW(10, ("region_uni enter.\n"));
#endif /* DBG */

	num_var = variable_region->arraySize;
	
	for (i = 0; i < num_var; i++){
		if(GenUniformRand(0, 1) < mutation_rate){
			out_gene->gene_r[i] = 
				GenUniformRand(variable_region->minArray[i],
						variable_region->maxArray[i]);
		} else {
			out_gene->gene_r[i] = ga_gene->gene_r[i];
		}
	}

	out_gene->fitness = -DBL_MAX;

#ifdef DBG
	DBGW(10, ("region_uni exit(0)\n"));
#endif /* DBG */
	
	return EXIT_NORMAL;
}

/*-----
 * fixed_uniؿ
 * 	ѰۤԤˤѰۤΤ֤
 *	ΰۤͤäϡͤդ
 *	ΰ˼ޤޤǷ֤
 *	θGaGene_T*(I), ѰΨdouble(I),double(I)
 *	      ѿΰVariableRegion_T*(I), ѿο(I)
 */
static int fixed_uni(const GaGene_T *ga_gene, double mutation_rate, double fixed_width, const VariableRegion_T *variable_region, GaGene_T *out_gene)
{
	int		i;
	int 	num_var;

#ifdef DBG
	DBGW(10, ("fixed_uni enter.\n"));
#endif /* DBG */
	
	num_var = variable_region->arraySize;

	for (i = 0; i < num_var;){
		if(GenUniformRand(0, 1) < mutation_rate){
			out_gene->gene_r[i] = 
				GenUniformRand(ga_gene->gene_r[i] - fixed_width, 
								ga_gene->gene_r[i] + fixed_width);
			if(out_gene->gene_r[i] < variable_region->minArray[i] ||
				out_gene->gene_r[i] > variable_region->maxArray[i]){
				continue;
			}
		} else {
			out_gene->gene_r[i] = ga_gene->gene_r[i];
		}
		i++;
	}

	out_gene->fitness = -DBL_MAX;

#ifdef DBG
	DBGW(10, ("fixed_uni exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}

/*-----
 * fixed_normalؿ
 * 	ѰۤԤˤѰۤΤ֤
 *	ΰۤͤäϡͤդ
 *	ΰ˼ޤޤǷ֤
 *	θGaGene_T*(I), ѰΨdouble(I),ɸкdouble(I)
 *	      ѿΰVariableRegion_T*(I), ѿο(I)
 */
static int fixed_normal(const GaGene_T *ga_gene, double mutation_rate, double sd, const VariableRegion_T *variable_region, GaGene_T *out_gene)
{
	int		i;
	int 	num_var;
#ifdef DBG
	DBGW(10, ("fixed_normal enter.\n"));
#endif /* DBG */

	num_var = variable_region->arraySize;

	for (i = 0; i < num_var;){
		if(GenUniformRand(0, 1) < mutation_rate){
			out_gene->gene_r[i] =
				GenNormalRand(ga_gene->gene_r[i], sd * sd);
			if(out_gene->gene_r[i] < variable_region->minArray[i] ||
				out_gene->gene_r[i] > variable_region->maxArray[i]){
				continue;
			}
		} else {
			out_gene->gene_r[i] = ga_gene->gene_r[i];
		}
		i++;
	}

	out_gene->fitness = -DBL_MAX;
#ifdef DBG
	DBGW(10, ("fixed_normal exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}

/*-----
 * variable_uniؿ
 * 	ѰۤԤˤѰۤΤ֤
 *	ΰۤͤäϡͤդ
 *	ΰ˼ޤޤǷ֤
 *	θGaGene_T*(I), Τ°κɸGaGene_T*(I)
 *	      ѰΨdouble(I), ѿΰVariableRegion_T*(I), ѿο(I)
 */
static int variable_uni(const GaGene_T *ga_gene, const GaGene_T *island_bestgene, double mutation_rate, const VariableRegion_T *variable_region, GaGene_T *out_gene)
{
	int		i;
	double	width;
	int 	num_var;
#ifdef DBG
	DBGW(10, ("variable_uni enter.\n"));
#endif /* DBG */

	num_var = variable_region->arraySize;
	
	for (i = 0; i < num_var;){
		width = fabs(island_bestgene->gene_r[i] - ga_gene->gene_r[i]);
		if(GenUniformRand(0, 1) < mutation_rate){
			out_gene->gene_r[i] =
				GenUniformRand(ga_gene->gene_r[i] - width,
					ga_gene->gene_r[i] + width);
			if(out_gene->gene_r[i] < variable_region->minArray[i] ||
				out_gene->gene_r[i] > variable_region->maxArray[i]){
				continue;
			}
		} else {
			out_gene->gene_r[i] = ga_gene->gene_r[i];
		}
		i++;
	}

	out_gene->fitness = -DBL_MAX;

#ifdef DBG
	DBGW(10, ("variable_uni exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}

/*-----
 * variable_normalؿ
 * 	ѰۤԤˤѰۤΤ֤
 *	ΰۤͤäϡͤդ
 *	ΰ˼ޤޤǷ֤
 *	θGaGene_T*(I), Τ°κɸGaGene_T*(I)
 *	      ѰΨdouble(I), ѿΰVariableRegion_T*(I), ѿο(I)
 */
static int variable_normal(const GaGene_T *ga_gene, const GaGene_T *island_bestgene, double mutation_rate, const VariableRegion_T *variable_region, GaGene_T *out_gene)
{
	int		i;
	double	sd;
	int 	num_var;

#ifdef DBG
	DBGW(10, ("variable_normal enter.\n"));
#endif /* DBG */

	num_var = variable_region->arraySize;

	for (i = 0; i < num_var;){
		sd = fabs(island_bestgene->gene_r[i] - ga_gene->gene_r[i]);
		if(GenUniformRand(0, 1) < mutation_rate){
			out_gene->gene_r[i] =
				GenNormalRand(ga_gene->gene_r[i], sd * sd);
			if(out_gene->gene_r[i] < variable_region->minArray[i] ||
				out_gene->gene_r[i] > variable_region->maxArray[i]){
				continue;
			}
		} else {
			out_gene->gene_r[i] = ga_gene->gene_r[i];
		}
		i++;
	}

	out_gene->fitness = -DBL_MAX;

#ifdef DBG
	DBGW(10, ("variable_normal exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}

/*-----
 * bit_reverse
 */
static int bit_reverse(GaGene_T *ga_gene, double mutation_rate, int gene_length)
{
	int		i;

#ifdef DBG
	DBGW(10, ("bit_reverse enter.\n"));
#endif /* DBG */

#ifdef MUT_DBG
	for(i = 0; i < gene_length; i++){
		printf("%d, ", ga_gene->gene_b[i]);
	}
	printf("\n");
#endif /* MUT_DBG */

	for(i = 0; i < gene_length; i++){
		if(GenUniformRand(0, 1) < mutation_rate){
			ga_gene->gene_b[i] ^= 1;
		}
	}

#ifdef MUT_DBG
	for(i = 0; i < gene_length; i++){
		printf("%d, ", ga_gene->gene_b[i]);
	}
	printf("\n");
#endif /* MUT_DBG */
	
#ifdef DBG
	DBGW(10, ("bit_reverse exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}
	
