/*
 * Real-coded Genetic Algorithm
 *
 * selection module
 */
#include <stdlib.h>
#include <assert.h>

#include "randb.h"
#include "utl_rand.h" 
#include "utl_alloc.h"
#include "logUtil.h"

/*
 * Macros
 */

/* check_select_no's return type */
#define NOT_SELECT	1
#define SELECT_YET	2

/* search_roulette_index's return type */
#define NOT_FOUND	-1

/*
 * Prottype declarations of a static function
 */

/*-----
 * check_select_noؿ
 * 	NOФƤʤɤå
 */
static int check_select_no(int *parent_no, int num_select_yet, int check_no);

/*-----
 * search_roulette_index
 * 	ꤵ줿ͤɤοƤֹؤΤ򸡺
 */
static int search_roulette_index(double *roulette, int roulette_size, double key);


/***********************
 * Public functions    *
 ***********************/

/*-----
 * SelectParentؿ
 * 	Ŭڤ˰ʲ3ĤδؿƤӡ	
 * 	̤Ф줿Ƥֹ֤
 */
int SelectParent(const GaParam_T *ga_param, const GaIsland_T *island, int *parents_no)
{
	int	num_select;
	int	tournament_size;
	int 	rv;

#ifdef DBG
	DBGW(10, ("SelectParent enter.\n"));
#endif /* DBG */
	
	num_select = ga_param->crossoverParam.numParent;
	tournament_size = ga_param->digenesisParam.tournamentSize;
	
	switch(ga_param->digenesisParam.selectionType){
	case ROULETTE:
		rv = SelectRoulette(island, num_select, parents_no);
		break;
	case TOURNAMENT:
	  rv = SelectTournament(island, num_select, tournament_size, parents_no);
		break;
	case RANDOM:
		rv = SelectRandom(island, num_select, parents_no);
		break;
	default:
		log_sys(" ## System Error: In SelectParent.\n");
		assert(1);
	}

#ifdef DBG
	DBGW(10, ("SelectParent exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}

/*-----
 * SelectRouletteؿ
 * 	롼åǿƤ
 */
int SelectRoulette(const GaIsland_T *island, int num_select, int *parents_no)
{
	int		i;
	int		zero_count = 0;
	int 	island_pop;
	int		rv = EXIT_ERROR;
	double	min_fitness;		/* κǾŬ */
	double 	sum_fitness = 0;	/* (Ŭ-ǾŬ)ι */
	double	*roulette = NULL;	/* 롼å */

#ifdef DBG
	DBGW(10, ("SelectRoulette enter.\n"));
#endif /* DBG */

	island_pop = island->population;
	min_fitness = 
			island->residents[island->sortedIndex[island_pop - 1]].fitness;
	
	/* 롼åȤκ */
	if((roulette = DoubleVectorAlloc(island_pop)) == NULL){
		goto out;
	}
	
	roulette[0] = 
		island->residents[island->sortedIndex[0]].fitness - min_fitness;
	for(i = 1; i < island_pop; i++){
		roulette[i] = roulette[i-1] + 
			island->residents[island->sortedIndex[i]].fitness - min_fitness;
		if (roulette[i] == 0) zero_count++;
	}
	if(island_pop - zero_count < num_select){
		log_sys(" ## Error: Infinite roop in roulette.\n");
		goto out;
	}
	sum_fitness = roulette[island_pop - 1];

	/* Ԥ */
	for(i = 0; i < num_select;){
		double 	deme;	/* 롼åȤν */
		int 	candidate;
		
		deme = GenUniformRand(0, sum_fitness);
		candidate = search_roulette_index(roulette, island_pop, deme);
		if(candidate == NOT_FOUND){
			log_sys(" ## System Error:In SelectRoulette.\n");
			goto out;
		}
		
		if(check_select_no(parents_no, i, 
						island->sortedIndex[candidate]) == SELECT_YET){
			continue;
		}

		parents_no[i] = island->sortedIndex[candidate];
		i++;
	}

	rv = EXIT_NORMAL;
out:
	if(roulette != NULL) FreeDoubleVector(roulette);

#ifdef DBG
	DBGW(10, ("SelectRoulette exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}

/*-----
 * SelectTounamentؿ
 * 	ȡʥǿƤ
 */
int SelectTournament(const GaIsland_T *island, int num_select, int tournament_size, int *parents_no)
{
	int	i, j;
	int	candidate;
	int	new_candidate;
	int	island_pop;

#ifdef DBG
	DBGW(10, ("SelectTournament enter.\n"));
#endif /* DBG */
	
	island_pop = island->population;
	
	for(j = 0; j < num_select;){
		for(i = 0; i < tournament_size; i ++){
			if(i == 0){	
				candidate = GenIntRand(0, island_pop);
			} else {
				new_candidate = GenIntRand(0,island_pop);
				if(island->residents[island->sortedIndex[candidate]].fitness 
					< island->residents[island->sortedIndex[new_candidate]].fitness){
					candidate = new_candidate;
				}
			}
		}

		if(check_select_no(parents_no, j, 
						island->sortedIndex[candidate]) == SELECT_YET){
			continue;
		}

		parents_no[j] = island->sortedIndex[candidate];
		j++;
	}

#ifdef DBG
	DBGW(10, ("SelectTournament exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}
	
	
		

	
/*------
 * SelectRandomؿ
 * 	Ŭ٤̵뤷ƥ()˿Ƥ
 */
int SelectRandom(const GaIsland_T *island, int num_select, int *parents_no)
{
	int 	i;
	int	candidate;
	int island_pop;

#ifdef DBG
	DBGW(10, ("SelectRandom enter.\n"));
#endif /* DBG */

	island_pop = island->population;
	
	for(i=0; i < num_select; ){
		candidate = GenIntRand(0, island_pop);

		if(check_select_no(parents_no, i, 
						island->sortedIndex[candidate]) ==SELECT_YET){
			continue;
		}

		parents_no[i] = island->sortedIndex[candidate];
		i++;
	}

#ifdef DBG
	DBGW(10, ("SelectRandom exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}


/***********************
 * Static functions    *
 ***********************/
		
/*-----
 * check_select_noؿ
 * 	NOФƤʤɤå
 * 	͡NOT_SELECT, SELECT_YET
 * 	Ƥֹint*(I), 
 * 	      礭int(I), Τֹint(I)
 */
static int check_select_no(int *parent_no, int num_select_yet, int check_no)
{
	int i;
	int rv;

#ifdef DBG
	DBGW(10, ("check_select_no enter.\n"));
#endif /* DBG */

	for (i = 0; i < num_select_yet; i++){
		if(parent_no[i] == check_no){
			rv = SELECT_YET;
			goto out;
		}
	}

	rv =  NOT_SELECT;
out:
#ifdef DBG
	DBGW(10, ("check_select_no exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}

/*-----
 * search_roulette_index
 * 	ꤵ줿ͤɤοƤֹؤΤ򸡺
 * 	õ르ꥺʬõ
 */
static int search_roulette_index(double *roulette, int roulette_size, double key)
{
	int low, high, middle;
	int rv;

#ifdef DBG
	DBGW(10, ("search_roulette_index enter.\n"));
#endif /* DBG */

	low = 0;
	high = roulette_size - 1;

	while(low <= high){
		double 	inf, sup;
	
		middle = (low + high)/2;
		if(middle == 0) {
			inf = 0.0;
		} else {
			inf = roulette[middle - 1];
		}
		sup = roulette[middle];
		
		if(key <= sup && key >= inf){
			rv = middle;
			goto out;
		} else if(key < inf) {
			high = middle - 1;
		} else {
			low = middle + 1;
		}
	}

	rv = NOT_FOUND;
out:
#ifdef DBG
	DBGW(10, ("search_roulette_index exit(%d)\n", rv));
#endif /* DBG */
	return rv;
}
