#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "randb.h"

#include "utl_alloc.h"
#include "utl_encode.h"
#include "utl_i2d.h"
#include "utl_text.h"

static int print_read_summary(FILE *fp, char *file_name, int *read_island_pop, int num_island, int num_read_island, int island_pop);
		
int SetIndividualFromFile(GaParam_T *ga_param, GaIsland_T *islands)
{
	int i;
	int num_var;
	int num_island;
	int island_pop;
	int rv = EXIT_ERROR;
	int individual_no = 0;
	int island_no = 0;
	char *read_top;
	char buf[BUFSIZ];
	FILE *fp;
	int *read_island_pop;
	int	transform_buf[NUM_VARIABLE_MAX];
	double readin_buf[NUM_VARIABLE_MAX];
	
#ifdef DBG
	DBGW(10, ("SetIndividualFromFile enter.\n"));
#endif /* DBG */

	
	num_var = ga_param->gaEncodeParam.numVariable;
	num_island = ga_param->gaTypeParam.numIsland;
	island_pop = ga_param->gaTypeParam.islandPopulation;

	if((fp = fopen(ga_param->ioParam.individualFileName, "r")) == NULL){
		log_sys(" ## Error: Cannot open a file of initial individuals:%s\n",
						ga_param->ioParam.individualFileName);
		goto out;
	}

	if((read_island_pop = IntVectorAlloc(num_island)) == NULL){
		goto out;
	}
	memset(read_island_pop, 0, num_island * sizeof(int));
	
	while(fgets(buf, sizeof(buf), fp) != NULL){
		printf("%s", buf);
		
		/* ȹԤξ̵ */ 
		if (IsCommentLine(buf) == COMMENT_LINE){
			continue;
		}

		/*
		 * 롼()ζڤ꤬ä
		 * ꤵ줿οʲɤ߹οϿɤ߹ֹ򹹿
		 * ꤵ줿礭ֿΤ߷׿
		 */
		if (!strcmp(buf,"**\n")){
			printf("new island\n");
			if(island_no < num_island){
				read_island_pop[island_no] = individual_no;
			}
			island_no++;
			individual_no = 0;
			continue;
		}

		/* ξɤ߹ */		
		if(island_no < num_island && individual_no < island_pop){
			/* ǽ2ϻѤʤ */
			if(strtok(buf, ",\n") == NULL || strtok(NULL, ",\n") == NULL){
				log_sys(" ## Error: Invalid records in a file of initial individuals.\n");
				goto out;
			}

			for(i = 0; (read_top = strtok(NULL, ",\n")) != NULL ; i++){
				if(i < NUM_VARIABLE_MAX) readin_buf[i] = atof(read_top);
			}
			
			if(i != num_var){
				log_sys(" ## Error: Invalid dimensions of a file of initial individuales.\n");
				goto out;
			}

			if(ga_param->gaEncodeParam.encodeType == REAL){ /* RGA */
				/* ɤ߹ͤ򤽤Τޤޥԡ */
				memcpy(islands[island_no].residents[individual_no].gene_r,
							readin_buf, sizeof(double) * num_var);
			} else if(ga_param->gaEncodeParam.encodeType == BINARY){ /* BGA(Binary) */
				/* ¿ͤͤѴ */
				Double2Int(&(ga_param->gaEncodeParam), &(ga_param->variableRegion), readin_buf, transform_buf, num_var);
				/* ͤХʥꥳǥ󥰤ƽ񤭹 */
				if(BinaryEncode(transform_buf, num_var, 
						islands[island_no].residents[individual_no].gene_b,
						ga_param->gaEncodeParam.geneLength,
						ga_param->gaEncodeParam.quantumNumber) == EXIT_ERROR){
					goto out;
				}
			} else if(ga_param->gaEncodeParam.encodeType == GRAY){ /* BGA(Gray) */
				/* ¿ͤͤѴ */
				Double2Int(&(ga_param->gaEncodeParam), &(ga_param->variableRegion), readin_buf, transform_buf, num_var);

				/* ͤ򥰥쥤ǥ󥰤ƽ񤭹 */
				if(GrayEncode(transform_buf, num_var, 
						islands[island_no].residents[individual_no].gene_b,
						ga_param->gaEncodeParam.geneLength,
						ga_param->gaEncodeParam.quantumNumber) == EXIT_ERROR){
					goto out;
				}
			} else {
				log_sys(" ## System Error: In SetIndividualFromFile.\n");
				goto out;
			}
		}
		individual_no++;
	}
	if(island_no <= num_island){
		read_island_pop[island_no] = individual_no;
	}
	island_no++;
	
	/* ɤ߹󥵥ޥ꡼ɽ */
	print_read_summary(stdout, ga_param->ioParam.individualFileName, read_island_pop,
					num_island, island_no, island_pop);
	
	rv = EXIT_NORMAL;
out:

#ifdef DBG
	DBGW(10, ("SetIndividualFromFile exit(%d).\n", rv));
#endif /* DBG */

	return rv;
}

static int print_read_summary(FILE *fp, char *file_name, int *read_island_pop, int num_island, int num_read_island, int island_pop){
	int i;

#ifdef DBG
	DBGW(10, ("print_read_summary enter.\n"));
#endif /* DBG */

	fprintf(fp, "===Initial Generation File==============\n");
	fprintf(fp, "File name:%s\n", file_name);
	fprintf(fp, "Island, Population, Number of read in, Random, Redundant\n");
	for(i = 0; i < num_island; i++){
		fprintf(fp, "Island[%d], %d, %d, %d, %d\n",
				i, island_pop, read_island_pop[i],
				read_island_pop[i] > island_pop ? 0 : island_pop - read_island_pop[i],
				read_island_pop[i] > island_pop ? read_island_pop[i] - island_pop : 0);
	}
	if(num_island < num_read_island){
		fprintf(fp, "Number of redundant islands: %d\n", num_read_island - num_island);
	}
	fprintf(fp, "========================================\n");

#ifdef DBG
	DBGW(10, ("print_read_summary exit(0)\n"));
#endif /* DBG */

	return 0;
}
