#include "randb.h"
#include "logUtil.h"
#include "utl_i2d.h"

/*
 * static functions
 */
static int binary_search(double in_value, double min, double delta, int qn);


int Int2Double(const VariableRegion_T *variable_region, const int *in_array, double *out_array, int array_size)
{
	int i;

#ifdef DBG
	DBGW(10, ("Int2Double enter.\n"));
	for(i = 0; i < array_size; i++){
		DBGW(15, ("%d, ", in_array[i]));
	}
	DBGW(15, ("\n"));
#endif /* DBG */
	
	for(i = 0; i < array_size; i++){
		out_array[i] = variable_region->minArray[i] + 
			in_array[i] * variable_region->delta[i];
	}

#ifdef DBG
	for(i = 0; i < array_size; i++){
		DBGW(15, ("%f, ", out_array[i]));
	}
	DBGW(15, ("\n"));
	
	DBGW(10, ("Int2Double exit(0)\n"));
#endif /* DBG */

	return EXIT_NORMAL;
}

int Double2Int(const GaEncodeParam_T *encode_param, const VariableRegion_T *variable_region, const double *in_array, int *out_array, int array_size)
{
	int i;
	int rv = EXIT_ERROR;
	
#ifdef DBG
	DBGW(10, ("Double2Int enter.\n"));
#endif /* DBG */
	
	for(i = 0; i < array_size; i++){
		out_array[i] = binary_search(in_array[i], variable_region->minArray[i],
					variable_region->delta[i], encode_param->quantumNumber);
		if(out_array[i] < 0){
			goto out;
		}
	}

	rv = EXIT_NORMAL;

out:
#ifdef DBG
	DBGW(10, ("Double2Int exit(%d).\n", rv));
#endif /* DBG */

	return rv;
}

static int binary_search(double in_value, double min, double delta, int qn)
{
	int low	= 0;
	int high;
	int mid;
	int rv = EXIT_ERROR;
	double candidate;
	
#ifdef DBG
	DBGW(10, ("binary_search enter(0).\n"));
#endif /* DBG */

	high = (int)pow(2, qn) - 1;

	while(low <= high){
		mid = (low + high) / 2;
		candidate = min + delta * mid;
		if (fabs(candidate - in_value) <= delta / 2) {
			rv = mid;
			goto out; /* Normal Exit */
		} else if (candidate > in_value){
			high = mid - 1;
		} else {
			low = mid + 1;
		}
	}

out:
	if(rv == EXIT_ERROR){
		log_sys(" ## Error: In binary_search\n");
	}
	
#ifdef DBG
	DBGW(10, ("binary_search exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}
