#include "life.h"

static int initTimeSolver(Parameter param_start, double y_start[], double y[], double y_pre[]);
static int odefunc(double t, double *y, double *f);
static int jacfunc(double t, double *y, double *dfdy0);
static void _copy_origin1to0(double x1[][MAX_VAR_NUM], double x0[], int n);
static int _get_simultaneous_events(double t_st, double t_ed, Event *evts[]);
static void _get_adding_periods(int nevts, Event *evts[], int is_adding[]);
static int _do_events(Event evts[], int len, double y[], double y_add[]);
static void _dump_param();
static int timeEvent(double y[], double y_add[], double t, double h, int f_reset);
static int doSolver1_2(Parameter param_start, double h, double t_start, double t_end,
			   			double dt_save, double y_start[], double y[], double y_pre[]);
static int doSolver3(Parameter param_start, double h, double t_start, double t_end,
			  		 	double dt_save, double y_start[], double y[], double y_pre[]);
static int doSolver11(double y[], double y_pre[]);
static int doSolver12(double y[]);
static void write_start(double y[], double t_start);


/*****************************************************************************
 * ؿ̾
 *	initTimeSolver: ȯŸС
 * ǽ
 *	ȯŸС
 *
 *	
 *	1: 
 * 
 *      Parameter param_start;	I: ѥ᡼¤
 *      double y_start[];	I: ѿͤؤΥݥ
 *      double y[];		O: ѿؤΥݥ
 *      double y_pre[];		O: ѿͤؤΥݥ
 * ؿ
 *	 0:	
 *	-1:	۾
 * 
 *****************************************************************************/
static int initTimeSolver(Parameter param_start, double y_start[], double y[], double y_pre[])
{
	int	i;

	 /* ѥ᡼٥Ѥνͺ */
	for (i=1;i<=Param.n_var;i++) y[i]=y_start[i];
	Param = param_start;

	/*
	 * 2ʬΥξ硢
	 * ѥ᡼򥹥ͤͤ
	 * Ū˶ŤϢΩ
	 */
	if (Param.n_varNR) {
		if (SetNRparam(y)) {
			if(!F_GA) fprintf(Fp_out, "\n$$ Failure !!\n");
			goto err;
		}

#ifdef DBG
		DBGW_L(20, ("after SetNRparam\n"));
		for(i=1; i<=Param.n_var; i++)
			DBGW(20,("y[%2d]=%e\n", i, y[i]));
#endif
	}
	
	ParamP = Param = ParamT;  /* 1ƥåͤ */
	for (i=1;i<=Param.n_var;i++)
		y_pre[i]=y[i];  /* 1ƥåͤ */

	return 0;

err:
	return -1;
}

/*****************************************************************************
 * ؿ̾
 *	DoSolver: doSolver* θƤӽФ
 * ǽ
 *	SolverNo ˱ doSolver* ƤӽФ
 *
 *	
 *	1: 
 * 
 *      Parameter param_start;	I: ѥ᡼¤
 *      double *h;		I: ƥåؤΥݥ
 *      double t_start;		I: ϻ
 *      double t_end;		I: λ
 *      double dt_save;		I: ϻ
 *      double y_start[];	I: ѿͤؤΥݥ
 *      double y[];		O: ѿؤΥݥ
 *      double y_pre[];		O: ѿͤؤΥݥ
 * ؿ
 *	 0:	
 *	-1:	۾
 * 
 *****************************************************************************/
int DoSolver(Parameter param_start, double h, double t_start, double t_end, double dt_save, double y_start[], double y[], double y_pre[])
{
	int	ret = 0;

	switch(SolverNo){
	case	1:
	case	2:
		ret = doSolver1_2(param_start, h, t_start, t_end, dt_save, y_start, y, y_pre);
		break;
	case	3:
		ret = doSolver3(param_start, h, t_start, t_end, dt_save, y_start, y, y_pre);
		break;
	case	11:
		ret = doSolver11(y, y_pre);
		break;
	case	12:
		ret = doSolver12(y);
		break;
	default	:
		log_sys("## Wrong Solver!! No =%3d\n", SolverNo);
		exit(-1);
	}

	return ret;
}

/*****************************************************************************
 * ؿ̾
 *	doSolver1_2: Runge-KuttaλȯŸСμ¹ԡ
 * ǽ
 *	Runge-KuttaλȯŸС¹Ԥ롣
 *
 *	
 *	1: 
 * 
 *      Parameter param_start;	I: ѥ᡼¤
 *      double h;		I: ƥå
 *      double t_start;		I: ϻ
 *      double t_end;		I: λ
 *      double dt_save;		I: ϻ
 *      double y_start[];	I: ѿͤؤΥݥ
 *      double y[];		O: ѿؤΥݥ
 *      double y_pre[];		O: ѿͤؤΥݥ
 * ؿ
 *	 0:	
 *	-1:	۾
 * 
 *****************************************************************************/
static int doSolver1_2(Parameter param_start, double h, double t_start, double t_end,
			   	double dt_save, double y_start[], double y[], double y_pre[])
{
	int	ret = 0;
	int	i, i_save = 0, f_event, f_eventReset;
	double	t, t_save;
	double y_add[MAX_VAR_NUM];

	memset(y_add, 0, sizeof(y_add));

	 /* Runge-KuttaȯŸС */
	ret = initTimeSolver(param_start, y_start, y, y_pre);
	if( ret != 0 ){
		goto err;
	}

	write_start(y, t_start);

	t=t_start;
	t_save = dt_save;

	while (t < t_end) {	 /* ȯŸСΥॹƥåץ롼 */

		 /* ।٥Ȥ */
		if (t == t_start) {
			f_eventReset = true;
		} else {
			f_eventReset = false;
		}
		f_event = timeEvent(y, y_add, t, h, f_eventReset);

		if (SolverNo == 1) {
		 	/* Runge-KuttaȯŸСλ֥ƥåפμ¹ */
			if(SolveDiffEqNormalRungeKutta(
				Param.n_var, y, y_pre, &t, &h, f_event)) {
				log_sys("## Error at Time=%10.3E in NormalRungeKutta\n", t);
				if(!F_GA) fprintf(Fp_out, "\n$$ Failure !!\n");
				goto err;
			}
		} else if (SolverNo == 2) {
		 	/* ŬRunge-KuttaȯŸСλ֥ƥåפμ¹ */
			if (SolveDiffEqAdaptiveRungeKutta(
				Param.n_var, y, y_pre, &t, &h, t_end, f_event)) {
				log_sys("## Error at Time=%10.3E in AdaptiveRungeKutta\n", t);
				if(!F_GA) fprintf(Fp_out, "\n$$ Failure !!\n");
				goto err;
			}
		} else {
			log_sys("## Wrong Solver No.\n");
			goto err;
		}

		 /* Y_ADDν */
		for (i=1+Param.n_varNR; i<=Param.n_var; i++)
			y[i] += y_add[i] * (h);

		 /* ˤʤäΥ˥ */
		for(i=1;i<=Param.n_var;i++) { 
			if(y[i] < 0.0) {
				log_sys("Negative Value!! t=%10.3E:y[%3d]=%10.3E\n", t, i, y[i]);
			}
		}

		 /* ˥󥰥󥿡Х˱ */
		if((t + h/2.0) > t_save) {
			i_save++;
			if(F_GA){
				SolutionMatrix[StepNum][0] = t;
				for(i = 1; i<= Param.n_var; i++){
					SolutionMatrix[StepNum][i] = y[i]/GeneConc;
				}
				StepNum++;
			} else {
				fprintf(Fp_out, "%10.3E", t);
				for(i=1;i<=Param.n_var;i++) fprintf(Fp_out, ", %10.3E", y[i]/GeneConc);
				fprintf(Fp_out, "\n");
			}
			t_save = (i_save + 1)*dt_save;
			t_save = t_save > t_end ? t_end : t_save;
			/*
			 *  Ŭξ硢h  dt_save 礭ʤ
			 * t_save  t ٤ƤޤΤ
			 *  h δֳ֤ǽϤ褦ˤʤ롣
			 *  Τ褦ʻ TimeEvent  h ޤ˾ʤ
			 * t_save  t ˤĤޤǤ dt_save 
			 *  h δֳ֤Ϥ롣
			 */
		}
	}	 /* end of Time step loop. */

	return 0;

err:
	log_sys("## Error!! in doSolver1_2\n");

	return -1;
}

/****************************************************************************
 *
 */
static void
write_start(double y[], double t_start)
{
	int i;

	if(!F_GA){
	 	/* ϥե˥٥񤭹 */
		fprintf(Fp_out, "%-10s", "Time");
		for (i=1;i<=Param.n_var;i++)
			fprintf(Fp_out,",     y[%3d]", i);
		fprintf(Fp_out,"\n");
	 	/* ϥե˥Ȼͤ񤭹 */
		fprintf(Fp_out, "%10.3E", t_start);
		for(i=1;i<=Param.n_var;i++)
			fprintf(Fp_out, ", %10.3E", y[i]/GeneConc);
		fprintf(Fp_out, "\n");
	}
}

/*****************************************************************************
 * ؿ̾
 *	doSolver3: NDFȯŸСμ¹ԡ
 * ǽ
 *	NDFȯŸС¹Ԥ롣
 *
 *	
 *	1: 
 * 
 *      Parameter param_start;	I: ѥ᡼¤
 *      double h;		I: ƥå
 *      double t_start;		I: ϻ
 *      double t_end;		I: λ
 *      double dt_save;		I: ϻ
 *      double y_start[];	I: ѿͤؤΥݥ
 *      double y[];		O: ѿؤΥݥ
 *      double y_pre[];		O: ѿͤؤΥݥ
 * ؿ
 *	 0:	
 *	-1:	۾
 * 
 *****************************************************************************/
static int doSolver3(Parameter param_start, double h, double t_start, double t_end, double dt_save, double y_start[], double y[], double y_pre[])
{
	int		ret;

	int		nevts, ie;
	Event		*evts[MAX_EVENT_NUM];	/* Ʊ磻٥ȤƬ */
	int		is_yadding[MAX_EVENT_NUM];

	double		div, mod;
	int		idiv;

	ndf_option_t	option;
	ndf_input_t	in;
	ndf_solution_t	sol;
	ndf_status_t	stats;
	double		*mass = NULL;
	double		*t0[MAX_EVENT_NUM];
	int		t0num[MAX_EVENT_NUM];
	int		size;
	int		i, it, n_var;
	int		it0, it0max;
	double		tdel;

	int	hasParamEvents = 0;
	static double y_add[MAX_VAR_NUM];

	memset(&option, 0, sizeof(ndf_option_t));
	memset(&in, 0, sizeof(ndf_input_t));
	memset(&sol, 0, sizeof(ndf_solution_t));
	memset(&stats, 0, sizeof(ndf_status_t));
	memset(t0, 0, MAX_EVENT_NUM * sizeof(double *));
	memset(t0num, 0, MAX_EVENT_NUM * sizeof(int));


	/* NDFȯŸС */
	ret = initTimeSolver(param_start, y_start, y, y_pre);
	if( ret != 0 ){
		goto err;
	}

	DBGW(0, ("rtol=%10.3E, atol=%10.3E\n", Ndf_rtol, Ndf_atol));

	/*
	 * ٥ȤλĴ٤롣
	 */
	memset(evts, 0, MAX_EVENT_NUM * sizeof(Event *));
	nevts = _get_simultaneous_events(t_start, t_end, evts);
DBGW(50, ("Events Num[%d], Events Time Num[%d]\n", EventsNum, nevts));

	/*
	 * Y_ADD Τ֤Ĵ٤롣
	 */
	_get_adding_periods(nevts, evts, is_yadding);

	/*
	 * ٥ȤȼΥ٥Ȼδ֤Ȥˡ
	 * νϻꤹ롣
	 */
	for( ie = 0 ; ie < nevts - 1; ie++ ){
		/*
		 * ϻꡣ
		 * ϴֳֻdt_save=0ˤ̵ϡ
		 * ׻ݥȤϡ<= t0num[ie]=2 Ȼꤹ롣
		 */
		t0num[ie] = 2;
		if( dt_save > 0 ){
			div = (evts[ie + 1]->time - evts[ie]->time) / dt_save;
			idiv = (int) div;
			mod = div - idiv;
			/*
			 * ˵10ˤϽ
			 */
			if( idiv != 0 && mod < 0.1 ){
				idiv--;
			}
			t0num[ie] += idiv;
		}

		/*
		 * ϻꡣ
		 */
		t0[ie] = (double *) malloc(t0num[ie] * sizeof(double));
		if( t0[ie] == NULL ){
			goto err;
		}

		t0[ie][0] = evts[ie]->time;
		for( i = 1 ; i < t0num[ie] ; i++ ){
			t0[ie][i] = t0[ie][i - 1] + dt_save;
		}
		t0[ie][t0num[ie] - 1] = evts[ie + 1]->time;

#ifdef DBG
		DBGW(50, ("t0num[%d] = %d\n", ie, t0num[ie]));
		for( i = 0 ; i < t0num[ie] ; i++ ){
			DBGW(50, ("t0[%d] = %e\n", i, t0[ie][i]));
		}
#endif
	}

	/*
	 * ̹롣
	 */
	n_var = Param.n_var;
	size = n_var * n_var * sizeof(double);
	mass = (double *) malloc(size);
	if( mass == NULL ){
		log_sys("malloc(mass) Failed!!!\n");
		goto err;
	}
	memset(mass, 0, size);
	for( i = Param.n_varNR ; i < n_var ; i++ ){
		mass[i * n_var + i] = 1.0;
	}

	/*
	 * NDFСΥץꡣ
	 */
	ndf_init(&option);

	option.dae.mconst = mass;
	option.jac.jfunc = (F_Numerical_Jac > 0)? NULL : jacfunc;
	option.step.initstep = h;
	option.tol.rel = Ndf_rtol;
	option.tol.abs = Ndf_atol;
	option.tol.positive = ON;

	/*
	 * NDFСνꡣ
	 */
	in.y0 = y + 1;
	in.y0num = n_var;

	/*
	 * ϥե˥٥񤭹
	 */
	if(!F_GA){
		fprintf(Fp_out, "%-10s", "Time");
		for (i=1;i<=Param.n_var;i++)
			fprintf(Fp_out,",     y[%3d]", i);
		fprintf(Fp_out,"\n");
	}
		
	/*
	 * ٥Ȥȥ٥Ȥδ֤ǥС¹ԡ
	 */
	for( ie = 0 ; ie < nevts - 1 ; ie++ ){
#ifdef DBG
		_dump_param();
		DBGW(30, ("Init Y:\n"));
		for( i = 0 ; i < n_var ; i++ ){
			DBGW(30, ("[%2d] %e(%e)\n", i, in.y0[i], y_start[i+1]));
		}
#endif
		/*
		 * ٥ȼ¹ԡ
		 */
		hasParamEvents =
			_do_events(evts[ie], evts[ie + 1] - evts[ie], y, y_add);
#if 1
		if (Param.n_varNR) {	/* 2ʬΥξ */
			/*
			 * ѥ᡼ѹƤ
			 * ŪͤѤϢΩľ
			 */
			if( hasParamEvents ){
				if( CheckNRparam(y_pre) ){
					goto err;
				}
				/* βȿǤ */
				for( i = 1 ; i <= Param.n_varNR ; i++ ){
					y[i] = y_pre[i];
				}
			}
			/*
			 * ϢΩ굡Ū˲򤭡
			 * 򤱤ʤˤäƲľ
			 */
			
			if( SolveAlgebraicEq(y, y_pre) ){
				goto err;
			}
		}
#endif

		/*
		 * Y_ADD 硢ϴֳ֤ǥС¹ԡ
		 *         ʤ硢٥ȴֳ֤ǥС¹ԡ
		 */
		it0max = (is_yadding[ie])? t0num[ie] : 2;
		for( it0 = 0 ; it0 < it0max - 1 ; it0++ ){
			/*
			 * NDFСλꡣ
			 */
			in.t0 = &t0[ie][it0];
			in.t0num = (is_yadding[ie])? 2 : t0num[ie];

			/*
			 * NDFСμ¹ԡ
			 */
			if( ndf_solve(&in, &sol, odefunc, &option, &stats) < 0 ){
				log_sys("NDF ndf_solve() Failed!!!\n");
				goto err;
			}


			/*
			 * ̽ϡ
			 * Y_ADD 硢Τ߽ϡ
			 */
			it = (is_yadding[ie])? sol.tnum - 1 : 0;
			for( ; it < sol.tnum ; it++ ){
				if(F_GA){
					SolutionMatrix[StepNum][0] = sol.tt[it];
				} else {
					fprintf(Fp_out, "%10.3E", sol.tt[it]);
				}
				for( i = 0 ; i < n_var ; i++ ){
					/*
					 * Y_ADD ¹ԡ
					 */
					if( is_yadding[ie]
					 && i >= Param.n_varNR ){
					 	tdel = sol.tt[it] - sol.tt[0];
						sol.yy[it * n_var + i]
							+= y_add[i + 1] * tdel;
					}
					/*
					 * ˤʤä˥󥰡
					 */
					if(sol.yy[it * n_var + i] < 0.0) {
						log_sys("Negative Value!! t=%10.3E:y[%3d]=%10.3E\n", sol.tt[it], i + 1, sol.yy[it * n_var + i]);
					}
					/*  */
					if(F_GA){
						SolutionMatrix[StepNum][i + 1] = sol.yy[it * n_var + i];
					} else {
						fprintf(Fp_out, ", %10.3E",
							sol.yy[it * n_var + i]
							/ GeneConc);
					}
					/*
					 * ͤꡣʺν
					 */
					if( it == sol.tnum - 1 ){
						in.y0[i] =
							sol.yy[it * n_var + i];
						y_pre[i + 1] =
							sol.yy[it * n_var + i];
					}
				}
				if(F_GA){
					StepNum++;
				} else {
					fprintf(Fp_out, "\n");
				}
			}


			/*
			 * NDFСθ
			 */
			ndf_term(&sol);
		}

	}

	free(mass);
	for( ie = 0 ; ie < nevts - 1 ; ie++ ){
		free(t0[ie]);
	}

	return 0;

err:
	/*
	 * NDFСθ
	 */
	ndf_term(&sol);
	if(mass)	free(mass);
	for( ie = 0 ; ie < nevts - 1 ; ie++ ){
		if(t0[ie])	free(t0[ie]);
	}
	log_sys("## Error!! in doSolver3\n");

	return -1;
}
/*****************************************************************************
 * ؿ̾
 *	odefunc: դͤ롣
 * ǽ
 *	ϢΩʬαդͤ롣
 *
 *	
 *	1: 
 * 
 *      double t;	I: 
 *      double y[];	I: ѿؤΥݥ
 *      double f[];	O: ϢΩʬͤؤΥݥ
 * ؿ
 *	 0:	
 * 
 *	ndf_solve() ؿƤӽФ륳Хåؿ
 *****************************************************************************/
static int odefunc(double t, double *y, double *f)
{
	/* double	dfdy[MAX_VAR_NUM][MAX_VAR_NUM]; */

	/* usr_algebraicEq(y - 1, f - 1, dfdy, GeneConc, &Param); */
	usr_fvec(f - 1, y - 1, GeneConc, &Param);

	return 0;
}
/*****************************************************************************
 * ؿ̾
 *	janfunc: 䥳ӥ롣
 * ǽ
 *	ϢΩʬΥ䥳ӥ롣
 *
 *	
 *	1: 
 * 
 *      double t;	I: 
 *      double y[];	I: ѿؤΥݥ
 *      double dfdy0[];	O: 䥳ӥؤΥݥ
 * ؿ
 *	 0:	
 * 
 *	ndf_solve() ؿƤӽФ륳Хåؿ
 *****************************************************************************/
static int jacfunc(double t, double *y, double *dfdy0)
{
	double	dfdy[MAX_VAR_NUM][MAX_VAR_NUM];
	/* double	f[MAX_VAR_NUM]; */
	int	i, j;

	for( i = 0 ; i < Param.n_var ; i++ ){
		for( j = 0 ; j < Param.n_var ; j++ ){
			dfdy[i + 1][j + 1] = 0.;
		}
	}
	/* usr_algebraicEq(y - 1, f - 1, dfdy, GeneConc, &Param); */
	usr_fjac(dfdy, y - 1, GeneConc, &Param);

	_copy_origin1to0(dfdy, dfdy0, Param.n_var);

	return 0;
}
/*
 * 1ꥸ2󤫤0ꥸإԡ롣
 */
static void _copy_origin1to0(double x1[][MAX_VAR_NUM], double x0[], int n)
{
	int	i, j;

	for( i = 0 ; i < n ; i++ ){
		for( j = 0 ; j < n ; j++ ){
			x0[i * n + j] = x1[i + 1][j + 1];
		}
	}
}
/*
 * ٥Ȼ老Ȥʬ
 * Ʊȯ륤٥ȤEvent¤ΤƬɥ쥹롣
 *	t_st                                          t_ed
 *	+-------------+-------------+-----------------+
 *	e0           e1,e2         e3,e4,e5           e6
 *	evts[0] = e0 evts[1] = e1  evts[2] = e3       evts[3] = e6
 */
static int _get_simultaneous_events(double t_st, double t_ed, Event *evts[])
{
	int	i, nevts = 0;
	double	prev_time = t_st;

	/*
	 * ٥Ȥϡ¤Ǥ
	 * time_start ٥ȤϤޤäơtime_end ٥Ȥǽ롣
	 */
	evts[nevts++] = Events;
	for( i = 0 ; i < EventsNum ; i++ ){
		if( Events[i].time <= prev_time )	continue;
		if( Events[i].time > t_ed )		break;
		prev_time = Events[i].time;
		evts[nevts++] = Events + i;
	}

	return nevts;
}
/*
 * ٥Ȥȥ٥Ȥδ֤ˡY_ADD 뤫Ĵ٤롣
 */
static void _get_adding_periods(int nevts, Event *evts[], int is_adding[])
{
	int	i, ie, len;
	int	adding_cnt = 0;

	for( ie = 0 ; ie < nevts - 1 ; ie++ ){
		is_adding[ie] = 0;
		len = evts[ie + 1] - evts[ie];
		for( i = 0 ; i < len ; i++ ){ /* Ʊ磻٥ȥ롼 */
			if( !isPno(Y_ADD, evts[ie][i].p_no) )	continue;

			if( evts[ie][i].value == 0 ){
				adding_cnt--;
			}else{
				adding_cnt++;
			}
		}
		if( adding_cnt > 0 ){
			is_adding[ie] = 1;
		}
	}
}
/*
 * Ʊ磻٥ȤƤ¹Ԥ롣
 */
static int _do_events(Event evts[], int len, double y[], double y_add[])
{
	int	ie;
	int	hasParamEvents = 0;
DBGW(30, ("## Do Events: time[%e]\n", evts[0].time));

	for( ie = 0 ; ie < len ; ie++ ){
		if( isPno(Y_FORCE, evts[ie].p_no) ){
DBGW(30, ("## y_force: [%e] => [%e]\n", y[evts[ie].index], evts[ie].value));
			y[evts[ie].index] = evts[ie].value;
		}else if( isPno(Y_ADD, evts[ie].p_no) ){
DBGW(30, ("## y_add: [%e] => [%e]\n", y_add[evts[ie].index], evts[ie].value));
			y_add[evts[ie].index] = evts[ie].value;
		}else if( isPno(TIME_START, evts[ie].p_no) ){
DBGW(30, ("## : Simulation Starts\n"));
			/* do nothing */
		}else if( isPno(TIME_END, evts[ie].p_no) ){
DBGW(30, ("## : Simulation End\n"));
			/* do nothing */
		}else{
DBGW(30, ("## %s: [%e] => [%e]\n", Pno2Name(evts[ie].p_no),
		Param.val[evts[ie].p_no][evts[ie].index], evts[ie].value));
			Param.val[evts[ie].p_no][evts[ie].index]
							= evts[ie].value;
			hasParamEvents = 1;
		}
	}

	return hasParamEvents;
}
/*
 * ǥХåѤ˥ѥ᡼פ롣
 */
static void _dump_param()
{
	int	i, j;

	DBGW(30, ("Gene: %e\n", GeneConc));
	for( i = 1 ; i <= getParamNum() ; i++ ){
		DBGW(30, ("%s: ", Pno2Name(i)));
		for( j = 1 ; j <= Param.n_val[i] ; j++ ){
			DBGW(30, (" %e", Param.val[i][j]));
		}
		DBGW(30, ("\n"));
	}
}

/*****************************************************************************
 * ؿ̾
 *	doSolver11: Newton-Raphsonˤ򥽥С¹Ԥ롣
 * ǽ
 *	Newton-Raphsonˤ򥽥С¹Ԥ롣
 *
 *	
 *	1: 
 * 
 *      double y[];		O: ѿؤΥݥ
 *      double y_pre[];		O: ѿͤؤΥݥ
 * ؿ
 *	 0:	
 *	 1:	۾ʼΥ١ؿʤ
 *	-1:	۾
 * 
 *****************************************************************************/
static int doSolver11(double y[], double y_pre[])
{
	int	i;

	 /* Newton-Raphsonˤ򥽥С¹Ԥ */
	if (SolveSTDbyNR(y, y_pre)) {
		if(!F_GA) fprintf(Fp_out, "\n$$ Failure!!\n");
		goto err;
	} else {
		fprintf(Fp_out, "index,     y value\n");
		for (i=1; i<=Param.n_var; i++)
			if(F_GA){
				SolutionMatrix[StepNum][i] = y[i]/GeneConc;
			} else {
				fprintf(Fp_out, "%5d,%12.4e,  #%s\n", i, y[i]/GeneConc, getVarTag(i));
			}
	}
	if(F_GA){
		SolutionMatrix[StepNum][0] = 0.0;
		StepNum++;
	}
	 /* ٲϤϥѥ᡼٥ޤGAλˤϹԤʤ */
	if (!F_PSurvey && !F_GA) {
		if (Sensitivity(y, y_pre)) {
			goto err;
		}
	}

	return 0;

err:
	return -1;
}

/*****************************************************************************
 * ؿ̾
 *	doSolver12: S-systemˤ򥽥С¹Ԥ롣
 * ǽ
 *	S-system򥽥С¹Ԥ롣
 *
 *	
 *	1: 
 * 
 *      double y[];		O: ѿؤΥݥ
 * ؿ
 *	 0:	
 *	 1:	۾ʼΥ١ؿʤ
 *	-1:	۾
 * 
 *****************************************************************************/
static int doSolver12(double y[])
{
	int	i;

	 /* S-systemˤ򥽥С¹Ԥ */
	if (SsysSTD(y)) {
		if (!F_GA) fprintf(Fp_out, "\n$$ Failure!!\n");
		goto err;
	} else {
		fprintf(Fp_out, "index,     y value\n");
		if (F_GA) {
			for (i=1; i<=Param.n_var; i++) SolutionMatrix[StepNum][i] = y[i]/GeneConc;
		}
	}
	if(F_GA){
		SolutionMatrix[StepNum][0] = 0.0;
		StepNum++;
	}

	return 0;

err:
	return -1;
}

/*----------------timeEvent------------------------------------------
 *
 * Events¤Τͤ򸵤ˡꤵ줿äѥ᡼ͤ
 * åȤ롣doSolver1,2 ǻѡ
 *
 *	I:t, h (ХEvents¤)
 *	O:y, y_add (ХParam¤)
 *	꥿͡flag(ͤ򥻥åȤ1⤷ʤä0)
 *
 */
static int timeEvent(double y[], double y_add[], double t, double h, int f_reset)
{
	static int p=0;
	int flag=0;

	if (f_reset) p = 0;
	if (p >= EventsNum) return flag;
	while( (t + h/2.0) > Events[p].time) {
		flag = 1;
		if( isPno(Y_FORCE, Events[p].p_no) ){
			y[Events[p].index] = Events[p].value;
		}else if( isPno(Y_ADD, Events[p].p_no) ){
			y_add[Events[p].index] = Events[p].value;
		}else if( isPno(TIME_START, Events[p].p_no) ){
			/* do nothing */
		}else if( isPno(TIME_END, Events[p].p_no) ){
			/* do nothing */
		}else{
			Param.val[Events[p].p_no][Events[p].index]
							= Events[p].value;
		}
		p++;
		if (p >= EventsNum) return flag;
	}
	return flag;
}
