#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<float.h>

#include	"logUtil.h"

#include	"matrix.h"
#include	"condest.h"
#include	"lu.h"


#ifndef	DBL_MAX
#define DBL_MAX         1.7976931348623157E+308
#endif

#define	MTX_INFINITE	DBL_MAX

#define	DIM		0
#define	REAL		1
#define	NOTRANSPOSE	2
#define	TRANSPOSE	3

#define	ITER_MAX	5

static int _condest_f(int flag, m_matrix_t *v_x, m_matrix_t *m_lu, int *idx, m_matrix_t *v_f);
static int mtx_est_1norm(int (*estfunc)(int flag, m_matrix_t *v_x, m_matrix_t *m_lu, int *idx, m_matrix_t *v_f), int t0, m_matrix_t *m_lu, int *idx, double *est_norm);
static void _reverse(int len, int *isort);
#if 0
static int _any(m_matrix_t *mtx, int relation, double val);
#endif
static int _vector_cat(m_matrix_t *trg, m_matrix_t *cat, int n);
static int _undupli(m_matrix_t *m_s, m_matrix_t *m_sold);
static int _mysign(m_matrix_t *mtx, m_matrix_t *sgn);
static int _normapp(int (*estfunc)(int flag, m_matrix_t *v_x, m_matrix_t *m_lu, int *idx, m_matrix_t *v_f), int flag, m_matrix_t *v_x, m_matrix_t *m_lu, int *idx, m_matrix_t *v_f);
static int mtx_sort_vector(m_matrix_t *src, m_matrix_t *trg, int *indx);
static int _vec_sort(const void *ss1, const void *ss2);
static int mtx_rand(m_matrix_t *mtx);




/*****************************************************************************
 * ؿ̾
 *      mtx_est_condition_number: ξοꡣ
 * ǽ
 *      ξꤹ롣
 *
 *      
 * 
 *      m_matrix_t *mtx;   I: ¤ΤؤΥݥ
 * ؿ
 *      >= 0 :	ʾ
 *      <  0 :	۾
 * 
 *****************************************************************************/
double mtx_est_condition_number(m_matrix_t *mtx)
{
	double	condnum = 0;	/* estimated condition number	*/
	int	t = 2;		/* default columns number	*/
	m_matrix_t	m_lu, v_diag;
	int	*indx = NULL;
	m_index_t	*i_diag = NULL;
	int	len;
	double	a_norm, ainv_norm;
	int	ret;

	memset(&m_lu, 0, sizeof(m_matrix_t));
	memset(&v_diag, 0, sizeof(m_matrix_t));

	if( mtx == NULL ){
		goto err;
	}
	if( mtx->row != mtx->col ){
		goto err;
	}
	if( mtx->elem == NULL ){
		goto err;
	}

	MTX_SET_ALLOC(m_lu, mtx->row, mtx->col);
	MTX_SET_ALLOC(v_diag, mtx->row, 1);
	if( m_lu.elem == NULL || v_diag.elem == NULL ){
		goto err;
	}
	indx = (int *) malloc(mtx->row * sizeof(int));
	if( indx == NULL ){
		goto err;
	}

	/* LUʬ */
	ret = mtx_ludecomp(mtx, &m_lu, indx);
	if( ret == -1 ){
		goto err;
	}
	if( ret == -2 ){	/* Singular Matrix */
		condnum = MTX_INFINITE;
		goto end;
	}
	mtx_diagonal2vector(0, &m_lu, &v_diag);
	mtx_abs(&v_diag, &v_diag);
	len = mtx_index(&v_diag, MTX_EQ, 0, &i_diag);
	if( len < 0 ){
		goto err;
	}

	if( len > 0 ){
		condnum = MTX_INFINITE;
	}else{
		if( mtx_est_1norm(_condest_f, t, &m_lu, indx, &ainv_norm) < 0 ){
			goto err;
		}
		if( mtx_norm(mtx, 1, &a_norm) < 0 ){
			goto err;
		}
		condnum = ainv_norm * a_norm;
	}

end:
	MTX_SET_FREE(m_lu);
	MTX_SET_FREE(v_diag);
	free(indx);
	mtx_index_free(i_diag);
	return condnum;
err:
	MTX_SET_FREE(m_lu);
	MTX_SET_FREE(v_diag);
	if(indx)	free(indx);
	if(i_diag)	mtx_index_free(i_diag);
	log_sys("ERROR: mtx_est_condition_number()\n");
	return -1;
}

/*
 * mtx_est_1norm() ǻѤ륳Хåؿ
 */
static int _condest_f(int flag, m_matrix_t *v_x, m_matrix_t *m_lu, int *idx, m_matrix_t *v_f)
{
	int	f = 0;

	switch( flag ){
	case	DIM:
		f = (m_lu->row > m_lu->col)? m_lu->row : m_lu->col;
		break;
	case	REAL:
		f = 1;
		break;
	case	NOTRANSPOSE:
		if( mtx_lusolve(m_lu, idx, v_x, v_f) < 0 ){
			goto err;
		}
		break;
	case	TRANSPOSE:
		if( mtx_tlusolve(m_lu, idx, v_x, v_f) < 0 ){
			goto err;
		}
		break;
	default	:
		goto err;
	}

	return f;
err:
	log_sys("ERROR: _condest_f()\n");
	return -1;
}


/*
 * ݥΥꤹ롣
 */
static int mtx_est_1norm(int (*estfunc)(int flag, m_matrix_t *v_x, m_matrix_t *m_lu, int *idx, m_matrix_t *v_f), int t0, m_matrix_t *m_lu, int *idx, double *est_norm)
{
	m_matrix_t	v_tmp, m_tmp, s_tmp;
	double		scalar[1];

	int	n, a_is_real, max_t, t;
	int	rpt_s, rpt_e;

	m_matrix_t	m_x, m_y, m_z, v_w;
	m_matrix_t	m_s, m_sold, m_sold_tmp, m_ss;
	m_matrix_t	v_ind, v_ind_hist, v_vals, v_vals_ind;
	m_matrix_t	v_zvals, v_zvals_tmp;
	m_matrix_t	m_xtmp, v_vtmp;

	int	it, nmv;
	double	est_old, tmp;

	int	size, i, j, index;
	int	est_j, info, np, imax, rep, r;
	int	ret;
	int	*isort = NULL;

	memset(&v_tmp, 0, sizeof(m_matrix_t));
	memset(&m_tmp, 0, sizeof(m_matrix_t));
	memset(&m_x, 0, sizeof(m_matrix_t));
	memset(&m_y, 0, sizeof(m_matrix_t));
	memset(&m_z, 0, sizeof(m_matrix_t));
	memset(&v_w, 0, sizeof(m_matrix_t));
	memset(&m_s, 0, sizeof(m_matrix_t));
	memset(&m_sold, 0, sizeof(m_matrix_t));
	memset(&m_sold_tmp, 0, sizeof(m_matrix_t));
	memset(&m_ss, 0, sizeof(m_matrix_t));
	memset(&v_ind, 0, sizeof(m_matrix_t));
	memset(&v_ind_hist, 0, sizeof(m_matrix_t));
	memset(&v_vals, 0, sizeof(m_matrix_t));
	memset(&v_vals_ind, 0, sizeof(m_matrix_t));
	memset(&v_zvals, 0, sizeof(m_matrix_t));
	memset(&v_zvals_tmp, 0, sizeof(m_matrix_t));
	memset(&m_xtmp, 0, sizeof(m_matrix_t));
	memset(&v_vtmp, 0, sizeof(m_matrix_t));

	n = _normapp(estfunc, DIM, NULL, m_lu, idx, NULL);
	a_is_real = _normapp(estfunc, REAL, NULL, m_lu, idx, NULL);
	max_t = (n > 2)? n : 2;

	t = ( t0 < 0 )? -t0 : t0;
	if( t == 0 ){
		t = 2;
	}
	/* t must be an integer between 1 and N=max(size(a)) */
	if( t > max_t ){
		goto err;
	}

	rpt_s = rpt_e = 0;

	MTX_SET_ALLOC(v_tmp, m_lu->row, 1);
	MTX_SET_ALLOC(m_tmp, m_lu->row, m_lu->row);
	if( v_tmp.elem == NULL || m_tmp.elem == NULL ){
		goto err;
	}
	isort = (int *) malloc(n * sizeof(int));
	if( isort == NULL ){
		goto err;
	}

	if( t == n || n <= 4 ){
		/* X = eye(n) */
		mtx_unit(&m_tmp);
		/* Y = normapp(A, 'notransp', X, varargin{:}) */
		if( _normapp(estfunc, NOTRANSPOSE, &m_tmp, m_lu, idx, &m_tmp) < 0 ){
			goto err;
		}
		/* [temp, m] = sort( sum(abs(Y)) ); */
		mtx_abs(&m_tmp, &m_tmp);
		mtx_sum(MTX_COL, &m_tmp, &v_tmp);
		mtx_sort_vector(&v_tmp, &v_tmp, isort);
		/* est = temp(n); */
		*est_norm = v_tmp.elem[n - 1];
		/* ȤꤢפʤΤǾά
		 * v = zeros(n,1); v(m(n)) = 1; w = Y(:,m(n));
		 * iter = [0 1]; red = [0 0]; info = 6;
		 */
		goto end;
	}

	MTX_SET_ALLOC(m_x, n, t);
	MTX_SET_ALLOC(m_y, n, t);
	MTX_SET_ALLOC(m_z, n, t);
	if( m_x.elem == NULL || m_y.elem == NULL || m_z.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(m_s, n, t);
	MTX_SET_ALLOC(m_sold, n, t);
	MTX_SET_ALLOC(m_sold_tmp, t, n);
	MTX_SET_ALLOC(m_ss, t, t);
	if( m_s.elem == NULL || m_sold.elem == NULL
	 || m_sold_tmp.elem == NULL || m_ss.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_ind, t, 1);
	MTX_SET_ALLOC(v_ind_hist, t, 1);
	MTX_SET_ALLOC(v_vals, t, 1);
	MTX_SET_ALLOC(v_vals_ind, t, 1);
	MTX_SET_ALLOC(v_w, n, 1);
	MTX_SET_ALLOC(v_zvals, n, 1);
	MTX_SET_ALLOC(v_zvals_tmp, n, 1);
	if( v_ind.elem == NULL || v_ind_hist.elem == NULL
	 || v_vals.elem == NULL || v_vals_ind.elem == NULL
	 || v_w.elem == NULL
	 || v_zvals.elem == NULL || v_zvals_tmp.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(m_xtmp, n, t);
	MTX_SET_ALLOC(v_vtmp, t, 1);
	if( m_xtmp.elem == NULL || v_vtmp.elem == NULL ){
		goto err;
	}
	MTX_SET(s_tmp, 1, 1, scalar);

	/*
	 * X = ones(n, t);
	 * X(:, 2:t) = mysign(2 * rand(n, t-1) - ones(n, t-1));
	 * X = undupli(X, [], prnt);
	 * X = X / n;
	 */
	mtx_unique(1, &m_x);
	{
		m_matrix_t	m_tmp;

		MTX_SET_ALLOC(m_tmp, n, t - 1);
		if( m_tmp.elem == NULL ){
			goto err;
		}
		mtx_rand(&m_tmp);
		mtx_scalar_multiply(&m_tmp, 2, &m_tmp);
		mtx_scalar_addup(&m_tmp, -1, &m_tmp);
		_mysign(&m_tmp, &m_tmp);
		mtx_set_submatrix(&m_tmp, &m_x, 0, 1);
		MTX_SET_FREE(m_tmp);
	}
	if( _undupli(&m_x, NULL) < 0 ){
		goto err;
	}
	tmp = (double) n;
	tmp = 1. / tmp;
	mtx_scalar_multiply(&m_x, tmp, &m_x);

	/*
	 * Loop it < ITER_MAX
	 */
	it = nmv = 0;
	est_old = 0.;
	while( 1 ){
		it++;
		/*
		 * Y = normapp(A, 'notransp', X, varargin{:}) */
		if( _normapp(estfunc, NOTRANSPOSE, &m_x, m_lu, idx, &m_y) < 0 ){
			goto err;
		}
		nmv++;

		mtx_abs(&m_y, &m_xtmp);
		mtx_sum(MTX_COL, &m_xtmp, &v_vtmp);
		mtx_sort_vector(&v_vtmp, &v_vals, isort);
		_reverse(v_vals.row, isort);
		for( i = 0 ; i < v_vals.row ; i++ ){
			v_vals.elem[i] = v_vtmp.elem[isort[i]];
			v_vals_ind.elem[i] = v_ind.elem[isort[i]];
		}
		*est_norm = v_vals.elem[0];

		if( *est_norm > est_old || it == 2 ){
			est_j = (int) v_vals_ind.elem[0];
			mtx_get_submatrix(&m_y, &v_w, 0, isort[0]);
		}

		if( it >= 2 && *est_norm <= est_old ){
			*est_norm = est_old;
			info = 2;
			break;
		}
		est_old = *est_norm;

		if( it > ITER_MAX ){
			it = ITER_MAX;
			info = 1;
			break;
		}

		/* S_old = S; S = mysign(Y); */
		size = m_sold.row * m_sold.col * sizeof(double);
		memcpy(m_sold.elem, m_s.elem, size);
		_mysign(&m_y, &m_s);
		if( a_is_real ){
			/* SS = S_old' * S */
			mtx_transpose(&m_sold, &m_sold_tmp);
			if( mtx_multiply(&m_sold_tmp, &m_s, &m_ss) < 0 ){
				goto err;
			}
			/* np = sum(max(abs(SS)) == n) */
			mtx_abs(&m_ss, &m_ss);
			mtx_max(&m_ss, &v_vtmp);
			np = mtx_elem_count(n, &v_vtmp);
			if( np == t ){
				info = 3;
				break;
			}
			/*
			 * S or Sold ʿԤ S åꡣ
			 */
			r = _undupli(&m_s, &m_sold);
			if( r < 0 ){
				goto err;
			}
			rpt_s += r;
		}

		/* Z = normapp(A, 'transp', S, varargin{:}) */
		if( _normapp(estfunc, TRANSPOSE, &m_s, m_lu, idx, &m_z) < 0 ){
			goto err;
		}
		nmv++;

		/*
		 * for i=1:n, Zvals(i) = norm(Z(i,:), inf); end
		 * ι®ǡ
		 * Zvals = max(abs(Z), [], 2)
		 */
		mtx_abs(&m_z, &m_z);
		mtx_max_row(&m_z, &v_zvals);

		if( it >= 2 ){
			mtx_max(&v_zvals, &s_tmp);
			if( s_tmp.elem[0] == v_zvals.elem[est_j] ){
				info = 4;
				break;
			}
		}


		/*
		 * [temp, m] = sort(Zvals);
		 * m = m(n:-1:1);
		 */
		mtx_sort_vector(&v_zvals, &v_zvals_tmp, isort);
		_reverse(n, isort);

		/*
		 * ñ̥٥ȥο
		 * may be reduced below (if it > 1)
		 */
		imax = t;
		if( it == 1 ){
			for( i = 0 ; i < t ; i++ ){
				if( i < n ){
					v_ind.elem[i] = isort[i];
				}
				v_ind_hist.elem[i] = v_ind.elem[i];
			}
		}else{
			/* rep = sum(ismember(m(1:t), ind_hist)) */
			rep = 0;
			for( i = 0 ; i < t ; i++ ){
				if( i >= n )	break;
				for( j = 0 ; j < t ; j++ ){
					if( isort[i] == v_ind_hist.elem[j] ){
						rep++;
						break;
					}
				}
			}
			rpt_e += rep;
			if( rep == t ){
				info = 5;
				break;
			}
			j = 0;
			for( i = 0 ; i < t ; i++ ){
				if( j > n - 1 ){
					imax = i - 1;
					break;
				}
				while( 1 ){
					ret = mtx_any(&v_ind_hist,
							MTX_EQ, isort[j]);
					if( ret < 0 ){
						goto err;
					}
					if( ret == 0 ){
						break;
					}
					j++;
					if( j > n - 1 ){
						imax = i - 1;
						break;
					}
				}
				if( j > n - 1 )	break;
				v_ind.elem[i] = isort[j];
				j++;
			}
			/* ind_hist = [ind_hist; ind(1:imax)] */
			if( _vector_cat(&v_ind_hist, &v_ind, imax) < 0 ){
				goto err;
			}
		}

		mtx_unique(0, &m_x);
		for( j = 0 ; j < imax ; j++ ){
			index = (int) v_ind.elem[j];
			index = index * m_x.col + j;
			m_x.elem[index] = 1;
		}
	}
	
end:
	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(m_tmp);
	MTX_SET_FREE(m_x);
	MTX_SET_FREE(m_y);
	MTX_SET_FREE(m_z);
	MTX_SET_FREE(v_w);
	MTX_SET_FREE(m_s);
	MTX_SET_FREE(m_sold);
	MTX_SET_FREE(m_sold_tmp);
	MTX_SET_FREE(m_ss);
	MTX_SET_FREE(v_ind);
	MTX_SET_FREE(v_ind_hist);
	MTX_SET_FREE(v_vals);
	MTX_SET_FREE(v_vals_ind);
	MTX_SET_FREE(v_zvals);
	MTX_SET_FREE(v_zvals_tmp);
	MTX_SET_FREE(m_xtmp);
	MTX_SET_FREE(v_vtmp);
	if( isort )	free(isort);
	return 0;
err:
	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(m_tmp);
	MTX_SET_FREE(m_x);
	MTX_SET_FREE(m_y);
	MTX_SET_FREE(m_z);
	MTX_SET_FREE(v_w);
	MTX_SET_FREE(m_s);
	MTX_SET_FREE(m_sold);
	MTX_SET_FREE(m_sold_tmp);
	MTX_SET_FREE(m_ss);
	MTX_SET_FREE(v_ind);
	MTX_SET_FREE(v_ind_hist);
	MTX_SET_FREE(v_vals);
	MTX_SET_FREE(v_vals_ind);
	MTX_SET_FREE(v_zvals);
	MTX_SET_FREE(v_zvals_tmp);
	MTX_SET_FREE(m_xtmp);
	MTX_SET_FREE(v_vtmp);
	if( isort )	free(isort);
	log_sys("ERROR: mtx_est_1norm()\n");
	return -1;
}

/*
 * ս¤ؤ롣
 */
static void _reverse(int len, int *isort)
{
	int	half = len / 2;
	int	i, tmp;

	for( i = 0 ; i < half ; i++ ){
		tmp = isort[i];
		isort[i] = isort[len - 1 - i];
		isort[len - 1 - i] = tmp;
	}
}

#if 0
/*
 * ǤˡΤ뤫ɤȽꡣ
 */
static int _any(m_matrix_t *mtx, int relation, double val)
{
	int	i, j;
	int	rowmax, colmax;
	double	*ptr;
	int	found = 0;

	rowmax = mtx->row;
	colmax = mtx->col;
	ptr = mtx->elem;
	for( i = 0 ; i < rowmax ; i++ ){
		for( j = 0 ; j < colmax ; j++ ){
			switch( relation ){
			case	MTX_EQ:
				if( ptr[i * colmax + j] == val ){
					found = 1;
					goto end;
				}
				break;
			case	MTX_NOTEQ:
				if( ptr[i * colmax + j] != val ){
					found = 1;
					goto end;
				}
				break;
			case	MTX_LESS:
				if( ptr[i * colmax + j] < val ){
					found = 1;
					goto end;
				}
				break;
			case	MTX_LESSEQ:
				if( ptr[i * colmax + j] <= val ){
					found = 1;
					goto end;
				}
				break;
			case	MTX_MORE:
				if( ptr[i * colmax + j] > val ){
					found = 1;
					goto end;
				}
				break;
			case	MTX_MOREEQ:
				if( ptr[i * colmax + j] >= val ){
					found = 1;
					goto end;
				}
				break;
			default	:
				goto err;
			}
		}
	}

end:
	return found;
err:
	log_sys("ERROR: _any()\n");
	return 0;
}
#endif

/*
 * ٥ȥĤʤ
 */
static int _vector_cat(m_matrix_t *trg, m_matrix_t *cat, int n)
{
	double	*ptr = NULL;
	int	is_colvector, max, size, i;

	is_colvector = (trg->col == 1)? 1 : 0;
	max = (is_colvector)? trg->row : trg->col;

	size = (max + n) * sizeof(double);
	ptr = (double *) malloc(size);
	if( ptr == NULL ){
		goto err;
	}

	for( i = 0 ; i < max ; i++ ){
		ptr[i] = trg->elem[i];
	}
	for( i = 0 ; i < n ; i++ ){
		ptr[i + max] = cat->elem[i];
	}

	free(trg->elem);
	trg->elem = ptr;
	if( is_colvector ){
		trg->row = max + n;
	}else{
		trg->col = max + n;
	}

	return 0;
err:
	log_sys("ERROR: _vector_cat()\n");
	return -1;
}

static int _undupli(m_matrix_t *m_s, m_matrix_t *m_sold)
{
	int	rowmax, colmax, r, tmp_col;
	int	j, rpt;
	int	w_row, w_col;
	int	jstart, last_col;
	m_matrix_t	m_w, m_tmp0, m_tmp1, v_s;

	memset(&v_s, 0, sizeof(m_matrix_t));
	memset(&m_w, 0, sizeof(m_matrix_t));
	memset(&m_tmp0, 0, sizeof(m_matrix_t));
	memset(&m_tmp1, 0, sizeof(m_matrix_t));

	/*
	 * [n, t] = size(S)
	 */
	rowmax = m_s->row;
	colmax = m_s->col;
	r = 0;
	if( colmax == 1 )	goto end;

	MTX_SET_ALLOC(v_s, rowmax, 1);
	if( v_s.elem == NULL ){
		goto err;
	}

	if( m_sold == NULL ){
		/*
		 * W = [S(:,1) zeros(n,t-1)]
		 */
		w_row = rowmax;
		w_col = colmax;
		MTX_SET_ALLOC(m_w, w_row, w_col);
		MTX_SET_ALLOC(m_tmp0, w_row, w_col);
		MTX_SET_ALLOC(m_tmp1, w_row, w_col);
		if( m_w.elem == NULL
		 || m_tmp0.elem == NULL || m_tmp1.elem == NULL ){
			goto err;
		}
		mtx_get_submatrix(m_s, &v_s, 0, 0);
		mtx_set_submatrix(&v_s, &m_w, 0, 0);
		jstart = 1;
		last_col = 0;
	}else{
		/*
		 * W = [S_old zeros(n,t-1)]
		 */
		w_row = rowmax;
		w_col = m_sold->col + colmax - 1;
		MTX_SET_ALLOC(m_w, w_row, w_col);
		MTX_SET_ALLOC(m_tmp0, w_row, w_col);
		MTX_SET_ALLOC(m_tmp1, w_row, w_col);
		if( m_w.elem == NULL
		 || m_tmp0.elem == NULL || m_tmp1.elem == NULL ){
			goto err;
		}
		mtx_set_submatrix(m_sold, &m_w, 0, 0);
		jstart = 0;
		last_col = colmax - 1;
	}


	for( j = jstart ; j < colmax ; j++ ){
		rpt = 0;
		while( 1 ){
			/* S(:,j)' */
			mtx_get_submatrix(m_s, &v_s, 0, j);
			/* transpose v_s */
			tmp_col = v_s.col;
			v_s.col = v_s.row;
			v_s.row = tmp_col;

			/* W(:,1:last_col) */
			m_tmp0.col = last_col + 1;
			mtx_get_submatrix(&m_w, &m_tmp0, 0, 0);

			/* S(:,j)' * W(:,1:last_col) */
			m_tmp1.row = 1;
			m_tmp1.col = last_col + 1;
			if( mtx_multiply(&v_s, &m_tmp0, &m_tmp1) < 0 ){
			}
			/* max(abs(S(:,j)' * W(:,1:last_col))) */
			mtx_abs(&m_tmp1, &m_tmp1);
			m_tmp0.row = 1;
			m_tmp0.col = 1;
			mtx_max_row(&m_tmp1, &m_tmp0);

			/* re-transpose v_s */
			tmp_col = v_s.col;
			v_s.col = v_s.row;
			v_s.row = tmp_col;

			/* reset tmp*.row, col */
			m_tmp0.row = m_tmp1.row = w_row;
			m_tmp0.col = m_tmp1.col = w_col;

			/*
			 * while:
			 * max(max(abs( S(:,j)' * W(:,1:last_col) ))) == n
			 */
			if( ((int) m_tmp0.elem[0]) != rowmax ){
				break;
			}

			rpt++;
			/*
			 * S(:,j) =
			 * 	mysign(2*rand(rowmax,1) - ones(rowmax,1))
			 */
			mtx_rand(&v_s);
			mtx_scalar_multiply(&v_s, 2, &v_s);
			mtx_scalar_addup(&v_s, -1, &v_s);
			_mysign(&v_s, &v_s);
			mtx_set_submatrix(&v_s, m_s, 0, j);

			if( rpt > rowmax / colmax ){
				break;
			}
		}

		r += (rpt > 0)? 1 : 0;
		if( j < colmax - 1 ){
			last_col++;
			/* W(:,last_col) = S(:,j) */
			mtx_set_submatrix(&v_s, &m_w, 0, last_col);
		}
	}


end:
	MTX_SET_FREE(v_s);
	MTX_SET_FREE(m_w);
	MTX_SET_FREE(m_tmp0);
	MTX_SET_FREE(m_tmp1);
	return r;
err:
	MTX_SET_FREE(v_s);
	MTX_SET_FREE(m_w);
	MTX_SET_FREE(m_tmp0);
	MTX_SET_FREE(m_tmp1);
	log_sys("ERROR: _undupli()\n");
	return -1;
}

static int _mysign(m_matrix_t *mtx, m_matrix_t *sgn)
{
	int	i, j;
	int	rowmax, colmax;
	int	index_r, index;
	double	*m, *s;

	rowmax = mtx->row;
	colmax = mtx->col;

	if( rowmax != sgn->row || colmax != sgn->col ){
		goto err;
	}

	m = mtx->elem;
	s = sgn->elem;
	for( i = 0 ; i < rowmax ; i++ ){
		index_r = i * colmax;
		for( j = 0 ; j < colmax ; j++ ){
			index = index_r + j;
			if( m[index] < 0 ){
				s[index] = -1;
			}else{
				s[index] = 1;
			}
		}
	}

	return 0;
err:
	log_sys("ERROR: _mysign()\n");
	return -1;
}

static int _normapp(int (*estfunc)(int flag, m_matrix_t *v_x, m_matrix_t *m_lu, int *idx, m_matrix_t *v_f), int flag, m_matrix_t *v_x, m_matrix_t *m_lu, int *idx, m_matrix_t *v_f)
{
	int	ret;
	ret = estfunc(flag, v_x, m_lu, idx, v_f);
	return ret;
}

typedef struct _sort {
	int	index;		/* Υǥå	*/
	double	elem;		/* бǡ		*/
} _sort_t;

/*
 * ٥ȥ򥽡Ȥ롣
 */
static int mtx_sort_vector(m_matrix_t *src, m_matrix_t *trg, int *indx)
{
	int	max, i;
	_sort_t	*s = NULL;

	max = src->row * src->col;

	s = (_sort_t *) malloc(max * sizeof(_sort_t));
	if( s == NULL ){
		goto err;
	}

	for( i = 0 ; i < max ; i++ ){
		s[i].index = i;
		s[i].elem = src->elem[i];
	}

	qsort(s, max, sizeof(_sort_t), _vec_sort);

	for( i = 0 ; i < max ; i++ ){
		indx[i] = s[i].index;
		trg->elem[i] = s[i].elem;
	}

	free(s);
	return 0;
err:
	if(s)	free(s);
	log_sys("ERROR: mtx_sort_vector()\n");
	return -1;
}

/*
 * mtx_sort_vectorΤʤǡqsortϤ٥ȥӴؿ
 */
static int _vec_sort(const void *ss1, const void *ss2)
{
	_sort_t	*s1, *s2;
	int	comp;

	s1 = (_sort_t *) ss1;
	s2 = (_sort_t *) ss2;
	if( s1->elem > s2->elem ){
		comp = 1;
	}else if( s1->elem < s2->elem ){
		comp = -1;
	}else{
		comp = 0;
	}
	return comp;
}

/*
 * Ǥ롣
 */
static int mtx_rand(m_matrix_t *mtx)
{
	int	i, j;
	int	rowmax, colmax;
	int	index, r_index;
	double	*ptr;

	rowmax = mtx->row;
	colmax = mtx->col;
	ptr = mtx->elem;
	for( i = 0 ; i < rowmax ; i++ ){
		r_index = i * colmax;
		for( j = 0 ; j < colmax ; j++ ){
			index = r_index + j;
			ptr[index] = (double) rand() / (RAND_MAX + 1.0);
		}
	}

	return 0;
}
