#include	<stdio.h>
#include	<math.h>

#include	"logUtil.h"

#include	"lu.h"
#include	"nr_lu.h"
#include	"nr_mutil.h"

#define	TINY	1.0e-20


#if 0
static int ludcmp(double **a, int n, int *indx, double *d);
static void lubksb(double **a, int n, int *indx, double b[]);
static int tlubksb(double **a, int n, int *indx, double b[]);
#endif

/*****************************************************************************
 * ؿ̾
 *      mtx_ludecomp: LUʬ򤹤롣
 * ǽ
 *      LUʬ򤹤롣
 *
 *      
 *      1: Υꥸ򣰤飱Ѵ롣
 *      2: LUʬ򤹤롣
 *      3: Υꥸ򣱤飰Ѵ롣
 * 
 *      m_matrix_t *mtx;	I: LUʬ򤹤
 *      m_matrix_t *m_lu;	O: LU
 *      int *v_index;		O: ؤ
 * ؿ
 *      0 :	
 *	-1:	۾ʥƥ۾
 *	-2:	۾ʥ󥮥顼
 * 
 *****************************************************************************/
int mtx_ludecomp(m_matrix_t *mtx, m_matrix_t *m_lu, int *v_index)
{
	double	d;
	double	**lu = NULL;
	int	ret = -1;

	lu = nr_alloc_matrix(1, m_lu->row, 1, m_lu->col);
	if( lu == NULL ){
		log_sys("nr_alloc_matrix() ERROR\n");
		goto err;
	}
	nr_copy_origin0to1(mtx->elem, lu, m_lu->row, m_lu->col);

	ret = nr_ludcmp(lu, m_lu->row, v_index - 1, &d);
	if( ret < 0 ){
		goto err;
	}

	nr_copy_origin1to0(lu, m_lu->elem, m_lu->row, m_lu->col);

	nr_free_matrix(lu, 1, m_lu->row, 1, m_lu->col);
	return 0;
err:
	if(lu)	nr_free_matrix(lu, 1, m_lu->row, 1, m_lu->col);
	if( ret == -2 ){
		log_sys("## Singular matrix in mtx_ludecomp()\n");
	}else{
		log_sys("ERROR: mtx_ludecomp()\n");
	}
	return ret;
}
/*****************************************************************************
 * ؿ̾
 *      mtx_lusolve: ALUʬȤäϢΩAx=b򤯡
 * ǽ
 *      ALUʬȤäϢΩAx=b򤯡
 *
 *      
 *      1: Υꥸ򣰤飱Ѵ롣
 *      2: LUȤäơ٥ȥ뤴Ȥ˲򤯡
 * 
 *      m_matrix_t *m_lu;	I: LU
 *      int *v_index;		I: ؤ
 *      m_matrix_t *v_b;	I: ϢΩαա٥ȥ
 *      m_matrix_t *v_x;	O: ϢΩβ٥ȥ
 * ؿ
 *      0 :	
 *	-1:	۾
 * 
 *	v_b(v_x)ϡ٥ȥޤȤƹˤƤ褤
 *****************************************************************************/
int mtx_lusolve(m_matrix_t *m_lu, int *v_index, m_matrix_t *v_b, m_matrix_t *v_x)
{
	double	**lu = NULL;
	double	*vec = NULL;
	int	i, j;

	lu = nr_alloc_matrix(1, m_lu->row, 1, m_lu->col);
	vec = nr_alloc_vector(1, m_lu->col);
	if( lu == NULL || vec == NULL ){
		goto err;
	}
	nr_copy_origin0to1(m_lu->elem, lu, m_lu->row, m_lu->col);

	for( j = 0 ; j < v_b->col ; j++ ){
		for( i = 0 ; i < v_b->row ; i++ ){
			vec[i + 1] = v_b->elem[i * v_b->col + j];
		}

		nr_lubksb(lu, v_x->row, v_index - 1, vec);

		for( i = 0 ; i < v_b->row ; i++ ){
			v_x->elem[i * v_b->col + j] = vec[i + 1];
		}
	}

	nr_free_matrix(lu, 1, m_lu->row, 1, m_lu->col);
	nr_free_vector(vec, 1, m_lu->col);
	return 0;
err:
	if(lu)	nr_free_matrix(lu, 1, m_lu->row, 1, m_lu->col);
	if(vec)	nr_free_vector(vec, 1, m_lu->col);
	log_sys("ERROR: mtx_lusolve()\n");
	return -1;
}
/*****************************************************************************
 * ؿ̾
 *      mtx_tlusolve: ALUʬȤäϢΩA'x=b򤯡
 * ǽ
 *      ALUʬȤäϢΩ: A'x=b(A': Ažֹ)򤯡
 *
 *      
 *      1: Υꥸ򣰤飱Ѵ롣
 *      2: LUžǡˤȤäơ٥ȥ뤴Ȥ˲򤯡
 * 
 *      m_matrix_t *m_lu;	I: LU
 *      int *v_index;		I: ؤ
 *      m_matrix_t *v_b;	I: ϢΩαա٥ȥ
 *      m_matrix_t *v_x;	O: ϢΩβ٥ȥ
 * ؿ
 *      0 :	
 *	-1:	۾
 * 
 *	v_b(v_x)ϡ٥ȥޤȤƹˤƤ褤
 *****************************************************************************/
int mtx_tlusolve(m_matrix_t *m_lu, int *v_index, m_matrix_t *v_b, m_matrix_t *v_x)
{
	double	**lu = NULL;
	double	*vec = NULL;
	int	i, j;

	lu = nr_alloc_matrix(1, m_lu->row, 1, m_lu->col);
	vec = nr_alloc_vector(1, m_lu->col);
	if( lu == NULL || vec == NULL ){
		goto err;
	}
	nr_copy_origin0to1(m_lu->elem, lu, m_lu->row, m_lu->col);

	for( j = 0 ; j < v_b->col ; j++ ){
		for( i = 0 ; i < v_b->row ; i++ ){
			vec[i + 1] = v_b->elem[i * v_b->col + j];
		}

		if( nr_tlubksb(lu, v_x->row, v_index - 1, vec) < 0 ){
			goto err;
		}

		for( i = 0 ; i < v_b->row ; i++ ){
			v_x->elem[i * v_b->col + j] = vec[i + 1];
		}
	}

	nr_free_matrix(lu, 1, m_lu->row, 1, m_lu->col);
	nr_free_vector(vec, 1, m_lu->col);
	return 0;
err:
	if(lu)	nr_free_matrix(lu, 1, m_lu->row, 1, m_lu->col);
	if(vec)	nr_free_vector(vec, 1, m_lu->col);
	log_sys("ERROR: mtx_tlusolve()\n");
	return -1;
}
#if 0
/*----------------ludcmp------------------------------------------
 *
 *  LUʬ
 *
 *      I:a, n, indx
 *      O:a, indx, d
 */
static int ludcmp(double **a, int n, int *indx, double *d)
{
        int i,j,k,imax=0;
        double big,dum,sum,temp;
        double *vv;

	vv = mtx_alloc_vector(1, n);
	if( vv == NULL ){
		log_sys("mtx_alloc_vector() ERROR\n");
		goto err;
	}

        *d=1.0;
        for (i=1;i<=n;i++) {
                big=0.0;
                for (j=1;j<=n;j++)
                        if ((temp=fabs(a[i][j])) > big) big=temp;
                if (big == 0.0) {
                        log_sys("## Singular matrix in ludcmp\n");
                        goto err_singular;
                }
                vv[i]=1.0/big;

        }
        for (j=1;j<=n;j++) {
                for (i=1;i<j;i++) {
                        sum=a[i][j];
                        for (k=1;k<i;k++) sum -= a[i][k]*a[k][j];
                        a[i][j]=sum;
                }
                big=0.0;
                for (i=j;i<=n;i++) {
                        sum=a[i][j];
                        for (k=1;k<j;k++)
                                sum -= a[i][k]*a[k][j];
                        a[i][j]=sum;
                        if ( (dum=vv[i]*fabs(sum)) >= big) {
                                big=dum;
                                imax=i;
                        }
                }
#ifdef DBG
                if(imax==0) {
                        /* DBGW(1, ("##Error in ludcmp imax=%d\n")); */
                        goto err;
                }
#endif
                if (j != imax) {
                        for (k=1;k<=n;k++) {
                                dum=a[imax][k];
                                a[imax][k]=a[j][k];
                                a[j][k]=dum;
                        }
                        *d = -(*d);
                        vv[imax]=vv[j];
                }
                indx[j]=imax;
                if (a[j][j] == 0.0) a[j][j]=TINY;
                if (j != n) {
                        dum=1.0/(a[j][j]);
                        for (i=j+1;i<=n;i++) a[i][j] *= dum;
                }
        }

	mtx_free_vector(vv, 1, n);
	return 0;
err:
	if( vv )	mtx_free_vector(vv, 1, n);
	log_sys("ERROR: ludcmp()\n");
	return -1;
err_singular:
	if( vv )	mtx_free_vector(vv, 1, n);
	return -2;
}

/*----------------lubksb------------------------------------------
 *
 *  LU
 *  n Ĥ A*x = b 򤯡
 *  Ϥ a[1..n][1..n] ϡludcmp() ֤ A LUʬǤ롣
 *  Ϥ٥ȥ b[1..n] ϡդΥ٥ȥǤ롣
 *  ܥ롼ϡa, n, indx 񤭴ʤΤǡ
 *  դ b ѤäƤ֤ⷫȤ롣
 *  b κǽˡ¤ǤǽθƤΤǡ
 *  ΨŪ˵չ󤬵롣
 *
 *      I:a, n, indx, b
 *      O:b
 *      ꥿͡ʤ
 */
static void lubksb(double **a, int n, int *indx, double b[])
{
        int	i, ii = 0, ip ,j;
        double	sum;


	/*
	 * ii ʤ顢ϡb κǽΥǤʤǤźǤ롣
	 * Ը򴹤򸵤ᤷʤ鼰Ԥ
	 * A*x = (L*U)*x = L*(U*x) = b
	 * L*y = b, U*x = y
	 * y[1] = b[1] / L[1,1]
	 * y[i] = {b[i] - Sigma(j=1,i-1)(L[i,j]*y[j])} / L[i,i]
	 * i = 2, 3, ..., n; L[i,i] = 1
	 */
        for ( i = 1 ; i <= n ; i++ ) {
                ip = indx[i];
                sum = b[ip];
                b[ip] = b[i];
                if (ii){
                        for ( j = ii ; j <= i - 1 ; j++ ){
				sum -= a[i][j] * b[j];
			}
		}else if (sum){
			/*
			 * 󥼥Ǥ˽вä
			 * ʹߤϾΥ롼פ¤롣
			 */
			ii = i;
		}
                b[i] = sum;
        }
	/*
	 * θԤ
	 * x[n] = y[n] / U[n,n]
	 * x[i] = {y[i] - Sigma(j=i+1,n)(U[i,j] * x[j])} / U[i,i]
	 * i = n-1, n-2, ..., 1
	 */
        for ( i = n ; i >= 1 ; i-- ) {
                sum = b[i];
                for ( j = i + 1 ; j <= n ; j++ ){
			sum -= a[i][j] * b[j];
		}
                b[i] = sum / a[i][i];
        }
}


/*----------------tlubksb------------------------------------------
 *
 *  LUžǡ
 *  n Ĥ A'*x = b 򤯡A'ϡAžֹ
 *  Ϥ a[1..n][1..n] ϡludcmp() ֤ A LUʬǤ롣
 *  Ϥ٥ȥ b[1..n] ϡդΥ٥ȥǤ롣
 *  ܥ롼ϡa, n, indx 񤭴ʤΤǡ
 *  դ b ѤäƤ֤ⷫȤ롣
 *  b κǽˡ¤ǤǽθƤΤǡ
 *  ΨŪ˵չ󤬵롣
 *
 *      I:a, n, indx, b
 *      O:b
 *      ꥿͡ʤ
 */
static int tlubksb(double **a, int n, int *indx, double b[])
{
        int	i, ii = 0, ip ,j;
        double	sum;
	double	*ytmp	= NULL;
	double	*y;

	ytmp = (double *) malloc(n * sizeof(double));
	if( ytmp == NULL){
		goto err;
	}
	y = ytmp - 1;


	/*
	 * ii ʤ顢ϡb κǽΥǤʤǤźǤ롣
	 * Ԥ
	 * A'*x = (L*U)'*x = (U'*L')*x = U'*(L'*x) = b
	 * U'*y = b, L'*x = y
	 * y[1] = b[1] / U'[1,1]
	 * y[i] = {b[i] - Sigma(j=1,i-1)(U'[i,j]*y[j])} / U'[i,i]
	 * i = 2, 3, ..., n
	 */
        for ( i = 1 ; i <= n ; i++ ) {
                sum = b[i];
                if (ii){
                        for ( j = ii ; j <= i - 1 ; j++ ){
				sum -= a[j][i] * y[j];
			}
		}else if (sum){
			/*
			 * 󥼥Ǥ˽вä
			 * ʹߤϾΥ롼פ¤롣
			 */
			ii = i;
		}
                y[i] = sum / a[i][i];
        }
	/*
	 * Ը򴹤򸵤ᤷʤ鼰θԤ
	 * x[n] = y[n] / L'[n,n]
	 * x[i] = {y[i] - Sigma(j=i+1,n)(L'[i,j] * x[j])} / L'[i,i]
	 * i = n-1, n-2, ..., 1; L'[i,i] = 1
	 */
        for ( i = n ; i >= 1 ; i-- ) {
                sum = y[i];
                for ( j = i + 1 ; j <= n ; j++ ){
			sum -= a[j][i] * y[j];
		}
		y[i] = sum;
                ip = indx[i];
		b[i] = b[ip];
                b[ip] = sum;
        }


	free(ytmp);
	return 0;
err:
	if(ytmp)	free(ytmp);
	log_sys("ERROR: tlubksb()\n");
	return -1;
}
#endif
