/*
 *
 *	NDF solver
 *
 *	ndf.c    Ver 5.1 02/04/08        20:27:02
 *
 */
#ifndef lint
static char     *sccsid = "@(#)ndf.c     5.1     02/04/08 (solverlib)";
#endif
/* for SCCS */

#include	<stdio.h>
#include	<string.h>
#include	<math.h>

#include	"logUtil.h"

#include	"ndf.h"
#include	"matrix.h"
#include	"lu.h"
#include	"svd.h"
#include	"condest.h"
#include	"c_que.h"

#define	NDF_NUMERICAL_RELATION
/*
*/
#ifdef	NDF_NUMERICAL_RELATION
#define NDF_DELTA	1.0e-6
#define NDF_ISEQ(a, b)		( fabs((a) - (b)) <= NDF_DELTA * fabs(a) )
#define NDF_ISNOTEQ(a, b)	( !NDF_ISEQ(a, b) )
#define NDF_ISGT(a, b)		( (a) > (b) && NDF_ISNOTEQ(a, b) )
#define NDF_ISGTEQ(a, b)	( (a) > (b) || NDF_ISEQ(a, b) )
#define NDF_ISLT(a, b)		( (a) < (b) && NDF_ISNOTEQ(a, b) )
#define NDF_ISLTEQ(a, b)	( (a) < (b) || NDF_ISEQ(a, b) )
#else
#define NDF_ISEQ(a, b)		( (a) == (b) )
#define NDF_ISNOTEQ(a, b)	( (a) != (b) )
#define NDF_ISGT(a, b)		( (a) > (b) )
#define NDF_ISGTEQ(a, b)	( (a) >= (b) )
#define NDF_ISLT(a, b)		( (a) < (b) )
#define NDF_ISLTEQ(a, b)	( (a) <= (b) )
#endif

#define SIGN(x)	(((x) < 0)? -1 : 1)
#define MAX(x, y)	(((x) > (y))? (x) : (y))
#define MIN(x, y)	(((x) < (y))? (x) : (y))

#define	MAX_ITER	4	/* ˥塼ȥȿκ	*/
#define	MAX_ORDER	5	/* NDF(BDF)κ缡	*/

/* mass type */
#define TYPE_NOMASS		0
#define TYPE_CONST		1
#define TYPE_YINDEPENDENT	2
#define TYPE_WEAK		3
#define TYPE_STRONG		4

/* ic type */
#define IC_DIAGONAL	1
#define IC_FULL		2
#define IC_SPARSE	3

/* result que */
typedef struct _que_result {
	c_que_t	que;
	double	t;
	double	*y;
} que_result_t;


/* static functions */
static int _daeic_state_independent(ndf_input_t *in, ndf_option_t *opt, m_matrix_t *m_m0, int ictype, double t0, m_matrix_t *v_y0, m_matrix_t *v_yp0, m_matrix_t *v_f0, m_matrix_t *v_threshold, m_matrix_t *v_fac, m_matrix_t *m_dfdy0, int (*odefunc)(double t, double *y, double *f), ndf_status_t *stats);
static int _set_yp(m_matrix_t *v_f, m_matrix_t *v_d, m_matrix_t *m_v, int ndif_index, m_index_t *dif_index, m_matrix_t *v_yp0);
static int _numerical_jacobian(double t, m_matrix_t *v_y, m_matrix_t *v_f, int (*odefunc)(double t, double *y, double *f), m_matrix_t *v_threshold, int isempty, m_matrix_t *v_fac, m_matrix_t *m_dfdy, ndf_status_t *stats);
static int _matrix_rowscale(m_matrix_t *v_rowscale, m_matrix_t *m_iter);
static int _change_stepsize(int order_k, int max_k, double h_rate, m_matrix_t *m_kJ, m_matrix_t *m_kI, m_matrix_t *m_difU, m_matrix_t *m_dif);
static int _get_psi(int order_k, double *inv_g, m_matrix_t *v_gamma, m_matrix_t *m_dif, m_matrix_t *v_psi);

static int _interpolate(double h, int k, double tnew, m_matrix_t *v_ynew, m_matrix_t *m_dif, double tinterp, m_matrix_t *v_yinterp);

static que_result_t *_alloc_que(double t, m_matrix_t *v_y);
static void _free_que(que_result_t *que);

static int _predict_y(ndf_input_t *in, int order_k, m_matrix_t *m_dif, m_matrix_t *v_y, m_matrix_t *v_pred);

/*****************************************************************************
 * ؿ̾
 *	ndf_init: NDFʿʬ˥ץν
 * ǽ
 *	NDFʿʬˤΥץ롣
 *
 *	
 *	1: ץ롣
 * 
 *	ndf_option_t *option;	IO: NDFץݥ
 * ؿ
 *	̵
 * 
 *****************************************************************************/
void ndf_init(ndf_option_t *option)
{
	memset(option, 0, sizeof(ndf_option_t));

	/* Ƹץѥƥ */
	option->tol.rel = 1.0e-3;
	option->tol.abs = 1.0e-6;
	option->tol.normctrl = OFF;
	option->tol.positive = OFF;

	/* 䥳ӥץѥƥ */

	/* ̹ץѥƥ */
	option->dae.ydepend = WEAK;
	option->dae.singular= MAYBE;

	/* ƥåץץѥƥ */

	/* BDFץѥƥ */
	option->bdf.order = MAX_ORDER;
	option->bdf.isbdf = OFF;
}

/*****************************************************************************
 * ؿ̾
 *	ndf_term: NDFʿʬ˾ʬθ
 * ǽ
 *	NDFʿʬˤξʬΥ꥽롣
 *
 *	
 *	1: Υ꥽롣
 * 
 *	ndf_solution_t *sol;	I: NDFݥ
 * ؿ
 *	̵
 * 
 *****************************************************************************/
void ndf_term(ndf_solution_t *sol)
{
	if( sol->tt )	free(sol->tt);
	if( sol->yy )	free(sol->yy);
	sol->tt = NULL;
	sol->yy = NULL;
}

/*****************************************************************************
 * ؿ̾
 *	ndf_solve: NDFʿʬˤȤäƾʬ
 * ǽ
 *	NDFʿʬˤȤäƾʬ򤯡
 *
 *	
 *	1: 
 *	2: ᥤ롼ס
 *	3: 1ƥåץ롼ס
 *	4: ˥塼ȥ󥤥졼롼ס
 *	5: ϲ
 * 
 *	ndf_input_t *in;	I: NDF;ݥ
 *	double (*odefunc)(double , double*);	I: ʬմؿݥ
 *	ndf_option_t *option;	I: NDFץݥ
 *	ndf_status_t *stats;	O: NDF¹Ծ־ݥ
 *	ndf_solution_t *sol;	O: NDFݥ
 * ؿ
 *	0:	
 *	-1:	۾
 * 
 *****************************************************************************/
int ndf_solve(ndf_input_t *in, ndf_solution_t *sol, int (*odefunc)(double t, double *y, double *f), ndf_option_t *option, ndf_status_t *stats)
{
	/* å */
	double	tspan, htspan, tfinal;
	int	tdir;
	m_matrix_t	v_y0;
	m_matrix_t	v_f0;
	double	normy;
	double	hmax;
	double	threshold;
	m_matrix_t	v_threshold, v_jthreshold;
	int f_outNegative=true;	/* add 2003.04.02 by Satoh */

	int	size, i, j;

	/* ̹ */
	int	masstype;
	m_matrix_t	m_mass;
	int	isdae;
	m_matrix_t	v_yp0;
	int	ictype, m_current, yp0_ok;
	int	nonzero_num;
	m_matrix_t	m_mtnew;

	/* 䥳ӥ */
	m_matrix_t	m_dfdy, v_fac;

	/* ˡѥ᡼ν */
	double	gamma[] = {1., 3./2., 11./6., 25./12., 137./60.};
	double	ndf_kappa[] = {-37./200., -1./9., -0.0823, -0.0415, 0};
	double	bdf_kappa[] = {0, 0, 0, 0, 0};
	double	difU[] = {
		-1, -2, -3, -4, -5,
		 0,  1,  3,  6, 10,
		 0,  0, -1, -4,-10,
		 0,  0,  0,  1,  5,
		 0,  0,  0,  0, -1,
	};
	m_matrix_t	v_gamma, v_kappa, m_difU;
	double	inv_g[MAX_ORDER];
	double	errconst[MAX_ORDER];
	int	max_k;
	double	maxK[] = {1, 2, 3, 4, 5};
	m_matrix_t	v_inv_g, v_errconst, v_maxK;
	double	kJ[] = {
		1, 2, 3, 4, 5,
		1, 2, 3, 4, 5,
		1, 2, 3, 4, 5,
		1, 2, 3, 4, 5,
		1, 2, 3, 4, 5,
	};
	double	kI[] = {
		1, 1, 1, 1, 1,
		2, 2, 2, 2, 2,
		3, 3, 3, 3, 3,
		4, 4, 4, 4, 4,
		5, 5, 5, 5, 5,
	};
	m_matrix_t	m_kJ, m_kI;

	/*  yp  */
	double	t;
	int	j_current;
	m_matrix_t	v_y, v_yp;
	m_matrix_t	m_lu;
	int		*v_index = NULL;

	/* ƥåץѥ᡼ꡣhmin, abs_h, h */
	double	hmin, abs_h, h;

	/*  */
	int	order_k;
	m_matrix_t	m_order_k, m_dif;
	int	prev_k, nconhk;
	double	prev_h, hinv_g_k;

	/* ˥塼ȥȿκդη */
	m_matrix_t	m_iter, v_psi, m_psidmdy;

	/* DAE򤯳ˡŪ󥰤Ŭѡ */
	m_matrix_t	v_rowscale;

	/* ΰγݡ */
#if 0
	m_matrix_t	v_tout, m_yout;
#endif
	que_result_t	qh_result, *que;
	int	len;

	/* 롼Хåե */
	double	tnew;
	double	normynew, invwt, min_norm;
	double	old_norm, new_norm, rate;
	double	err_est;
	m_matrix_t	v_tmp, v_tmp1;
	m_matrix_t	v_pred, v_ynew, v_difkp1, v_invwt, v_rhs, v_del;
	m_matrix_t	v_yinterp;
	int		i_t0;

	/* 롼ե饰 */
	int	havrate, done, gotynew, tooslow, nofailed;
	int	forced_positive = 0;



	/* ¤Τ */
	memset(&m_mass, 0, sizeof(m_matrix_t));
	memset(&v_yp0, 0, sizeof(m_matrix_t));
	memset(&v_y0, 0, sizeof(m_matrix_t));
	memset(&v_f0, 0, sizeof(m_matrix_t));
	memset(&v_threshold, 0, sizeof(m_matrix_t));
	memset(&v_jthreshold, 0, sizeof(m_matrix_t));
	memset(&m_mtnew, 0, sizeof(m_matrix_t));
	memset(&m_dfdy, 0, sizeof(m_matrix_t));
	memset(&v_fac, 0, sizeof(m_matrix_t));
	memset(&v_gamma, 0, sizeof(m_matrix_t));
	memset(&v_kappa, 0, sizeof(m_matrix_t));
	memset(&m_difU, 0, sizeof(m_matrix_t));
	memset(&v_inv_g, 0, sizeof(m_matrix_t));
	memset(&v_errconst, 0, sizeof(m_matrix_t));
	memset(&v_maxK, 0, sizeof(m_matrix_t));
	memset(&v_y, 0, sizeof(m_matrix_t));
	memset(&v_yp, 0, sizeof(m_matrix_t));
	memset(&m_lu, 0, sizeof(m_matrix_t));
	memset(&m_order_k, 0, sizeof(m_matrix_t));
	memset(&m_dif, 0, sizeof(m_matrix_t));
	memset(&m_iter, 0, sizeof(m_matrix_t));
	memset(&v_psi, 0, sizeof(m_matrix_t));
	memset(&m_psidmdy, 0, sizeof(m_matrix_t));
	memset(&v_rowscale, 0, sizeof(m_matrix_t));
	memset(&v_tmp, 0, sizeof(m_matrix_t));
	memset(&v_tmp1, 0, sizeof(m_matrix_t));
	memset(&v_pred, 0, sizeof(m_matrix_t));
	memset(&v_ynew, 0, sizeof(m_matrix_t));
	memset(&v_difkp1, 0, sizeof(m_matrix_t));
	memset(&v_invwt, 0, sizeof(m_matrix_t));
	memset(&v_rhs, 0, sizeof(m_matrix_t));
	memset(&v_del, 0, sizeof(m_matrix_t));
	memset(&v_yinterp, 0, sizeof(m_matrix_t));

	/* å */
	if( in == NULL || sol == NULL || odefunc == NULL
	 || option == NULL || stats == NULL ){
		log_sys("Illegal arguments\n");
		goto err;
	}

	if( in->t0num < 2 ){
		log_sys("t0num[%d] < 2\n", in->t0num);
		goto err;
	}
	htspan = in->t0[1] - in->t0[0];
	tspan  = fabs(in->t0[in->t0num - 1] - in->t0[0]);
	tdir = SIGN(in->t0[in->t0num - 1] - in->t0[0]);
	tfinal = in->t0[in->t0num - 1];

	MTX_SET(v_y0, in->y0num, 1, in->y0);

	MTX_SET_ALLOC(v_f0, in->y0num, 1);
	if( v_f0.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	if( odefunc(in->t0[0], in->y0, v_f0.elem) < 0 ){
		log_sys("odefunc()\n");
		goto err;
	}
	stats->nfevals++;
mtx_dump(30, "Odefunc:", &v_f0);

	if( option->tol.rel < 100 * NDF_EPSILON ){
		option->tol.rel = 100 * NDF_EPSILON;
	}
	if( option->tol.normctrl ){
		mtx_norm(&v_y0, 2, &normy);
	}else{
		/* ??? */
	}

	threshold = option->tol.abs / option->tol.rel;
	MTX_SET_ALLOC(v_threshold, in->y0num, 1);
	MTX_SET_ALLOC(v_jthreshold, in->y0num, 1);
	if( v_threshold.elem == NULL || v_jthreshold.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	for( i = 0 ; i < v_threshold.row ; i++ ){
		v_threshold.elem[i] = threshold;
		v_jthreshold.elem[i] = option->tol.abs;
	}

	/* by default, hmax is 1/10 of the interval */
	hmax = option->step.maxsize;
	hmax = (hmax > 0)? hmax : 0.1 * tspan;
	hmax = (hmax < tspan)? hmax : tspan;

	/*
	 * ׻ѤΥƥݥ꡼ΰݡ
	 */
	MTX_SET_ALLOC(v_tmp, in->y0num, 1);
	if( v_tmp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}

	/*
	 * ̹
	 */
	if( option->dae.mfunc == NULL ){
		if( option->dae.mconst == NULL ){
			masstype = TYPE_NOMASS;
		}else{
			masstype = TYPE_CONST;
		}
	}else{
		switch( option->dae.ydepend ){
		case	NONE:	masstype = TYPE_YINDEPENDENT;	break;
		case	WEAK:	masstype = TYPE_WEAK;		break;
		case	STRONG:	masstype = TYPE_STRONG;		break;
		default	:
			log_sys("Unknown ydepend(%d)\n", option->dae.ydepend);
			goto err;
		}
	}
	isdae = 0;
	if( masstype != TYPE_NOMASS ){
		MTX_SET_ALLOC(m_mass, in->y0num, in->y0num);
		if( m_mass.elem == NULL ){
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		if( masstype == TYPE_CONST ){
			size = in->y0num * in->y0num * sizeof(double);
			memcpy(m_mass.elem, option->dae.mconst, size);
		}else{
			option->dae.mfunc(in->t0[0], in->y0, m_mass.elem);
		}

		nonzero_num = mtx_nonzero(&m_mass);
		if( nonzero_num <= 0 ){
			log_sys("nonzero num(%d) <= 0\n", nonzero_num);
			goto err;
		}
		switch( option->dae.singular ){
		case	NO:	isdae = 0;	break;
		case	YES:	isdae = 1;	break;
		case	MAYBE:	isdae = (NDF_EPSILON * nonzero_num *
					mtx_est_condition_number(&m_mass) > 1);
				break;
		default	:
			log_sys("Unknown singular(%d)\n", option->dae.singular);
			goto err;
		}
	}
DBGW(10, ("yp0_ok??? isdae[%d]\n", isdae));
	yp0_ok = 0;
	if( isdae ){
		MTX_SET_ALLOC(v_yp0, in->y0num, 1);
		if( v_yp0.elem == NULL ){
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		size = in->y0num * sizeof(double);
		memset(v_yp0.elem, 0, size);
		if( option->dae.initslope == NULL ){
			yp0_ok = 0;
		}else{
			m_matrix_t	v_mass_yp, v_mass_yp_f;
			double	norm_mypf, norm_myp, norm_f;

			memcpy(v_yp0.elem, option->dae.initslope, size);
			/* ̵̷ʽۤå			*/
			/* M(t0, y0)yp0 = f(t0, y0)			*/
			/* test yp0_ok = (norm(Mt*yp0-f0) <=		*/
			/* 1e-3 * rtol * max(norm(Mt * yp), norm(f0))	*/
			memset(&v_mass_yp, 0, sizeof(m_matrix_t));
			memset(&v_mass_yp_f, 0, sizeof(m_matrix_t));
			MTX_SET_ALLOC(v_mass_yp, in->y0num, 1);
			MTX_SET_ALLOC(v_mass_yp_f, in->y0num, 1);
			if( v_mass_yp.elem == NULL || v_mass_yp_f.elem == NULL ){
				MTX_SET_FREE(v_mass_yp);
				MTX_SET_FREE(v_mass_yp_f);
				log_sys("MTX_SET_ALLOC()\n");
				goto err;
			}
			if( mtx_multiply(&m_mass, &v_yp0, &v_mass_yp) < 0 ){
				log_sys("mtx_multiply(l.413)");
				goto err;
			}
			mtx_subtract(&v_mass_yp, &v_f0, &v_mass_yp_f);
			mtx_norm(&v_mass_yp_f, 2, &norm_mypf);
			mtx_norm(&v_mass_yp, 2, &norm_myp);
			mtx_norm(&v_f0, 2, &norm_f);
			yp0_ok = (norm_mypf <= 1e-3 * option->tol.rel *
				((norm_myp > norm_f)? norm_myp : norm_f));
			MTX_SET_FREE(v_mass_yp);
			MTX_SET_FREE(v_mass_yp_f);
		}
		if( !yp0_ok ){
			if( masstype == TYPE_WEAK || masstype == TYPE_STRONG ){
				ictype = IC_SPARSE;
			}else{	/* M, M(t) */
				if( mtx_isdiagonal(&m_mass) ){
					ictype = IC_DIAGONAL;
				}else{
					if( mtx_issparse(&m_mass) ){
						ictype = IC_SPARSE;
					}else{
						ictype = IC_FULL;
					}
				}
			}
		}
	}
	m_current = 1;
	MTX_SET_ALLOC(m_mtnew, in->y0num, in->y0num);
	if( m_mtnew.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	size = m_mtnew.row * m_mtnew.col * sizeof(double);
	memcpy(m_mtnew.elem, m_mass.elem, size);


	/*
	 * 䥳ӥ
	 */
	MTX_SET_ALLOC(m_dfdy, in->y0num, in->y0num);
	if( m_dfdy.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	if( option->jac.jconst != NULL ){
		size = in->y0num * in->y0num * sizeof(double);
		memcpy(m_dfdy.elem, option->jac.jconst, size);
	}else if( option->jac.jfunc != NULL ){
		option->jac.jfunc(in->t0[0], in->y0, m_dfdy.elem);
		stats->npds++;
	}else{
		/* ͥ䥳ӥ */
		MTX_SET_ALLOC(v_fac, in->y0num, 1);
		if( v_fac.elem == NULL ){
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		if( _numerical_jacobian(in->t0[0], &v_y0, &v_f0, odefunc,
		 &v_jthreshold, 1, &v_fac, &m_dfdy, stats) < 0 ){
			log_sys("_numerical_jacobian()\n");
			goto err;
		}
		stats->npds++;
	}
mtx_dump(30, "Jacobian:", &m_dfdy);

	/*
	 * ˡѥ᡼ν
	 */
	/* gamma(k) = Sigma(j=1,k)(1/j) */
	MTX_SET(v_gamma, MAX_ORDER, 1, gamma);
	MTX_SET(v_kappa, MAX_ORDER, 1, ndf_kappa);
	if( option->bdf.isbdf ){
		v_kappa.elem = bdf_kappa;
	}

	/* inv_g = 1 ./ ((1 - kappa) .* gamma) */
	MTX_SET(v_inv_g, MAX_ORDER, 1, inv_g);
	mtx_elem_subtract_from_scalar(1, &v_kappa, &v_inv_g);
	mtx_elem_multiply(&v_gamma, &v_inv_g, &v_inv_g);
	mtx_elem_divide_into_scalar(1, &v_inv_g, &v_inv_g);

	/* errconst(k) = kappa(k) * gamma(k) + 1/(k+1) */
	MTX_SET(v_maxK, MAX_ORDER, 1, maxK);
	MTX_SET(v_errconst, MAX_ORDER, 1, errconst);
	mtx_scalar_addup(&v_maxK, 1, &v_errconst);
	mtx_elem_divide_into_scalar(1, &v_errconst, &v_errconst);
	{
		double	tmp[MAX_ORDER];
		m_matrix_t	v_tmp;
		MTX_SET(v_tmp, MAX_ORDER, 1, tmp);
		mtx_elem_multiply(&v_kappa, &v_gamma, &v_tmp);
		mtx_addup(&v_tmp, &v_errconst, &v_errconst);
	}

	/* maxK = (1, 2, ..., max_k) */
	max_k = v_maxK.row = option->bdf.order;
	/* kJ = 1, 2, ..., max_k	kI = 1, 1, ..., 1	*/
	/*      1, 2, ..., max_k	     2, 2, ..., 2	*/
	/*            ...       	           ...   	*/
	/*      1, 2, ..., max_k	     max_k,..., max_k	*/
	MTX_SET(m_kJ, option->bdf.order, option->bdf.order, kJ);
	MTX_SET(m_kI, option->bdf.order, option->bdf.order, kI);

	/* diff_U(MAX_ORDER*MAX_ORDER) -> diff_U(max_k*max_k)	*/
	MTX_SET_ALLOC(m_difU, option->bdf.order, option->bdf.order);
	if( m_difU.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	{
		m_matrix_t	m_tmp;
		MTX_SET(m_tmp, MAX_ORDER, MAX_ORDER, difU);
		mtx_get_submatrix(&m_tmp, &m_difU, 0, 0);
	}

	/*
	 *  yp 
	 * DAEФơǥեȤǤϡ̵̷ʽۤ׻롣
	 */
	t = in->t0[0];
	MTX_SET_ALLOC(v_y, in->y0num, 1);
	if( v_y.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	size = in->y0num * sizeof(double);
	memcpy(v_y.elem, in->y0, size);
	MTX_SET_ALLOC(v_yp, in->y0num, 1);
	if( v_yp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(m_lu, in->y0num, in->y0num);
	if( m_lu.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	v_index = (int *) malloc(in->y0num * sizeof(int));
	if( v_index == NULL ){
		log_sys("malloc()\n");
		goto err;
	}
	if( isdae && !yp0_ok ){
		size = in->y0num * sizeof(double);
		memcpy(v_yp.elem, v_yp0.elem, size);
		if( masstype == TYPE_WEAK || masstype == TYPE_STRONG ){
			/* ̤ݡ */
			log_sys("Not Support(y dependent mass type)\n");
			goto err;
#if 0
			_daeic_state_dependent();
			option->mass.mfunc(t, v_y.elem, &m_mass);
			size = m_mtnew.row * m_mtnew.col * sizeof(double);
			memcpy(m_mtnew.elem, m_mass.elem, size);
			m_current = 1;
#endif
		}else{
			if( _daeic_state_independent(in, option, &m_mass,
				ictype, t, &v_y, &v_yp, &v_f0, &v_jthreshold,
				&v_fac, &m_dfdy, odefunc, stats) < 0 ){
				goto err;
			}
		}
	}else{
		if( masstype == TYPE_NOMASS ){
			size = v_yp.row * v_yp.col * sizeof(double);
			memcpy(v_yp.elem, v_f0.elem, size);
		}else{
			if( isdae && yp0_ok ){
				size = v_yp.row * v_yp.col * sizeof(double);
				memcpy(v_yp.elem, v_yp0.elem, size);
			}else{	/* LUʬ */
				if( mtx_ludecomp(&m_mass, &m_lu, v_index) < 0 ){
					log_sys("mtx_ludecomp()\n");
					goto err;
				}
				if( mtx_lusolve(&m_lu, v_index, &v_f0, &v_yp) < 0 ){
					log_sys("mtx_lusolve()\n");
					goto err;
				}
				stats->ndecomps++;
				stats->nsolves++;
			}
		}
		/*
		 * 䥳ӥκꡣ
		 */
		if( option->jac.jconst != NULL ){
			memcpy(m_dfdy.elem, option->jac.jconst, size);
		}else if( option->jac.jfunc != NULL ){
			option->jac.jfunc(in->t0[0], v_y.elem, m_dfdy.elem);
		}else{
			/* ͥ䥳ӥ */
			if( _numerical_jacobian(in->t0[0], &v_y, &v_f0,
				odefunc, &v_jthreshold, 1, &v_fac,
						&m_dfdy, stats) < 0 ){
				log_sys("_numerical_jacobian()\n");
				goto err;
			}
			stats->npds++;
		}
	}
	j_current = 1;

	/*
	 * ƥåץѥ᡼ꡣhmin, abs_h, h
	 * hmin ϡ׻ǡ
	 * t + hmin  t Τ˶̤Ǥ٤˾̡
	 * Ǥϡt = 0 -> hmin = 0
	 */
#if 0
	hmin = 16 * NDF_EPSILON * fabs(t);
#else
	hmin = NDF_EPSILON * fabs(t); /* del 16 2003.03.14 */
#endif

	if( option->step.initstep > 0 ){
		abs_h = MAX(hmin, option->step.initstep);
		abs_h = MIN(hmax, abs_h);
	}else{	/* ƥåפο */
		/* yp = y'(t) Ȥäƽƥåץ h ׻ */
		double	wt, rh;
		m_matrix_t	v_wt;
		if( option->tol.normctrl ){
			/*
			 * wt = max(normy,threshold);
			 * rh = 1.25 * (norm(yp) / wt) / sqrt(rtol);
			 *	% 1.25 = 1 / 0.8
			 */
			double	norm_yp;

			mtx_norm(&v_yp, 2, &norm_yp);
			wt = MAX(normy, threshold);
			rh = 1.25 * (norm_yp / wt) / sqrt(option->tol.rel);
		}else{
			/*
			 * wt = max(abs(y),threshold);
			 * rh = 1.25 * norm(yp ./ wt,inf) / sqrt(rtol);
			 */
			double	norm_yp_wt;
			m_matrix_t	v_tmp;

			MTX_SET_ALLOC(v_tmp, v_yp.row, v_yp.col);
			if( v_tmp.elem == NULL ){
				log_sys("MTX_SET_ALLOC()\n");
				goto err;
			}
			size = v_yp.row * v_yp.col * sizeof(double);
			memcpy(v_tmp.elem, v_yp.elem, size);

			MTX_SET_ALLOC(v_wt, v_y.row, v_y.col);
			if( v_wt.elem == NULL ){
				MTX_SET_FREE(v_tmp);
				log_sys("MTX_SET_ALLOC()\n");
				goto err;
			}
			mtx_abs(&v_y, &v_wt);
			mtx_max2(&v_wt, &v_threshold, &v_wt);
			mtx_elem_divide(&v_tmp, &v_wt, &v_tmp);
			mtx_norm(&v_tmp, PLUS_INF, &norm_yp_wt);
			rh = 1.25 * (norm_yp_wt) / sqrt(option->tol.rel);
			MTX_SET_FREE(v_tmp);
			MTX_SET_FREE(v_wt);
		}
DBGW(30, ("hmax[%e], htspan[%e], rh[%e]\n", hmax, htspan, rh));
		abs_h = MIN(hmax, htspan);
		if( abs_h * rh > 1 ){
			abs_h = 1 / rh;
		}
		abs_h = MAX(abs_h, hmin);

		if( !isdae ){
			double	tdel, norm;
			m_matrix_t	v_f1, v_dfdt;

			/*
			 * BDF1 θ 0.5 * h^2 * y''(t)
			 * 顡Ŭ h ꤹ롣
			 */
			h = tdir * abs_h;
DBGW(30, ("h[%e]\n", h));
			/*
			 * tdel = (t + tdir *
			 * 	min(sqrt(eps) * max(abs(t), abs(t+h)), absh))
			 * 	- t
			 */
			tdel = MAX(fabs(t), fabs(t + h));
			tdel = MIN(sqrt(NDF_EPSILON) * tdel, abs_h);
			tdel = (t + tdir * tdel) - t;

			MTX_SET_ALLOC(v_f1, v_y.row, v_y.col);
			MTX_SET_ALLOC(v_dfdt, v_y.row, v_y.col);
			if( v_f1.elem == NULL || v_dfdt.elem == NULL ){
				MTX_SET_FREE(v_f1);
				MTX_SET_FREE(v_dfdt);
				log_sys("MTX_SET_ALLOC()\n");
				goto err;
			}
			size = v_y.row * v_y.col * sizeof(double);
			memset(v_f1.elem, 0, size);
			memset(v_dfdt.elem, 0, size);
			odefunc(t + tdel, v_y.elem, v_f1.elem);
			/* dfdt = (f1 - f0) ./ tdel */
			mtx_subtract(&v_f1, &v_f0, &v_dfdt);
			mtx_scalar_multiply(&v_dfdt, 1./tdel, &v_dfdt);
			/* */
			if( mtx_multiply(&m_dfdy, &v_yp, &v_tmp) < 0 ){
				log_sys("mtx_multiply(l.707)");
				goto err;
			}
			mtx_addup(&v_dfdt, &v_tmp, &v_tmp);
			if( option->tol.normctrl ){
				if( masstype != TYPE_NOMASS ){
					if( mtx_lusolve(&m_lu, v_index,
							&v_tmp, &v_tmp) < 0 ){
						MTX_SET_FREE(v_f1);
						MTX_SET_FREE(v_dfdt);
						log_sys("mtx_lusolve()\n");
						goto err;
					}
				}
				mtx_norm(&v_tmp, 2, &norm);
				norm /= wt;
			}else{
				if( masstype != TYPE_NOMASS ){
					if( mtx_lusolve(&m_lu, v_index,
							&v_tmp, &v_tmp) < 0 ){
						MTX_SET_FREE(v_f1);
						MTX_SET_FREE(v_dfdt);
						log_sys("mtx_lusolve()\n");
						goto err;
					}
				}
				mtx_elem_divide(&v_tmp, &v_wt, &v_tmp);
				mtx_norm(&v_tmp, PLUS_INF, &norm);
			}
			rh = 1.25 * sqrt(0.5 * norm / option->tol.rel);
			abs_h = MIN(hmax, htspan);
			if( abs_h * rh > 1 ){
				abs_h = 1 / rh;
			}
			abs_h = MAX(abs_h, hmin);

			MTX_SET_FREE(v_f1);
			MTX_SET_FREE(v_dfdt);
		}
	}
	h = tdir * abs_h;

	/*
	 * 
	 * order = 1BDF1鳫ϡ
	 * order(k) = (1, 2, ..., k) && k = 1
	 */
	order_k = 1;
	MTX_SET_ALLOC(m_order_k, 1, 1);
	if( m_order_k.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	prev_k = order_k;
	prev_h = abs_h;
	MTX_SET_ALLOC(m_dif, in->y0num, max_k + 2);
	if( m_dif.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	size = in->y0num * (max_k + 2) * sizeof(double);
	memset(m_dif.elem, 0, size);
	{
		m_matrix_t	v_tmp;
		MTX_SET_ALLOC(v_tmp, in->y0num, 1);
		if( v_tmp.elem == NULL ){
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		mtx_scalar_multiply(&v_yp, h, &v_tmp);
		mtx_set_submatrix(&v_tmp, &m_dif, 0, 0);
		MTX_SET_FREE(v_tmp);
	}
	hinv_g_k = h * v_inv_g.elem[(order_k) - 1];
	nconhk = 0;	/* steps taken with current h and k */

	/* ˥塼ȥȿκդη				*/
	/* Miter * del(i) = -Mt(dif(k+1) + psi) + hinv_g_k * odefunc	*/
	/* Miter = Mt - hinv_g_k * dfdy + dMt/dy * psi			*/
	MTX_SET_ALLOC(m_iter, in->y0num, in->y0num);
	if( m_iter.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	mtx_scalar_multiply(&m_dfdy, hinv_g_k, &m_iter);
	mtx_subtract(&m_mass, &m_iter, &m_iter);
	/* strongly state-dependent mass matrix: Not support */
	MTX_SET_ALLOC(v_psi, in->y0num, 1);
	MTX_SET_ALLOC(m_psidmdy, in->y0num, in->y0num);
	if( v_psi.elem == NULL || m_psidmdy.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	if( masstype == TYPE_STRONG ){
#if 0
		/* psi = inv_g(k) * Sigma(m=1,k)(gamma(m)*Delta(m)y(n))	*/
		if( _get_psi(order_k, inv_g, &v_gamma, &m_dif, &v_psi) < 0 ){
			log_sys("_get_psi()\n");
			goto err;
		}
		/* psi * dm/dy */
		_numerical_jacobian(&m_psidmdy);
		_numerical_jacobian(t, &v_y, &v_f, odefunc, &v_jthreshold, 1, &v_fac, &m_dfdy, stats);
		/* Miter += psi * dm/dy */
		mtx_addup(&m_iter, &m_psidmdy, &m_iter);
#endif
	}

	/*
	 * DAE򤯳ˡŪ󥰤Ŭѡ
	 * rowscale = 1 ./ max(abs(m_iter), [], 2)
	 */
	if( isdae ){
		MTX_SET_ALLOC(v_rowscale, in->y0num, 1);
		if( v_rowscale.elem == NULL ){
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		_matrix_rowscale(&v_rowscale, &m_iter);
mtx_dump(30, "v_rowscale:", &v_rowscale);
	}
	/* LUʬ */
	if( mtx_ludecomp(&m_iter, &m_lu, v_index) < 0 ){
		log_sys("mtx_ludecomp()\n");
		goto err;
	}
	stats->ndecomps++;

DBGW(30, ("h[%e] k[%d] inv_g_k[%e] hinv_g_k[%e]\n",
	h, order_k, v_inv_g.elem[(order_k) - 1], hinv_g_k));
mtx_dump(30, "v_y0:", &v_y0);
mtx_dump(30, "v_yp:", &v_yp);
mtx_dump(30, "v_f0:", &v_f0);
mtx_dump(50, "m_dfdy:", &m_dfdy);
mtx_dump(100, "Mass:", &m_mass);
mtx_dump(100, "Miter:", &m_iter);

	/*
	 * ΰγݡ
	 */
	c_initque( &qh_result.que );
	que = _alloc_que(t, &v_y);
	if( que == NULL ){
		log_sys("_alloc_que()\n");
		goto err;
	}
	c_enque(&qh_result.que, &que->que);
	i_t0 = 1;
#if 0
	MTX_SET_ALLOC(v_tout, in->t0num, 1);
	if( v_tout.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	size = in->t0num * 1 * sizeof(double);
	memset(v_tout.elem, 0, size);
	MTX_SET_ALLOC(m_yout, in->t0num, in->y0num);
	if( m_yout.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	size = in->t0num * in->y0num * sizeof(double);
	memset(m_yout.elem, 0, size);
	nout = 0;
	v_tout.elem[0] = t;
	mtx_set_submatrix(&v_y, &m_yout, 0, 0);
#endif

	/*
	 * 롼Хåեΰγݡ
	 */
	MTX_SET_ALLOC(v_pred, in->y0num, 1);
	if( v_pred.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(v_tmp1, in->y0num, 1);
	if( v_tmp1.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(v_ynew, in->y0num, 1);
	if( v_ynew.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(v_difkp1, in->y0num, 1);
	if( v_difkp1.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(v_invwt, v_y.row, v_y.col);
	if( v_invwt.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(v_rhs, v_y.row, v_y.col);
	if( v_rhs.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(v_del, v_y.row, v_y.col);
	if( v_del.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(v_yinterp, in->y0num, 1);
	if( v_yinterp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}


DBGW(30, ("Loop start\n"));
	/*
	 * The Main Loop.
	 */
	havrate = 0;
	done = 0;
	while( !done ){
static int loop_cnt;
DBGW(30, ("********  Loop[%d] : t[%e] abs_h[%e] order_k[%d]\n", ++loop_cnt, t, abs_h, order_k));
#if 0
		hmin = 16 * NDF_EPSILON * fabs(t);
#else
		hmin = NDF_EPSILON * fabs(t); /* del 16 2003.03.14 */
#endif
#ifdef DBG
		f_outNegative= true;	/* add 2003.04.02 by Satoh */
#endif
		abs_h = MAX(hmin, abs_h);
		abs_h = MIN(hmax, abs_h);
		h = tdir * abs_h;

		/* tfinal - t ΣʤСƥå򿭤Ф */
		if( 1.1 * abs_h >= fabs(tfinal - t) ){
			h = tfinal - t;
			abs_h = fabs(h);
			done = 1;
		}

		if( abs_h != prev_h || order_k != prev_k ){
DBGW(30, ("Change step size: h[%e -> %e] k[%d -> %d]\n", prev_h, abs_h, prev_k, order_k));
			if( _change_stepsize(order_k, max_k, abs_h/prev_h,
					&m_kJ, &m_kI, &m_difU, &m_dif) < 0 ){
				log_sys("_change_stepsize()\n");
				goto err;
			}

			hinv_g_k = h * inv_g[(order_k) - 1];
			nconhk = 0;

			/*
			 * շ
			 * Miter = Mt + dM/dy * psi - hinv_g_k * df/dy
			 */
			mtx_scalar_multiply(&m_dfdy, hinv_g_k, &m_iter);
			mtx_subtract(&m_mass, &m_iter, &m_iter);
			if( masstype == TYPE_STRONG ){
				mtx_addup(&m_psidmdy, &m_iter, &m_iter);
			}
			if( isdae ){	/*  */
				if( _matrix_rowscale(&v_rowscale, &m_iter) < 0 ){
					log_sys("_matrix_rowscale()\n");
					goto err;
				}
			}
			/* LUʬ */
			if( mtx_ludecomp(&m_iter, &m_lu, v_index) < 0 ){
				log_sys("mtx_ludecomp()\n");
				goto err;
			}
			stats->ndecomps++;
			havrate = 0;
		}

DBGW(30, ("*****  Loop for advancing 1 step start\n"));
		/*
		 * Loop for advancing 1 step.
		 */
		nofailed = 0;	/* no failed attempts */
		while( 1 ){		/* Evaluate the formula. */
DBGW(30, ("***  Loop to get YNEW\n"));
			gotynew = 0;	/* Is ynew evaluated yet ? */
			while( !gotynew ){
				/*
				 * ynew ׻
				 * psi = dif(:,K) * (G(K) * inv_g_k)
				 */
				if( _get_psi(order_k, inv_g,
						&v_gamma, &m_dif, &v_psi) < 0 ){
					log_sys("_get_psi()\n");
					goto err;
				}

				/*
				 * t + h βͽ¬
				 */
				tnew = t + h;
				if( done ){
					tnew = tfinal;
				}
				h = tnew - t;	/* Purify h */
				/* pred = y + sum(dif(:,K), 2) */
				if( _predict_y(in, order_k, &m_dif, &v_y, &v_pred) < 0 ){
					goto err;
				}
				/* ynew = pred */
				size = in->y0num * sizeof(double);
				memcpy(v_ynew.elem, v_pred.elem, size);
DBGW(30, ("Estimate Y(tnew[%e] = t[%e] + h[%e])\n", tnew, t, h));
mtx_dump(100, "Previous v_y:", &v_y);
mtx_dump(100, "v_pred:", &v_pred);

				/*
				 * dif(k+1) = ynew(final) - pred
				 *          = k+1 Υynewθʬ
				 */
				/* ˽ */
				size = in->y0num * sizeof(double);
				memset(v_difkp1.elem, 0, size);

				/* Υ */
				if( option->tol.normctrl ){
					/*
					 * invwt =
					 * 	1 / max(max(normy,normynew),
					 * 	threshold);
					 */
					mtx_norm(&v_ynew, 2, &normynew);
					invwt = MAX(normy, normynew);
					invwt = MAX(invwt, threshold);
					invwt = 1 / invwt;
					/*
					 * minnrm = 100*eps*(normynew * invwt);
					 */
					min_norm = (normynew * invwt);
					min_norm *= 100 * NDF_EPSILON;
				}else{
					/*
					 * invwt =
					 * 	1 ./ max(max(abs(y),abs(ynew))
					 * 	,threshold);
					 */
					mtx_abs(&v_y, &v_invwt);
					mtx_abs(&v_ynew, &v_tmp);
					mtx_max2(&v_invwt, &v_tmp, &v_tmp);
					mtx_max2(&v_tmp, &v_threshold, &v_tmp);
					mtx_elem_divide_into_scalar(
							1, &v_tmp, &v_invwt);
					/*
					 * minnrm = 100 * eps *
					 * 	norm(ynew .* invwt,inf);
					 */
					mtx_elem_multiply(&v_ynew,
							&v_invwt, &v_tmp);
					mtx_norm(&v_tmp, PLUS_INF, &min_norm);
					min_norm *= 100 * NDF_EPSILON;
				}

				/*
				 * RHSʱաˤ뤿ο̹
				 * ֡yˤ˰¸ʤ硣
				 * M(t,y) = M(t)
				 */
				if( masstype == TYPE_YINDEPENDENT ){
					option->dae.mfunc( tnew,
						v_ynew.elem, m_mtnew.elem);
				}

				/*
			 	 *  add 2003.0314
			 	 */
				if( option->tol.positive){
					forced_positive = mtx_any(&v_ynew, MTX_LESS, 0.0);
DBGW(30, ("forced_positive[%d]\n", forced_positive));
					if (forced_positive && f_outNegative){
						log_sys("Get Negative !!t,tnew,h:=%23.16e,%23.16e,%23.16e\n",t,tnew,h);
						for(i=0;i<in->y0num;i++){
							if( v_ynew.elem[i] < 0. ){
								log_sys("vy.elem[%d],m_dif,v_f0:=%23.16e,%23.16e,%23.16e\n",
									i+1,v_y.elem[i],v_ynew.elem[i]-v_y.elem[i],v_f0.elem[i]);
							}
						}
						f_outNegative = false;
						break;
					}
				}

DBGW(30, ("**  Newton Iteration Loop start\n"));
				/*
				 * Iterate with simplified Newton Method
				 */
				tooslow = 0;
				for( i = 0 ; i < MAX_ITER ; i++ ){
					/*
					 * RHSʱաˤ뤿
					 * ̹
					 * ֡yˤ˰¸硣
					 * M(t,y) = M(t,y)
					 */
					if( masstype == TYPE_WEAK
					 || masstype == TYPE_STRONG ){
						option->dae.mfunc( tnew,
							v_ynew.elem, m_mtnew.elem);
					}

					/*
					 * RHSʱե٥ȥˤ
					 * = hinv_g_k * odefunc(tnew, ynew)
					 *   - Mtnew * (psi + difkp1)
					 */
					odefunc(tnew, v_ynew.elem, v_f0.elem);
DBGW(30, ("tnew[%e] t[%e]\n", tnew, t));
					mtx_scalar_multiply(
						&v_f0, hinv_g_k, &v_rhs);
					mtx_addup(&v_psi, &v_difkp1, &v_tmp);
					if( mtx_multiply(&m_mtnew, &v_tmp, &v_tmp1) < 0 ){
						log_sys("mtx_multiply(l.1116)");
						goto err;
					}
					mtx_subtract(&v_rhs, &v_tmp1, &v_rhs);

					/*
					 * RHSʱաˤ򥹥󥰤
					 * rhs = RowScale .* rhs
					 */
					if( isdae ){
						mtx_elem_multiply(&v_rowscale, &v_rhs, &v_rhs);
					}

					/*
					 * ˥塼ȥˡβ
					 * del = U \ (L \ rhs)
					 */
					if( mtx_lusolve(&m_lu, v_index, &v_rhs, &v_del) < 0 ){
						log_sys("mtx_lusolve()\n");
						goto err;
					}

					/*
					 * Υ
					 */
					if( option->tol.normctrl ){
						/* newnrm = norm(del) * invwt */
						mtx_norm(&v_del, 2, &new_norm);
						new_norm *= invwt;
					}else{
						/* newnrm = norm(del .* invwt, inf) */
						mtx_elem_multiply(&v_del, &v_invwt, &v_tmp);
						mtx_norm(&v_tmp, PLUS_INF, &new_norm);
					}

					/*
					 * difkp1 = difkp1 + del
					 * ynew = pred + difkp1
					 */
					mtx_addup(&v_difkp1, &v_del, &v_difkp1);
					mtx_addup(&v_pred, &v_difkp1, &v_ynew);


					/* Ƚ */
DBGW(30, ("Judge: new_norm[%e] <= min_norm[%e] ???\n", new_norm, min_norm));
					/* if( new_norm <= min_norm ){} */
					if( NDF_ISLTEQ(new_norm, min_norm) ){
						gotynew = 1;
						break;
					}
					if( i == 0 ){
						if( havrate ){
							double	err_iter;
							err_iter = new_norm * rate / (1 - rate);
DBGW(30, ("Judge: err_iter[%e] <= 0.05*rtol[%e] ???\n", err_iter, 0.05*(option->tol.rel) ));
							/* if( err_iter <= 0.05 * (option->tol.rel) ){} */
							if( NDF_ISLTEQ(err_iter, 0.05 * (option->tol.rel)) ){
								gotynew = 1;
								break;
							}
						}else{
							rate = 0;
						}
					}else{
						double	err_iter;
						/* if( new_norm > 0.9 * old_norm ){} */
						if( NDF_ISGT(new_norm, 0.9 * old_norm) ){
							tooslow = 1;
							break;
						}
						rate = MAX(0.9*rate, new_norm/old_norm);
						havrate = 1;
						err_iter = new_norm * rate / (1 - rate);
DBGW(30, ("Judge: err_iter[%e] <= 0.5*rtol[%e] ???\n", err_iter, 0.5*(option->tol.rel) ));
						/* if( err_iter <= 0.5 * (option->tol.rel) ){} */
						if( NDF_ISLTEQ(err_iter, 0.5 * (option->tol.rel)) ){
							gotynew = 1;
							break;
						}
						if( i == MAX_ITER - 1 ){
							tooslow = 1;
							break;
						}
						/* if( 0.5 * (option->tol.rel) < err_iter * pow(rate, (double) (MAX_ITER - i - 1)) ){} */
						if( NDF_ISLT(0.5 * (option->tol.rel), err_iter * pow(rate, (double) (MAX_ITER - i - 1))) ){
							tooslow = 1;
							break;
						}
					}

					old_norm = new_norm;

					/*
			 		 *  add 2003.03.14
			 		 */
					if( option->tol.positive ){
						int	li;
						forced_positive = mtx_any(&v_ynew, MTX_LESS, 0.0);
DBGW(30, ("forced_positive[%d]\n", forced_positive));
						if (forced_positive && f_outNegative){
							log_sys("After Newton Get Negative !!t,tnew,h:=%23.16e,%23.16e,%23.16e\n",t,tnew,h);
							gotynew = 1;
							for(li=0;li<in->y0num;li++){
								if( v_ynew.elem[li] < 0. ){
									log_sys("vy.elem[%d],m_dif,v_f0:=%23.16e,%23.16e,%23.16e\n",
										li+1,v_y.elem[li],m_dif.elem[li],v_f0.elem[li]);
								}
							}
							f_outNegative = false;
							break;
						}
					}
				}	/* for(MAX_ITER) */

DBGW(30, ("**  Newton Iteration Loop end[%d] tooslow[%d]\n", i, tooslow));
				if( tooslow ){
					if( !j_current ){
						/* Jacobian κƷ׻ */
						if( option->jac.jconst != NULL ){
							/* skip */
						}else if( option->jac.jfunc != NULL ){
							option->jac.jfunc(t, v_y.elem, m_dfdy.elem);
						}else{
							/* ͥ䥳ӥ */
							if( _numerical_jacobian(
								t, &v_y, &v_f0,
								odefunc,
								&v_jthreshold,
								0, &v_fac,
								&m_dfdy, stats)
								< 0 ){
								log_sys("_numerical_jacobian()\n");
								goto err;
							}
						}
						stats->npds++;
						j_current = 1;
						if( !m_current ){
							/* m_mass κƷ׻ */
							if( option->dae.mfunc != NULL ){
								option->dae.mfunc(t, v_y.elem, m_mass.elem);
								m_current = 1;
								if( masstype == TYPE_STRONG ){
									/* not support */
								}
							}
						}
					}else{
						if( abs_h <= hmin ){
							log_sys("ERROR: tooslow && abs_h(%e) <= hmin(%e)\n", abs_h, hmin);
							goto err;
						}
						/* abs_h ̾ */
						prev_h = abs_h;
						abs_h = MAX(0.3 * abs_h, hmin);
						h = tdir * abs_h;
						done = 0;

						/*
						 * difRU = cumprod(
						 *	(kI - 1 -
						 *	kJ*(abs_h/prev_h))
						 *	./ kI
						 *	) * difU
						 * dif(:,K) =
						 *	dif(:,K) * difRU(K,K)
						 */
						if( _change_stepsize(order_k,
							max_k, abs_h/prev_h,
							&m_kJ, &m_kI,
							&m_difU, &m_dif) < 0 ){
							log_sys("_change_stepsize()\n");
							goto err;
						}

						hinv_g_k = h * inv_g[(order_k) - 1];
						nconhk = 0;
					}
					/*
					 * ˥塼ȥȿκդη
					 * Miter κƷ׻
					 *	Miter = Mt - hinv_g_k * dfdy
					 *		+ dMt/dy * psi
					 *	Miter * del(i) =
					 *		-Mt(dif(k+1) + psi)
					 *		+ hinv_g_k * odefunc
					 */
					mtx_scalar_multiply(&m_dfdy,
							hinv_g_k, &m_iter);
					mtx_subtract(&m_mass, &m_iter, &m_iter);
					/*
					 * strongly state-dependent
					 * mass matrix: Not support
					 */
					if( masstype == TYPE_STRONG ){
						/* Miter += psi * dm/dy */
						mtx_addup(&m_iter,
							&m_psidmdy, &m_iter);
					}

					/*
					 * DAE򤯳ˡ
					 * Ū󥰤Ŭѡ
					 * rowscale =
					 *	1 ./ max(abs(m_iter), [], 2)
					 */
					if( isdae ){
						_matrix_rowscale(
							&v_rowscale, &m_iter);
					}
					/* LUʬ */
					if( mtx_ludecomp(&m_iter,
							&m_lu, v_index) < 0 ){
						log_sys("After Newton Iter mtx_ludecomp()\n");
						goto err;
					}
					stats->ndecomps++;
					havrate = 0;
				}
			}	/* while(gotynew) */
DBGW(30, ("***  END of Loop to get YNEW\n"));

			/*
		 	* difkp1 
		 	* k+1Υ ynew θʬˤʤäƤ롣
		 	*/

			if( option->tol.normctrl ){
				double	norm;
				mtx_norm(&v_difkp1, 2, &norm);
				err_est = (norm * invwt) * errconst[(order_k) - 1];
			}else{
				double	norm;
				mtx_elem_multiply(&v_difkp1, &v_invwt, &v_tmp);
				mtx_norm(&v_tmp, PLUS_INF, &norm);
				err_est = norm * errconst[(order_k) - 1];
			}
DBGW(30, ("err_est[%e]\n", err_est));

			/*
		 	 *  (ꥸʥȽꡢ񤭽ФʬΤ 2003.03.14 ɲá)
		 	 */
			if( option->tol.positive && !forced_positive){
				int	li;
				forced_positive = mtx_any(&v_ynew, MTX_LESS, 0.0);
DBGW(30, ("forced_positive[%d]\n", forced_positive));
				if( forced_positive && f_outNegative){
					log_sys("After gotnew Get Negative !!t,tnew,h:=%23.16e,%23.16e,%23.16e\n",t,tnew,h);
					for(li=0;li<in->y0num;li++){
						if( v_ynew.elem[li] < 0. ){
							log_sys("vy.elem[%d],m_dif,v_f0:=%23.16e,%23.16e,%23.16e\n",
								li+1,v_y.elem[li],v_ynew.elem[li]-v_y.elem[li],v_f0.elem[li]);
						}
					}
					f_outNegative = false;
				}
			}

			/* 顼Ƚ */
			/* if( err_est <= option->tol.rel ){} */ /*  */
			if (NDF_ISLTEQ(err_est, option->tol.rel) && !forced_positive) {
				break;
			}else{					    /*  */
				if( abs_h <= hmin ){
					log_sys("ERROR: err_est(%e) > rtol(%e) && abs_h(%e) <= hmin(%e)\n", err_est, option->tol.rel, abs_h, hmin);
					goto err;
				}

				/* ƥåץѹ */
				prev_h = abs_h;
				if( !nofailed ){
					abs_h = MAX(hmin, 0.5 * abs_h);
				}else{
					double	hopt;
					double	tmp;
					nofailed = 0;

					/*
					 * ƥå h κŬ
					 * hopt =
					 * 	abs_h * max(0.1,
					 * 	0.833 * (rtol/err)^(1/(k+1)))
					 * 	0.833 = 1/1.2
					 */
					tmp = (double) (order_k + 1);
					tmp = 1. / tmp;
					hopt = option->tol.rel / err_est;
					hopt = 0.833 * pow(hopt, tmp);
					hopt = MAX(0.1, hopt);
					hopt = abs_h * hopt;
					/*
					 *  order_k ѹ
					 */
					if( order_k > 1 ){
						double	errkm1, hkm1, norm;

						mtx_get_submatrix( &m_dif,
							&v_tmp, 0, order_k - 1);
						mtx_addup(&v_tmp,
							&v_difkp1, &v_tmp);

						if( option->tol.normctrl ){
							mtx_norm(&v_tmp, 2, &norm);
							errkm1 = (norm * invwt)
								* errconst[(order_k - 1) - 1];
						}else{
							mtx_elem_multiply(
								&v_tmp,
								&v_invwt,
								&v_tmp);
							mtx_norm(&v_tmp, PLUS_INF, &norm);
							errkm1 = norm * errconst[(order_k - 1) - 1];
						}


						/*
						 * hkm1 = abs_h * max(0.1,
						 * 	0.769 * (rtol/errkm1)^
						 * 	(1/k))
						 * 	0.769 = 1/1.3
						 */
						hkm1 = option->tol.rel / errkm1;
						hkm1 = pow(hkm1, (double) (1./((double)order_k)));
						hkm1 = MAX(0.1, 0.769 * hkm1);
						hkm1 = abs_h * hkm1;

						/* if( hkm1 > hopt ){} */
						if( NDF_ISGT(hkm1, hopt) ){
							hopt = MIN(abs_h, hkm1);
							/*
							 * k = k - 1
							 * K = 1:k
							 */
							order_k--;
						}
					}

					abs_h = MAX(hmin, hopt);
					/* for forced_positive */
					abs_h = MIN(hmax, abs_h);
				}
				h = tdir * abs_h;
				/* if( abs_h < prev_h ){} */
				if( NDF_ISLT(abs_h, prev_h) ){
					done = 0;
				}

				/*
				 * difRU = cumprod(
				 *	(kI - 1 - kJ*(abs_h/prev_h)) ./ kI
				 *	) * difU
				 * dif(:,K) = dif(:,K) * difRU(K,K)
				 */
				if( _change_stepsize(order_k, max_k,
					abs_h/prev_h,
					&m_kJ, &m_kI, &m_difU, &m_dif) < 0 ){
					log_sys("_change_stepsize()\n");
					goto err;
				}

				hinv_g_k = h * inv_g[(order_k) - 1];
				nconhk = 0;

				/*
				 * ˥塼ȥȿκդη
				 * Miter κƷ׻
				 *	Miter = Mt - hinv_g_k * dfdy
				 *		+ dMt/dy * psi
				 *	Miter * del(i) =
				 *		-Mt(dif(k+1) + psi)
				 *		+ hinv_g_k * odefunc
				 */
				mtx_scalar_multiply(&m_dfdy,
						hinv_g_k, &m_iter);
				mtx_subtract(&m_mass, &m_iter, &m_iter);
				/*
				 * strongly state-dependent
				 * mass matrix: Not support
				 */
				if( masstype == TYPE_STRONG ){
					/* Miter += psi * dm/dy */
					mtx_addup(&m_iter,
						&m_psidmdy, &m_iter);
				}

				/*
				 * DAE򤯳ˡ
				 * Ū󥰤Ŭѡ
				 * rowscale =
				 *	1 ./ max(abs(m_iter), [], 2)
				 */
				if( isdae ){
					_matrix_rowscale(
						&v_rowscale, &m_iter);
				}
				/* LUʬ */
				if( mtx_ludecomp(&m_iter,
						&m_lu, v_index) < 0 ){
					log_sys("mtx_ludecomp()\n");
					goto err;
				}
				stats->ndecomps++;
				havrate = 0;
			}

		}	/* while(1) for advancing 1 step */

		stats->nsteps++;

		/*
		 * dif(:, k+2) = difkp1 - dif(:, k+1)
		 * dif(:, k+1) = difkp1
		 */
		mtx_get_submatrix(&m_dif, &v_tmp, 0, (order_k + 1) - 1);
		mtx_subtract(&v_difkp1, &v_tmp, &v_tmp);
		mtx_set_submatrix(&v_tmp, &m_dif, 0, (order_k + 2) - 1);
		mtx_set_submatrix(&v_difkp1, &m_dif, 0, (order_k + 1) - 1);

		for( j = order_k ; j > 0 ; j-- ){
			/* dif(:,j) = dif(:,j) + dif(:,j+1) */
			mtx_get_submatrix(&m_dif, &v_tmp, 0, (j) - 1);
			mtx_get_submatrix(&m_dif, &v_tmp1, 0, (j + 1) - 1);
			mtx_addup(&v_tmp, &v_tmp1, &v_tmp);
			mtx_set_submatrix(&v_tmp, &m_dif, 0, (j) - 1);
		}
mtx_dump(50, "Final m_dif:", &m_dif);


		/*
		 * 
		 */
		if( in->t0num == 2 ){
		/* 졼˻Ȥä򤽤Τޤ޽ */
			que = _alloc_que(tnew, &v_ynew);
			if( que == NULL ){
				log_sys("_alloc_que()\n");
				goto err;
			}
			c_enque(&qh_result.que, &que->que);
		}else{
		/* δؿ֤ͤƽ */
			/* while( tnew > in->t0[i_t0] ){} */
			while( NDF_ISGT(tnew, in->t0[i_t0]) ){
				if( _interpolate(h, order_k,
					tnew, &v_ynew, &m_dif,
					in->t0[i_t0], &v_yinterp) < 0 ){
					goto err;
				}
				que = _alloc_que(
					in->t0[i_t0], &v_yinterp);
				if( que == NULL ){
					log_sys("_alloc_que()\n");
					goto err;
				}
				c_enque(&qh_result.que, &que->que);
				i_t0++;
			}
			/* if( tnew == tfinal ){} */
			if( NDF_ISEQ(tnew, tfinal) ){
				que = _alloc_que(tnew, &v_ynew);
				if( que == NULL ){
					log_sys("_alloc_que()\n");
					goto err;
				}
				c_enque(&qh_result.que, &que->que);
			}
		}

		/*
		 * h, k κŬ
		 */
		prev_k = order_k;
		prev_h = abs_h;
		nconhk = MIN(nconhk + 1, max_k + 2);

		if( nconhk >= order_k + 2 ){
			double	tmp, hopt;
			int	kopt;

			tmp = (double) (order_k + 1);
			tmp = 1.2 * pow(err_est / option->tol.rel, 1. / tmp);
			/* if( tmp > 0.1 ){} */
			if( NDF_ISGT(tmp, 0.1) ){
				hopt = abs_h / tmp;
			}else{
				hopt = 10 * abs_h;
			}
			kopt = order_k;

			/*
			 * k -> k - 1 ?
			 */
			if( order_k > 1 ){
				double	norm, hkm1, errkm1;

				mtx_get_submatrix(&m_dif, &v_tmp,
							0, order_k - 1);
				if( option->tol.normctrl ){
					mtx_norm(&v_tmp, 2, &norm);
					errkm1 = (norm * invwt)
						* errconst[(order_k - 1) - 1];
				}else{
					mtx_elem_multiply(&v_tmp, &v_invwt, &v_tmp);
					mtx_norm(&v_tmp, PLUS_INF, &norm);
					errkm1 = norm
						* errconst[(order_k - 1) - 1];
				}
				tmp = (double) order_k;
				tmp = 1.3 * pow(errkm1/option->tol.rel, 1./tmp);
				/* if( tmp > 0.1 ){} */
				if( NDF_ISGT(tmp, 0.1) ){
					hkm1 = abs_h / tmp;
				}else{
					hkm1 = 10 * abs_h;
				}
				/* if( hkm1 > hopt ){} */
				if( NDF_ISGT(hkm1, hopt) ){
					hopt = hkm1;
					kopt = order_k - 1;
				}
			}

			/*
			 * k -> k + 1 ?
			 */
			if( order_k < max_k ){
				double	norm, hkp1, errkp1;

				/* dif(:,k+2) */
				mtx_get_submatrix(&m_dif, &v_tmp,
							0, (order_k + 2) - 1);
				if( option->tol.normctrl ){
					/* errkp1 =
					 *	(norm(dif(:,k+2)) * invwt)
					 *	* errconst(k+1)
					 */
					mtx_norm(&v_tmp, 2, &norm);
					errkp1 = (norm * invwt)
						* errconst[(order_k + 1) - 1];
				}else{
					/* errkp1 =
					 *	norm(dif(:,k+2) .* invwt, inf)
					 *	* errconst(k+1)
					 */
					mtx_elem_multiply(&v_tmp, &v_invwt, &v_tmp);
					mtx_norm(&v_tmp, PLUS_INF, &norm);
					errkp1 = norm
						* errconst[(order_k + 1) - 1];
				}
				tmp = (double) (order_k + 2);
				tmp = 1.4 * pow(errkp1/option->tol.rel, 1./tmp);
				/* if( tmp > 0.1 ){} */
				if( NDF_ISGT(tmp, 0.1) ){
					hkp1 = abs_h / tmp;
				}else{
					hkp1 = 10. * abs_h;
				}
				/* if( hkp1 > hopt ){} */
				if( NDF_ISGT(hkp1, hopt) ){
					hopt = hkp1;
					kopt = order_k + 1;
				}
			}

			/* if( hopt > abs_h ){} */
			if( NDF_ISGT(hopt, abs_h) ){
				abs_h = hopt;
				if( order_k != kopt ){
					/* k = kopt, K = 1:k; */
					order_k = kopt;
				}
			}
		}

		/*
		 * ʬ1ƥå׿ʤ롣
		 */
		t = tnew;
		size = v_y.row * v_y.col * sizeof(double);
		memcpy(v_y.elem, v_ynew.elem, size);
mtx_dump_withname(50, "Final v_y:", "Y", &v_y);

		if( option->tol.normctrl ){
			normy = normynew;
		}

		if( option->jac.jconst != NULL ){
			j_current = 1;
		}else{
			j_current = 0;
		}
#if 0
		{}else if( option->jac.jfunc != NULL ){
			j_current = 0;
		}else{	/* Numerical Jacobian */
			if( _numerical_jacobian(in->t0[0], &v_y0, &v_f0,
			odefunc, &v_jthreshold, 0, &v_fac, &m_dfdy, stats) < 0 ){
				log_sys("_numerical_jacobian()\n");
				goto err;
			}
			stats->npds++;
		}
#endif

		switch( masstype ){
		case	TYPE_NOMASS:
		case	TYPE_CONST:
			/*  Iñ̹or M */
			m_current = 1;
			break;
		case	TYPE_YINDEPENDENT:
			/* M(t) ϡtnew  Mtnew ǴɾѤߡ */
			size = m_mtnew.row * m_mtnew.col * sizeof(double);
			memcpy(m_mtnew.elem, m_mass.elem, size);
			m_current = 1;
			break;
		case	TYPE_WEAK:
		case	TYPE_STRONG:
			/* M(t,y) ϡǽ ynew ̤ɾ */
			m_current = 0;
			break;
		default	:
			log_sys("Unknown Mass type(%d)\n", masstype);
			goto err;
		}

	}	/* while(!done) */

	len = c_quelen(&qh_result.que);
	size = len * sizeof(double);
	sol->tt = (double *) malloc(size);
	if( sol->tt == NULL ){
		log_sys("malloc()\n");
		goto err;
	}
	size *= in->y0num;
	sol->yy = (double *) malloc(size);
	if( sol->yy == NULL ){
		log_sys("malloc()\n");
		goto err;
	}
	for( i = 0 ; i < len ; i++ ){
		que = (que_result_t *) c_deque( &qh_result.que );
		sol->tt[i] = que->t;
		for( j = 0 ; j < in->y0num ; j++ ){
			sol->yy[i * in->y0num + j] = que->y[j];
		}
		_free_que(que);
	}
	sol->tnum = len;


	MTX_SET_FREE(m_psidmdy);
	MTX_SET_FREE(v_yinterp);
	MTX_SET_FREE(m_mtnew);
	MTX_SET_FREE(v_yp);
	MTX_SET_FREE(v_f0);
	MTX_SET_FREE(v_threshold);
	MTX_SET_FREE(v_jthreshold);
	MTX_SET_FREE(m_mass);
	MTX_SET_FREE(v_yp0);
	MTX_SET_FREE(m_dfdy);
	MTX_SET_FREE(v_fac);
	MTX_SET_FREE(m_difU);
	MTX_SET_FREE(v_y);
	MTX_SET_FREE(m_lu);
	if(v_index)	free(v_index);
	MTX_SET_FREE(m_order_k);
	MTX_SET_FREE(m_dif);
	MTX_SET_FREE(m_iter);
	MTX_SET_FREE(v_psi);
	MTX_SET_FREE(v_rowscale);
	MTX_SET_FREE(v_pred);
	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(v_tmp1);
	MTX_SET_FREE(v_ynew);
	MTX_SET_FREE(v_difkp1);
	MTX_SET_FREE(v_invwt);
	MTX_SET_FREE(v_rhs);
	MTX_SET_FREE(v_del);
	return 0;
err:
	len = c_quelen(&qh_result.que);
	size = len * sizeof(double);
	sol->tt = (double *) malloc(size);
	if( sol->tt == NULL ){
		log_sys("malloc()\n");
		goto err;
	}
	size *= in->y0num;
	sol->yy = (double *) malloc(size);
	if( sol->yy == NULL ){
		log_sys("malloc()\n");
		goto err;
	}
	for( i = 0 ; i < len ; i++ ){
		que = (que_result_t *) c_deque( &qh_result.que );
		sol->tt[i] = que->t;
		for( j = 0 ; j < in->y0num ; j++ ){
			sol->yy[i * in->y0num + j] = que->y[j];
		}
		_free_que(que);
	}
	sol->tnum = len;

	MTX_SET_FREE(m_psidmdy);
	MTX_SET_FREE(v_yinterp);
	MTX_SET_FREE(m_mtnew);
	MTX_SET_FREE(v_yp);
	MTX_SET_FREE(v_f0);
	MTX_SET_FREE(v_threshold);
	MTX_SET_FREE(v_jthreshold);
	MTX_SET_FREE(m_mass);
	MTX_SET_FREE(v_yp0);
	MTX_SET_FREE(m_dfdy);
	MTX_SET_FREE(v_fac);
	MTX_SET_FREE(m_difU);
	MTX_SET_FREE(v_y);
	MTX_SET_FREE(m_lu);
	if(v_index)	free(v_index);
	MTX_SET_FREE(m_order_k);
	MTX_SET_FREE(m_dif);
	MTX_SET_FREE(m_iter);
	MTX_SET_FREE(v_psi);
	MTX_SET_FREE(v_rowscale);
	MTX_SET_FREE(v_pred);
	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(v_tmp1);
	MTX_SET_FREE(v_ynew);
	MTX_SET_FREE(v_difkp1);
	MTX_SET_FREE(v_invwt);
	MTX_SET_FREE(v_rhs);
	MTX_SET_FREE(v_del);
	for( i = len - 1 ; i >= 0 ; i-- ){
		que = (que_result_t *) c_deque( &qh_result.que );
		_free_que(que);
	}
	log_sys("ERROR: ndf_solve()\n");
	return -1;
}

#if 0
/*
 * ְ¸μ̥ޥȥåξDAE
 */
static int _daeic_state_dependent();
{
	return 0;
}
#endif
/*
 * ְ¸μ̥ޥȥåξDAE
 */
static int _daeic_state_independent(ndf_input_t *in, ndf_option_t *opt, m_matrix_t *m_m0, int ictype, double t0, m_matrix_t *v_y0, m_matrix_t *v_yp0, m_matrix_t *v_f0, m_matrix_t *v_threshold, m_matrix_t *v_fac, m_matrix_t *m_dfdy0, int (*odefunc)(double t, double *y, double *f), ndf_status_t *stats)
{
	int		size;

	m_matrix_t	m_u, m_v, v_w;
	m_matrix_t	v_diag, v_tmp;
	m_matrix_t	m_tu, m_tv;
	m_index_t	*alg_index, *dif_index;
	int		nalg_index, ndif_index;
	double		tol;

	m_matrix_t	v_f, m_dfdy, m_tmp, v_y, v_yp;
	m_matrix_t	m_j;
	int		need_j, n_nz;
	double		cond_num;
	m_matrix_t	v_subf;
	double		norm_f, norm_subf;

	m_matrix_t	m_lu;
	int		*lu_idx = NULL;

	m_matrix_t	v_dely, v_ynew;
	m_matrix_t	v_subtmp, v_suby;
	m_matrix_t	v_vynew, v_fnew, v_tufnew;
	int		i, probe;
	double		res, lambda;
	m_matrix_t	v_dsubf, v_dsubd;
	double		norm_subfnew, norm_fnew;
	double		resnew;
	double		norm_y, norm_ynew, ynorm;


	memset(&m_u, 0, sizeof(m_matrix_t));
	memset(&m_v, 0, sizeof(m_matrix_t));
	memset(&v_w, 0, sizeof(m_matrix_t));
	memset(&v_diag, 0, sizeof(m_matrix_t));
	memset(&v_tmp, 0, sizeof(m_matrix_t));
	memset(&m_tu, 0, sizeof(m_matrix_t));
	memset(&m_tv, 0, sizeof(m_matrix_t));
	memset(&v_f, 0, sizeof(m_matrix_t));
	memset(&m_tmp, 0, sizeof(m_matrix_t));
	memset(&m_dfdy, 0, sizeof(m_matrix_t));
	memset(&v_y, 0, sizeof(m_matrix_t));
	memset(&v_yp, 0, sizeof(m_matrix_t));
	memset(&m_j, 0, sizeof(m_matrix_t));
	memset(&v_subf, 0, sizeof(m_matrix_t));
	memset(&m_lu, 0, sizeof(m_matrix_t));
	memset(&v_dely, 0, sizeof(m_matrix_t));
	memset(&v_ynew, 0, sizeof(m_matrix_t));
	memset(&v_subtmp, 0, sizeof(m_matrix_t));
	memset(&v_suby, 0, sizeof(m_matrix_t));
	memset(&v_vynew, 0, sizeof(m_matrix_t));
	memset(&v_fnew, 0, sizeof(m_matrix_t));
	memset(&v_tufnew, 0, sizeof(m_matrix_t));
	memset(&v_dsubf, 0, sizeof(m_matrix_t));
	memset(&v_dsubd, 0, sizeof(m_matrix_t));

	/* df/dy */
	if( opt->jac.jconst != NULL ){
		size = m_dfdy0->row * m_dfdy0->col * sizeof(double);
		memcpy(m_dfdy0->elem, opt->jac.jconst, size);
	}else if( opt->jac.jfunc != NULL ){
		if( opt->jac.jfunc(t0, v_y0->elem, m_dfdy0->elem) < 0 ){
			goto err;
		}
	}else{	/* Numerial Jacobian */
		if( _numerical_jacobian(t0, v_y0, v_f0,
		odefunc, v_threshold, 1, v_fac, m_dfdy0, stats) < 0 ){
			log_sys("_numerical_jacobian()\n");
			goto err;
		}
		stats->npds++;
	}

	/* M */
	MTX_SET_ALLOC(m_u, m_m0->row, m_m0->col);
	MTX_SET_ALLOC(m_v, m_m0->row, m_m0->col);
	MTX_SET_ALLOC(v_w, m_m0->row, 1);
	if( m_u.elem == NULL || m_v.elem == NULL || v_w.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_diag, m_m0->row, 1);
	MTX_SET_ALLOC(v_tmp, m_m0->row, 1);
	if( v_diag.elem == NULL || v_tmp.elem == NULL ){
		goto err;
	}

	if( ictype == IC_DIAGONAL ){
		if( mtx_diagonal2vector(0, m_m0, &v_diag) < 0 ){
			goto err;
		}
		if( mtx_issparse(m_m0) ){
			/* not support */
			goto err;
		}else{	/* եñ̹ */
			mtx_unit(&m_u);
			mtx_unit(&m_v);
		}
		nalg_index = mtx_index(&v_diag, MTX_EQ, 0., &alg_index);
		if( nalg_index < 0 ){
			goto err;
		}
	}else{
		if( mtx_svdecomp(m_m0, &m_u, &v_w, &m_v) < 0 ){
			goto err;
		}
		memcpy(v_diag.elem, v_w.elem, m_m0->row * sizeof(double));
		mtx_max(&v_diag, &v_tmp);
		tol = in->y0num * v_tmp.elem[0] * NDF_EPSILON;
		nalg_index = mtx_index(&v_diag, MTX_LESSEQ, tol, &alg_index);
		if( nalg_index < 0 ){
			goto err;
		}
		mtx_set_elem(0., nalg_index, alg_index, &v_diag);
	}

	ndif_index = mtx_index(&v_diag, MTX_NOTEQ, 0., &dif_index);
	if( ndif_index < 0 ){
		goto err;
	}
	/* žϹ U', V' */
	MTX_SET_ALLOC(m_tu, m_m0->col, m_m0->row);
	MTX_SET_ALLOC(m_tv, m_m0->col, m_m0->row);
	if( m_tu.elem == NULL || m_tv.elem == NULL ){
		goto err;
	}
	mtx_transpose(&m_u, &m_tu);
	mtx_transpose(&m_v, &m_tv);
	/* F, DFDY, Y, yp */
	MTX_SET_ALLOC(v_f, in->y0num, 1);
	MTX_SET_ALLOC(m_tmp, in->y0num, in->y0num);
	MTX_SET_ALLOC(m_dfdy, in->y0num, in->y0num);
	MTX_SET_ALLOC(v_y, in->y0num, 1);
	MTX_SET_ALLOC(v_yp, in->y0num, 1);
	if( v_f.elem == NULL || m_tmp.elem == NULL || m_dfdy.elem == NULL
	 || v_y.elem == NULL || v_yp.elem == NULL ){
		goto err;
	}
	/* F = U' * f */
	mtx_multiply(&m_tu, v_f0, &v_f);
	/* DFDY = U' * DfDy * V */
	mtx_multiply(&m_tu, m_dfdy0, &m_tmp);
	mtx_multiply(&m_tmp, &m_v, &m_dfdy);
	/* Y = V' * y */
	mtx_multiply(&m_tv, v_y0, &v_y);
	/* yp = V' * yp0 */
	mtx_multiply(&m_tv, v_yp0, &v_yp);

	if( nalg_index == 0 ){
		/*
		 * mass.singular = yes λΤߵ롣
		 * DAEǤϤʤ
		 * yp = VM * (F ./ D)
		 */
		mtx_elem_divide(&v_f, &v_diag, &v_yp);
		mtx_multiply(&m_v, &v_yp, v_yp0);
		goto end;
	}

	/* J = DFDY(AlgVar, AlgVar) */
	MTX_SET_ALLOC(m_j, nalg_index, nalg_index);
	if( m_j.elem == NULL ){
		goto err;
	}
	if( mtx_get_submatrix_by_index(&m_dfdy, &m_j,
				nalg_index, alg_index) < 0 ){
		goto err;
	}
	need_j = 0;
	n_nz = mtx_nonzero(&m_j);
	cond_num = mtx_est_condition_number(&m_j); /* 2.9659e+007; */
DBGW(30, ("_daeic_state_independent set J n_nz[%d] cond_num[%e]\n", n_nz, cond_num));
	if( n_nz == 0 || NDF_EPSILON * n_nz * cond_num > 1 ){
		log_sys("This DAE appears to be of index greater than 1.\n");
		goto err;
	}

	/*
	 * ̵̷ͤå
	 */
	MTX_SET_ALLOC(v_subf, nalg_index, 1);
	if( v_subf.elem == NULL ){
		goto err;
	}
	if( mtx_get_submatrix_by_index(&v_f, &v_subf,
				nalg_index, alg_index) < 0 ){
		goto err;
	}
	if( mtx_norm(&v_f, 2, &norm_f) < 0 ){
		log_sys("mtx_norm(v_f)\n");
		goto err;
	}
	if( mtx_norm(&v_subf, 2, &norm_subf) < 0 ){
		log_sys("mtx_norm(v_subf)\n");
		goto err;
	}
	if( norm_subf <= 1000 * NDF_EPSILON * norm_f ){
		if( _set_yp(&v_f, &v_diag, &m_v,
				ndif_index, dif_index, v_yp0) < 0 ){
			goto err;
		}
		goto end;
	}


	/*
	 * ˥塼ȥȿˡ
	 */
	MTX_SET_ALLOC(m_lu, nalg_index, nalg_index);
	if( m_lu.elem == NULL ){
		goto err;
	}
	lu_idx = (int *) malloc(m_j.row * sizeof(int));
	if( lu_idx == NULL ){
		goto err;
	}
	memset(lu_idx, 0, m_j.row * sizeof(int));
	if( mtx_ludecomp(&m_j, &m_lu, lu_idx) < 0 ){
		goto err;
	}

	MTX_SET_ALLOC(v_dely, nalg_index, 1);
	if( v_dely.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_ynew, v_y.row, v_y.col);
	if( v_ynew.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_subtmp, nalg_index, 1);
	if( v_subtmp.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_suby, nalg_index, 1);
	if( v_suby.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_vynew, in->y0num, 1);
	if( v_vynew.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_fnew, in->y0num, 1);
	if( v_fnew.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_tufnew, in->y0num, 1);
	if( v_tufnew.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_dsubf, ndif_index, 1);
	if( v_dsubf.elem == NULL ){
		goto err;
	}
	MTX_SET_ALLOC(v_dsubd, ndif_index, 1);
	if( v_dsubd.elem == NULL ){
		goto err;
	}

	for( i = 0 ; i < 15 ; i++ ){
		/* df/dy */
		if( need_j ){
			if( opt->jac.jconst != NULL ){
				/* do Nothing */
			}else if( opt->jac.jfunc != NULL ){
				if( opt->jac.jfunc(t0, v_y0->elem,
							m_dfdy0->elem) < 0 ){
					goto err;
				}
			}else{	/* Numerial Jacobian */
				if( _numerical_jacobian(t0, v_y0, v_f0,
				odefunc, v_threshold, 0, v_fac,
				m_dfdy0, stats) < 0 ){
					log_sys("_numerical_jacobian()\n");
					goto err;
				}
				stats->npds++;
			}
			/* DFDY = U' * DfDy * V */
			mtx_multiply(&m_tu, m_dfdy0, &m_tmp);
			mtx_multiply(&m_tmp, &m_v, &m_dfdy);
			/* J = DFDY(AlgVar, AlgVar) */
			if( mtx_get_submatrix_by_index(&m_dfdy, &m_j,
						nalg_index, alg_index) < 0 ){
				goto err;
			}
			need_j = 0;
			if( mtx_ludecomp(&m_j, &m_lu, lu_idx) < 0 ){
				goto err;
			}
		}

		/*
		 * Estimate the error
		 * delY = - (U \ (L \ F(AlgVar)))
		 * res = norm(delY)
		 */
		if( mtx_get_submatrix_by_index(&v_f, &v_subf,
					nalg_index, alg_index) < 0 ){
			goto err;
		}
		if( mtx_lusolve(&m_lu, lu_idx, &v_subf, &v_dely) < 0 ){
			goto err;
		}
		mtx_scalar_multiply(&v_dely, -1., &v_dely);
		mtx_norm(&v_dely, 2, &res);

		/*
		 * Weak line search with affine invariant test.
		 */
		lambda = 1;
		/* Ynew = Y */
		size = v_y.row * v_y.col * sizeof(double);
		memcpy(v_ynew.elem, v_y.elem, size);
		for( probe = 0 ; probe < 3 ; probe++ ){
			/* Ynew(AlgVar) = Y(AlgVar) + lambda * delY */
			mtx_scalar_multiply(&v_dely, lambda, &v_subtmp);
			if( mtx_get_submatrix_by_index(&v_y, &v_suby,
						nalg_index, alg_index) < 0 ){
				log_sys("mtx_get_submatrix_by_index() ERROR\n");
				goto err;
			}
			mtx_addup(&v_suby, &v_subtmp, &v_subtmp);
			if( mtx_set_submatrix_by_index(&v_subtmp, &v_ynew,
						nalg_index, alg_index) < 0 ){
				log_sys("mtx_set_submatrix_by_index() ERROR\n");
				goto err;
			}
			/* ynew = V * Ynew */
			mtx_multiply(&m_v, &v_ynew, &v_vynew);
			/* fnew = odefunc(t0, ynew) */
			odefunc(t0, v_vynew.elem, v_fnew.elem);
			/* Fnew = U' * fnew */
			mtx_multiply(&m_tu, &v_fnew, &v_tufnew);

			/* norm(Fnew(AlgVar)) */
			if( mtx_get_submatrix_by_index(&v_tufnew, &v_subf,
						nalg_index, alg_index) < 0 ){
				goto err;
			}
			mtx_norm(&v_subf, 2, &norm_subfnew);

			/* norm(Fnew) */
			mtx_norm(&v_tufnew, 2, &norm_fnew);

			if( norm_subfnew <= 1e-3 * opt->tol.rel * norm_fnew ){
				/* y = ynew, f = fnew */
				size = v_y0->row * v_y0->col * sizeof(double);
				memcpy(v_y0->elem, v_vynew.elem, size);
				memcpy(v_f0->elem, v_fnew.elem, size);
				/*
				 * yp(DifVar) = Fnew(DifVar) ./ D(DifVar)
				 * yp = V * yp
				 */
				if( _set_yp(&v_tufnew, &v_diag, &m_v,
					ndif_index, dif_index, v_yp0) < 0 ){
					goto err;
				}
				goto end;
			}

			/*
			 * Estimate the error of ynew
			 */
			if( mtx_lusolve(&m_lu, lu_idx, &v_subf, &v_subtmp) < 0 ){
				goto err;
			}
			mtx_norm(&v_subtmp, 2, &resnew);
			if( resnew < 0.9 * res ){
				break;
			}else{
				lambda *= 0.5;
			}
		}

		/* norm(Y(AlgVar)) */
		if( mtx_get_submatrix_by_index(&v_y, &v_suby,
					nalg_index, alg_index) < 0 ){
			goto err;
		}
		mtx_norm(&v_suby, 2, &norm_y);
		/* norm(Ynew(AlgVar)) */
		if( mtx_get_submatrix_by_index(&v_ynew, &v_suby,
					nalg_index, alg_index) < 0 ){
			goto err;
		}
		mtx_norm(&v_subf, 2, &norm_ynew);

		ynorm = MAX(norm_y, norm_ynew);
		if( ynorm == 0 ){
			ynorm = NDF_EPSILON;
		}
		/* Y = Ynew, y = V * Ynew */
		size = v_y0->row * v_y0->col * sizeof(double);
		memcpy(v_y.elem, v_ynew.elem, size);
		mtx_multiply(&m_v, &v_ynew, v_y0);
		/* f = fnew, F = Fnew */
		memcpy(v_f0->elem, v_fnew.elem, size);
		memcpy(v_f.elem, v_tufnew.elem, size);
		if( resnew  <= 1e-3 * (opt->tol.rel) * ynorm ){
			/*
			 * yp(DifVar) = Fnew(DifVar) ./ D(DifVar)
			 * yp = V * yp
			 */
			if( _set_yp(&v_f, &v_diag, &m_v,
					ndif_index, dif_index, v_yp0) < 0 ){
				goto err;
			}
			goto end;
		}
		need_j = (resnew > (0.1 * res));
	}

end:
	MTX_SET_FREE(m_u);
	MTX_SET_FREE(m_v);
	MTX_SET_FREE(v_w);
	MTX_SET_FREE(v_diag);
	MTX_SET_FREE(m_tu);
	MTX_SET_FREE(m_tv);
	MTX_SET_FREE(v_f);
	MTX_SET_FREE(m_dfdy);
	MTX_SET_FREE(v_y);
	MTX_SET_FREE(v_yp);
	MTX_SET_FREE(m_j);
	MTX_SET_FREE(v_subf);
	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(m_tmp);
	MTX_SET_FREE(m_lu);
	free(lu_idx);
	MTX_SET_FREE(v_dely);
	MTX_SET_FREE(v_ynew);
	MTX_SET_FREE(v_subtmp);
	MTX_SET_FREE(v_suby);
	MTX_SET_FREE(v_vynew);
	MTX_SET_FREE(v_fnew);
	MTX_SET_FREE(v_tufnew);
	MTX_SET_FREE(v_dsubf);
	MTX_SET_FREE(v_dsubd);
	mtx_index_free(alg_index);
	mtx_index_free(dif_index);
	return 0;
err:
	MTX_SET_FREE(m_u);
	MTX_SET_FREE(m_v);
	MTX_SET_FREE(v_w);
	MTX_SET_FREE(v_diag);
	MTX_SET_FREE(m_tu);
	MTX_SET_FREE(m_tv);
	MTX_SET_FREE(v_f);
	MTX_SET_FREE(m_dfdy);
	MTX_SET_FREE(v_y);
	MTX_SET_FREE(v_yp);
	MTX_SET_FREE(m_j);
	MTX_SET_FREE(v_subf);
	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(m_tmp);
	MTX_SET_FREE(m_lu);
	if(lu_idx)	free(lu_idx);
	MTX_SET_FREE(v_dely);
	MTX_SET_FREE(v_ynew);
	MTX_SET_FREE(v_subtmp);
	MTX_SET_FREE(v_suby);
	MTX_SET_FREE(v_vynew);
	MTX_SET_FREE(v_fnew);
	MTX_SET_FREE(v_tufnew);
	MTX_SET_FREE(v_dsubf);
	MTX_SET_FREE(v_dsubd);
	if(alg_index)	mtx_index_free(alg_index);
	if(dif_index)	mtx_index_free(dif_index);
	log_sys("ERROR: _daeic_state_independent()\n");
	return -1;
}

/*
 * yp = dy/dt ꡣ
 */
static int _set_yp(m_matrix_t *v_f, m_matrix_t *v_d, m_matrix_t *m_v, int ndif_index, m_index_t *dif_index, m_matrix_t *v_yp0)
{
	int	size;
	m_matrix_t	v_subf, v_subd, v_yp;

	memset(&v_yp, 0, sizeof(m_matrix_t));
	memset(&v_subf, 0, sizeof(m_matrix_t));
	memset(&v_subd, 0, sizeof(m_matrix_t));
	MTX_SET_ALLOC(v_yp, v_yp0->row, v_yp0->col);
	MTX_SET_ALLOC(v_subf, ndif_index, 1);
	MTX_SET_ALLOC(v_subd, ndif_index, 1);
	if( v_yp.elem == NULL || v_subf.elem == NULL || v_subd.elem == NULL ){
		goto err;
	}
	/* yp0 -> yp */
	size = v_yp0->row * v_yp0->col * sizeof(double);
	memcpy(v_yp.elem, v_yp0->elem, size);

	/* F(DifVar) */
	if( mtx_get_submatrix_by_index(v_f, &v_subf,
				ndif_index, dif_index) < 0 ){
		goto err;
	}
	/* D(DifVar) */
	if( mtx_get_submatrix_by_index(v_d, &v_subd,
				ndif_index, dif_index) < 0 ){
		goto err;
	}
	/* yp(DifVar) = F(DifVar) ./ D(DifVar) */
	mtx_elem_divide(&v_subf, &v_subd, &v_subf);
	if( mtx_set_submatrix_by_index(&v_subf, &v_yp,
				ndif_index, dif_index) < 0 ){
		goto err;
	}

	/* yp = V * yp */
	mtx_multiply(m_v, &v_yp, v_yp0);


	MTX_SET_FREE(v_yp);
	MTX_SET_FREE(v_subf);
	MTX_SET_FREE(v_subd);
	return 0;
err:
	MTX_SET_FREE(v_yp);
	MTX_SET_FREE(v_subf);
	MTX_SET_FREE(v_subd);
	log_sys("ERROR: _set_yp()\n");
	return -1;
}

/*
 * 䥳ӥοͻС(Not support)
 */
static int _numerical_jacobian(double t, m_matrix_t *v_y, m_matrix_t *v_f, int (*odefunc)(double t, double *y, double *f), m_matrix_t *v_threshold, int isempty, m_matrix_t *v_fac, m_matrix_t *m_dfdy, ndf_status_t *stats)
{
	static double	br, bl, bu, facmin, facmax;
	int	ny, nf;

	m_matrix_t	v_yscale, v_del;
	m_index_t	*index = NULL;
	int		nindex;
	int		i, j, k, max;
	m_matrix_t	v_sign;

	m_matrix_t	m_ydel, m_fdel, m_diag;
	m_matrix_t	v_ytmp, v_ftmp;
	m_matrix_t	m_fdiff;
	m_matrix_t	m_fdiftmp, v_difmax;
	int		*rowmax = NULL;
	m_matrix_t	v_absFdelRm;

	m_matrix_t	v_absFty, v_absFtyRm;
	int		*jarray = NULL;
	int		anyJ;

	m_matrix_t	v_ydel, v_fscale, v_fdel;
	int		k1, isk;
	m_matrix_t	v_fdiff;
	m_matrix_t	v_fdiftmp, s_difmax;
	m_matrix_t	v_dfdyk;
	int		rowmax0[1];
	double		tmpfac, del;
	double		norm_tmp, norm_dfdyk;

	double		fscale;

DBGW(30, ("_numerical_jacobian() start\n"));
	memset(&v_yscale, 0, sizeof(m_matrix_t));
	memset(&v_del, 0, sizeof(m_matrix_t));
	memset(&v_sign, 0, sizeof(m_matrix_t));
	memset(&m_ydel, 0, sizeof(m_matrix_t));
	memset(&m_diag, 0, sizeof(m_matrix_t));
	memset(&m_fdel, 0, sizeof(m_matrix_t));
	memset(&v_ytmp, 0, sizeof(m_matrix_t));
	memset(&v_ftmp, 0, sizeof(m_matrix_t));
	memset(&m_fdiff, 0, sizeof(m_matrix_t));
	memset(&m_fdiftmp, 0, sizeof(m_matrix_t));
	memset(&v_difmax, 0, sizeof(m_matrix_t));
	memset(&v_absFdelRm, 0, sizeof(m_matrix_t));
	memset(&v_absFty, 0, sizeof(m_matrix_t));
	memset(&v_absFtyRm, 0, sizeof(m_matrix_t));
	memset(&v_ydel, 0, sizeof(m_matrix_t));
	memset(&v_fscale, 0, sizeof(m_matrix_t));
	memset(&v_fdel, 0, sizeof(m_matrix_t));
	memset(&v_fdiff, 0, sizeof(m_matrix_t));
	memset(&v_fdiftmp, 0, sizeof(m_matrix_t));
	memset(&s_difmax, 0, sizeof(m_matrix_t));
	memset(&v_dfdyk, 0, sizeof(m_matrix_t));

	/*
	 * 
	 */
	if( br == 0 )	br = pow(NDF_EPSILON, 0.875);
	if( bl == 0 )	bl = pow(NDF_EPSILON, 0.75);
	if( bu == 0 )	bu = pow(NDF_EPSILON, 0.25);
	if( facmin == 0 )	facmin = pow(NDF_EPSILON, 0.78);
	if( facmax == 0 )	facmax = 0.1;
	ny = v_y->row * v_y->col;
	nf = v_f->row * v_f->col;
	if( isempty ){
		mtx_unique(sqrt(NDF_EPSILON), v_fac);
	}
DBGW(30, ("_numerical_jacobian() br[%e] bl[%e] bu[%e]\n", br, bl, bu));
DBGW(30, ("_numerical_jacobian() facmin[%e] facmax[%e]\n", facmin, facmax));
DBGW(30, ("_numerical_jacobian() ny[%d] nf[%d]\n", ny, nf));

	/*
	 * dfdy  j ؤκʬФʬ del 򤹤롣
	 * ٥ȥ fac  _numerical_jacobian() ؿθƤӽФ
	 * и롣
	 */
	MTX_SET_ALLOC(v_yscale, v_y->row, v_y->col);
	MTX_SET_ALLOC(v_del, v_y->row, v_y->col);
	if( v_yscale.elem == NULL || v_del.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	/*
	 * yscale = max(abs(y), thresh);
	 * del = (y + fac .* yscale) - y;
	 */
	mtx_abs(v_y, &v_yscale);
	if( mtx_max2(&v_yscale, v_threshold, &v_yscale) < 0 ){
		goto err;
	}
	mtx_elem_multiply(v_fac, &v_yscale, &v_del);
	mtx_addup(&v_del, v_y, &v_del);
	mtx_subtract(&v_del, v_y, &v_del);
	nindex = mtx_index(&v_del, MTX_EQ, 0.0, &index);
	if( nindex < 0 ){
		goto err;
	}
	for( i = 0 ; i < nindex ; i++ ){
		j = (index[i].i > 0)? index[i].i : index[i].j;
		while( 1 ){
			/* if( v_fac->elem[j] < facmax ){} */
			if( NDF_ISLT(v_fac->elem[j], facmax) ){
				v_fac->elem[j] = MIN(100 * v_fac->elem[j], facmax);
				v_del.elem[j] = (v_y->elem[j] + v_fac->elem[j] * v_yscale.elem[j]) - v_y->elem[j];
				/* if( v_del.elem[j] ){} */
				if( NDF_ISNOTEQ(v_del.elem[j], 0.0) ){
					break;
				}
			}else{
				v_del.elem[j] = v_threshold->elem[j];
				break;
			}
		}
	}
	if( nf == ny ){
		MTX_SET_ALLOC(v_sign, v_y->row, v_y->col);
		if( v_sign.elem == NULL ){
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		max = v_y->row * v_y->col;
		/*
		 * del ΰؤ褦ݤġ
		 * s = (sign(fty) >= 0);
		 * del = (s - (~s)) .* abs(del);
		 */
		for( i = 0 ; i < max ; i++ ){
			/* if( v_f->elem[i] < 0 ){} */
			if( NDF_ISLT(v_f->elem[i], 0) ){
				v_sign.elem[i] = -1;
			}else{
				v_sign.elem[i] = 1;
			}
		}
		mtx_abs(&v_del, &v_del);
		mtx_elem_multiply(&v_sign, &v_del, &v_del);
		MTX_SET_FREE(v_sign);
	}
mtx_dump(50, "NumJac Fyt:", v_f);
mtx_dump(50, "NumJac yscale:", &v_yscale);
mtx_dump(50, "NumJac fac:", v_fac);
mtx_dump(50, "NumJac del:", &v_del);

	/*
	 * dfdy ؤκʬ롣
	 */
	MTX_SET_ALLOC(m_ydel, ny, ny);
	MTX_SET_ALLOC(m_diag, ny, ny);
	MTX_SET_ALLOC(m_fdel, nf, ny);
	if( m_ydel.elem == NULL || m_diag.elem == NULL || m_fdel.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	/*
	 * ydel = y(:,ones(1,ny)) + diag(del);
	 */
	for( j = 0 ; j < ny ; j++ ){
		mtx_set_submatrix(v_y, &m_ydel, 0, j);
	}
	mtx_diagonal(&v_del, &m_diag);
	mtx_addup(&m_ydel, &m_diag, &m_ydel);
mtx_dump(50, "NumJac ydel:", &m_ydel);
	/*
	 * fdel = zeros(nf,ny);
	 */
	MTX_SET_ALLOC(v_ytmp, ny, 1);
	MTX_SET_ALLOC(v_ftmp, nf, 1);
	if( v_ytmp.elem == NULL || v_ftmp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	mtx_unique(0.0, &m_fdel);
	for( j = 0 ; j < ny ; j++ ){
		/*
		 * fdel(:,j) = feval(F,t,ydel(:,j),args{:});
		 */
		mtx_get_submatrix(&m_ydel, &v_ytmp, 0, j);
		odefunc(t, v_ytmp.elem, v_ftmp.elem);
		mtx_set_submatrix(&v_ftmp, &m_fdel, 0, j);
	}
mtx_dump(50, "NumJac Fdel:", &m_fdel);
	/*
	 * Fdiff = Fdel - Fty(:,ones(1,ny));
	 * dFdy = Fdiff * diag(1./del);
	 */
	MTX_SET_ALLOC(m_fdiff, nf, ny);
	if( m_fdiff.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	for( j = 0 ; j < ny ; j++ ){
		mtx_set_submatrix(v_f, &m_fdiff, 0, j);
	}
mtx_dump(50, "NumJac Fty(:,ones(1,ny)):", &m_fdiff);
	mtx_subtract(&m_fdel, &m_fdiff, &m_fdiff);
	mtx_elem_divide_into_scalar(1.0, &v_del, &v_ytmp);
	mtx_diagonal(&v_ytmp, &m_diag);
	mtx_multiply(&m_fdiff, &m_diag, m_dfdy);
mtx_dump(50, "NumJac diag(1./del):", &m_diag);
mtx_dump(50, "NumJac Fdiff:", &m_fdiff);
	/*
	 * [Difmax,Rowmax] = max(abs(fdiff),[],1);
	 */
	MTX_SET_ALLOC(v_difmax, ny, 1);
	MTX_SET_ALLOC(m_fdiftmp, nf, ny);
	if( v_difmax.elem == NULL || m_fdiftmp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	rowmax = (int *) malloc(ny * sizeof(int));
	if( rowmax == NULL ){
		log_sys("malloc()\n");
		goto err;
	}
	mtx_abs(&m_fdiff, &m_fdiftmp);
	mtx_max_index(&m_fdiftmp, &v_difmax, rowmax);
	/*
	 * if fdel is a column vector, then index is a scalar,
	 * so indexing is okay.
	 * absFdelRm = abs(fdel((0:ny-1)*nF + Rowmax));
	 */
	MTX_SET_ALLOC(v_absFdelRm, ny, 1);
	if( v_absFdelRm.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	for( i = 0 ; i < ny ; i++ ){
		v_absFdelRm.elem[i] = fabs(m_fdel.elem[i * nf + rowmax[i]]);
	}

	/*
	 * _numerical_jacobian() ؿμθƤӽФΤ
	 * ٥ȥ fac 롣
	 */
	/*
	 * absFty = abs(Fty);
	 * absFtyRm = absFty(Rowmax);	not a column vector if absFty scalar
	 * absFty = absFtyRm(:)';	ensure that absFtyRm is a row vector
	 */
	MTX_SET_ALLOC(v_absFty, nf, 1);
	MTX_SET_ALLOC(v_absFtyRm, 1, ny);	/* row vector */
	if( v_absFty.elem == NULL || v_absFtyRm.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	mtx_abs(v_f, &v_absFty);
	for( i = 0 ; i < ny ; i++ ){
		v_absFtyRm.elem[i] = v_absFty.elem[rowmax[i]];
	}

	/*
	 * j = ((absFdelRm ~= 0) & (absFtyRm ~= 0)) | (Difmax == 0);
	 */
	jarray = (int *) malloc(ny * sizeof(int));
	if( jarray == NULL ){
		log_sys("malloc()\n");
		goto err;
	}
	anyJ = 0;
	for( i = 0 ; i < ny ; i++ ){
		/* jarray[i] = ((v_absFdelRm.elem[i] != 0) && (v_absFtyRm.elem[i] != 0)) || (v_difmax.elem[i] == 0); */
		jarray[i] = (NDF_ISNOTEQ(v_absFdelRm.elem[i], 0.0) && NDF_ISNOTEQ(v_absFtyRm.elem[i], 0.0)) || NDF_ISEQ(v_difmax.elem[i], 0.0);
		if( jarray[i] != 0 ){
			anyJ = 1;
		}
DBGW(50, ("jarray[%d]: %d\n", i, jarray[i]));
	}
DBGW(50, ("anyJ[%d]\n", anyJ));
	if( anyJ ){
		/*
		 * ydel = y;
		 * Fscale = max(absFdelRm,absFtyRm);
		 */
		MTX_SET_ALLOC(v_ydel, ny, 1);
		MTX_SET_ALLOC(v_fscale, nf, 1);
		MTX_SET_ALLOC(v_fdel, nf, 1);
		if( v_ydel.elem == NULL
		 || v_fscale.elem == NULL || v_fdel.elem == NULL ){
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		memcpy(v_ydel.elem, v_y->elem, ny * sizeof(double));
		mtx_max2(&v_absFdelRm, &v_absFtyRm, &v_fscale);

		/*
		 * If the differecen in f values is so small
		 * that the column might be just roundoff error,
		 * try a bigger increment.
		 */
		/*
		 * Difmax and Fscale might be zero
		 * k1 = (Difmax <= br * Fscale);
		 * for k = find(j & k1)
		 */
		MTX_SET_ALLOC(v_fdiff, nf, 1);
		MTX_SET_ALLOC(v_fdiftmp, nf, 1);
		MTX_SET_ALLOC(s_difmax, 1, 1);
		MTX_SET_ALLOC(v_dfdyk, nf, 1);
		if( v_fdiff.elem == NULL || v_fdiftmp.elem == NULL
		 || s_difmax.elem == NULL || v_dfdyk.elem == NULL ){
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		for( k = 0 ; k < ny ; k++ ){
			/* k1 = v_difmax.elem[k] <= br * v_fscale.elem[k]; */
			k1 = NDF_ISLTEQ(v_difmax.elem[k], br * v_fscale.elem[k]);
			isk = jarray[k] && k1;
DBGW(50, ("k[%d]: %d\n", k, isk));
			if( !isk )	continue;
			/*
			 * tmpfac = min(sqrt(fac(k)),facmax);
			 * del = (y(k) + tmpfac * yscale(k)) - y(k);
			 */
			tmpfac = MIN(sqrt(v_fac->elem[k]), facmax);
			del = (v_y->elem[k] + tmpfac * v_yscale.elem[k])
					- v_y->elem[k];
DBGW(50, ("k[%d]: tmpfac[%e] del[%e]\n", k, tmpfac, del));
			/* if( !((tmpfac != v_fac->elem[k]) && (del != 0)) ){} */
			if( !(NDF_ISNOTEQ(tmpfac, v_fac->elem[k]) && NDF_ISNOTEQ(del, 0)) ){
				continue;
			}
			if( nf == ny ){
				/*
				 * keep del pointing into region
				 */
				/* if( v_f->elem[k] >= 0 ){} */
				if( NDF_ISGTEQ(v_f->elem[k], 0) ){
					del = fabs(del);
				}else{
					del = -fabs(del);
				}
			}
			v_ydel.elem[k] = v_y->elem[k] + del;
			odefunc(t, v_ydel.elem, v_fdel.elem);
			v_ydel.elem[k] = v_y->elem[k];
			/*
			 * fdiff = fdel - Fty;
			 * tmp = fdiff ./ del;
			 */
			mtx_subtract(&v_fdel, v_f, &v_fdiff);
			mtx_scalar_multiply(&v_fdiff, 1./del, &v_ftmp);
			/*
			 * [difmax, rowmax] = max(abs(fdiff));
			 */
			mtx_abs(&v_fdiff, &v_fdiftmp);
			mtx_max_index(&v_fdiftmp, &s_difmax, rowmax0);
			/*
			 * norm_tmp = norm(tmp,inf)
			 * norm_dfdyk = norm(dFdy(:,k),inf);
			 */
			mtx_norm(&v_ftmp, PLUS_INF, &norm_tmp);
			mtx_get_submatrix(m_dfdy, &v_dfdyk, 0, k);
			mtx_norm(&v_dfdyk, PLUS_INF, &norm_dfdyk);
DBGW(50, ("k[%d]: norm_tmp[%e] norm_dfdyk[%e]\n", k, norm_tmp, norm_dfdyk));
			/* if( tmpfac * norm_tmp >= norm_dfdyk ){} */
			if( NDF_ISGTEQ(tmpfac * norm_tmp, norm_dfdyk) ){
				/*
				 * The new difference is more significant, so
				 * use the column computed with this increment.
				 * dFdy(:,k) = tmp;
				 */
				mtx_set_submatrix(&v_ftmp, m_dfdy, 0, k);

				/*
				 * Adjust fac for the next call to numjac.
				 * fscale = max(abs(fdel(rowmax)),absFty(rowmax));
				 */
				fscale = MAX(fabs(v_fdel.elem[rowmax0[0]]),
						v_absFty.elem[rowmax0[0]]);
DBGW(50, ("k[%d]: fscale[%e]\n", k, fscale));
				/* if( s_difmax.elem[0] <= bl * fscale ){} */
				if( NDF_ISLTEQ(s_difmax.elem[0], bl * fscale) ){
					/*
					 * The difference is small, so
					 * increase the increment.
					 */
					v_fac->elem[k] = MIN(10 * tmpfac, facmax);
				}else{
					/* if( s_difmax.elem[0] > bu * fscale ){} */
					if( NDF_ISGT(s_difmax.elem[0], bu * fscale) ){
						/*
						 * The difference is large, so
						 * reduce the increment.
						 */
						v_fac->elem[k] = MAX(0.1 * tmpfac, facmin);
					}else{
						v_fac->elem[k] = tmpfac;
					}
				}
			}
		}
mtx_dump(50, "NumJac fac:", v_fac);

		/*
		 * If the difference is small, increase the increment.
		 * k = find(j & ~k1 & (Difmax <= bl * Fscale));
		 * if ~isempty(k)
		 *	fac(k) = min(10 * fac(k), facmax);
		 * end
		 */
		for( k = 0 ; k < ny ; k++ ){
			/* k1 = v_difmax.elem[k] <= br * v_fscale.elem[k]; */
			k1 = NDF_ISLTEQ(v_difmax.elem[k], br * v_fscale.elem[k]);
			/* isk = jarray[k] && !k1 && (v_difmax.elem[k] <= bl * v_fscale.elem[k]); */
			isk = jarray[k] && !k1 && NDF_ISLTEQ(v_difmax.elem[k], bl * v_fscale.elem[k]);
			if( !isk )	continue;
			v_fac->elem[k] = MIN(10 * v_fac->elem[k], facmax);
		}
		/*
		 * If the difference is large, reduce the increment.
		 * k = find(j & (Difmax > bu * Fscale));
		 * if ~isempty(k)
		 *	fac(k) = max(0.1 * fac(k), facmin);
		 * end
		 */
		for( k = 0 ; k < ny ; k++ ){
			/* isk = jarray[k] && (v_difmax.elem[k] > bu * v_fscale.elem[k]); */
			isk = jarray[k] && NDF_ISGT(v_difmax.elem[k], bu * v_fscale.elem[k]);
			if( !isk )	continue;
			v_fac->elem[k] = MAX(0.1 * v_fac->elem[k], facmin);
		}
	}
mtx_dump(50, "NumJac fac:", v_fac);

	MTX_SET_FREE(v_yscale);
	MTX_SET_FREE(v_del);
	mtx_index_free(index);
	MTX_SET_FREE(m_ydel);
	MTX_SET_FREE(m_diag);
	MTX_SET_FREE(m_fdel);
	MTX_SET_FREE(v_ytmp);
	MTX_SET_FREE(v_ftmp);
	MTX_SET_FREE(m_fdiff);
	MTX_SET_FREE(m_fdiftmp);
	MTX_SET_FREE(v_difmax);
	free(rowmax);
	MTX_SET_FREE(v_absFdelRm);
	MTX_SET_FREE(v_absFty);
	MTX_SET_FREE(v_absFtyRm);
	free(jarray);
	MTX_SET_FREE(v_ydel);
	MTX_SET_FREE(v_fscale);
	MTX_SET_FREE(v_fdel);
	MTX_SET_FREE(v_fdiff);
	MTX_SET_FREE(v_fdiftmp);
	MTX_SET_FREE(s_difmax);
	MTX_SET_FREE(v_dfdyk);

DBGW(30, ("_numerical_jacobian() end\n"));
	return 0;
err:
	MTX_SET_FREE(v_yscale);
	MTX_SET_FREE(v_del);
	mtx_index_free(index);
	MTX_SET_FREE(m_ydel);
	MTX_SET_FREE(m_diag);
	MTX_SET_FREE(m_fdel);
	MTX_SET_FREE(v_ytmp);
	MTX_SET_FREE(v_ftmp);
	MTX_SET_FREE(m_fdiff);
	MTX_SET_FREE(m_fdiftmp);
	MTX_SET_FREE(v_difmax);
	if( rowmax )	free(rowmax);
	MTX_SET_FREE(v_absFdelRm);
	MTX_SET_FREE(v_absFty);
	MTX_SET_FREE(v_absFtyRm);
	if( jarray )	free(jarray);
	MTX_SET_FREE(v_ydel);
	MTX_SET_FREE(v_fscale);
	MTX_SET_FREE(v_fdel);
	MTX_SET_FREE(v_fdiff);
	MTX_SET_FREE(v_fdiftmp);
	MTX_SET_FREE(s_difmax);
	MTX_SET_FREE(v_dfdyk);

	log_sys("ERROR: _numerical_jacobian()\n");
	return -1;
}

/*
 * DAE򤯳ФŪ󥰤׻
 * rowscale = 1 ./ max(abs(m_iter), [], 2)
 */
static int _matrix_rowscale(m_matrix_t *v_rowscale, m_matrix_t *m_iter)
{
	int	row;
	m_matrix_t	m_tmp;

	row = v_rowscale->row;
	MTX_SET_ALLOC(m_tmp, row, row);
	if( m_tmp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	/*
	 * RowScale = 1 ./ max(abs(Miter), [], 2)
	 */
	mtx_abs(m_iter, &m_tmp);
	mtx_max_row(&m_tmp, v_rowscale);
	mtx_elem_divide_into_scalar(1, v_rowscale, v_rowscale);

	/*
	 * Miter = sparse(one2neq, one2neq, RowScale) * Miter
	 * sparse(...) -> гѹRowScale٥ȥǤгʬ
	 */
	mtx_diagonal(v_rowscale, &m_tmp);
	mtx_multiply(&m_tmp, m_iter, m_iter);

	MTX_SET_FREE(m_tmp);
	return 0;
err:
	MTX_SET_FREE(m_tmp);
	log_sys("ERROR: _matrix_rowscale()\n");
	return -1;
}

/*
 * ƥåNDFkˤѹ
 */
static int _change_stepsize(int order_k, int max_k, double h_rate, m_matrix_t *m_kJ, m_matrix_t *m_kI, m_matrix_t *m_difU, m_matrix_t *m_dif)
{
	m_matrix_t	m_difRU, m_tmp;

	memset(&m_difRU, 0, sizeof(m_matrix_t));
	memset(&m_tmp, 0, sizeof(m_matrix_t));

	MTX_SET_ALLOC(m_difRU, max_k, max_k);
	MTX_SET_ALLOC(m_tmp, max_k, max_k);
	if( m_difRU.elem == NULL || m_tmp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	/* difRU =
	 * cumprod((kI - 1 - kJ*(abs_h/prev_h)) ./ kI) * difU
	 */
	mtx_scalar_multiply(m_kJ, h_rate, &m_difRU);
	mtx_elem_subtract_from_scalar(-1, &m_difRU, &m_difRU);
	mtx_addup(m_kI, &m_difRU, &m_difRU);
	mtx_elem_divide(&m_difRU, m_kI, &m_difRU);
	mtx_cumprod(&m_difRU, 1, &m_tmp);
	if( mtx_multiply(&m_tmp, m_difU, &m_difRU) < 0 ){
		goto err;
	}

	/* dif(:,K) = dif(:,K) * difRU(K,K) */
	{
		m_matrix_t	m_tmp_dif, m_tmp_difRU, m_tmp;

		MTX_SET_ALLOC(m_tmp_dif, m_dif->row, order_k);
		if( m_tmp_dif.elem == NULL ){
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		MTX_SET_ALLOC(m_tmp_difRU, order_k, order_k);
		if( m_tmp_difRU.elem == NULL ){
			MTX_SET_FREE(m_tmp_dif);
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		MTX_SET_ALLOC(m_tmp, m_dif->row, order_k);
		if( m_tmp.elem == NULL ){
			MTX_SET_FREE(m_tmp_dif);
			MTX_SET_FREE(m_tmp_difRU);
			log_sys("MTX_SET_ALLOC()\n");
			goto err;
		}
		mtx_get_submatrix(m_dif, &m_tmp_dif, 0, 0);
		mtx_get_submatrix(&m_difRU, &m_tmp_difRU, 0, 0);
		mtx_multiply(&m_tmp_dif, &m_tmp_difRU, &m_tmp);
		mtx_set_submatrix(&m_tmp, m_dif, 0, 0);
		MTX_SET_FREE(m_tmp_dif);
		MTX_SET_FREE(m_tmp_difRU);
		MTX_SET_FREE(m_tmp);
	}

	MTX_SET_FREE(m_difRU);
	MTX_SET_FREE(m_tmp);
	return 0;
err:
	MTX_SET_FREE(m_difRU);
	MTX_SET_FREE(m_tmp);
	log_sys("ERROR: _change_stepsize()\n");
	return -1;
}

/*
 * psi = inv_g(k) * Sigma(m=1,k)(gamma(m)*Delta(m)y(n)) λС
 */
static int _get_psi(int order_k, double *inv_g, m_matrix_t *v_gamma, m_matrix_t *m_dif, m_matrix_t *v_psi)
{
	m_matrix_t	m_tmp_dif, v_tmp;
	int		gamma_row = 0;


	memset(&m_tmp_dif, 0, sizeof(m_matrix_t));
	memset(&v_tmp, 0, sizeof(m_matrix_t));

	/* dif(:,K) */
	MTX_SET_ALLOC(m_tmp_dif, m_dif->row, order_k);
	if( m_tmp_dif.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	mtx_get_submatrix(m_dif, &m_tmp_dif, 0, 0);

	/* G(K) */
	gamma_row = v_gamma->row;
	v_gamma->row = order_k;

	/* v_tmp(k*1) */
	MTX_SET_ALLOC(v_tmp, order_k, 1);
	if( v_tmp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}

	/* psi = dif(:,K) * (G(K) * inv_g(k)) */
	mtx_scalar_multiply(v_gamma, inv_g[(order_k) - 1], &v_tmp);
	mtx_multiply(&m_tmp_dif, &v_tmp, v_psi);

	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(m_tmp_dif);
	v_gamma->row = gamma_row;
	return 0;
err:
	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(m_tmp_dif);
	if( gamma_row > 0 )	v_gamma->row = gamma_row;
	log_sys("ERROR: _get_psi()\n");
	return -1;
}

/*
 *  tinterp βΤβ򤫤֤Ƶ롣
 */
static int _interpolate(double h, int k, double tnew, m_matrix_t *v_ynew, m_matrix_t *m_dif, double tinterp, m_matrix_t *v_yinterp)
{
	double	s;
	int	i;

	/*
	 * s = ((tinterp - tnew) / h)';        % may be a row vector
	 */
	s = (tinterp - tnew) / h;

	if( k == 1 ){
		m_matrix_t	v_tmp;

		MTX_SET_ALLOC(v_tmp, m_dif->row, 1);
		if( v_tmp.elem == NULL ){
			goto err;
		}
		/*
		 * yinterp = ynew(:,ones(length(tinterp),1)) + dif(:,1) * s;
		 */
		if( mtx_get_submatrix(m_dif, &v_tmp, 0, 0) < 0 ){
			MTX_SET_FREE(v_tmp);
			goto err;
		}
		mtx_scalar_multiply(&v_tmp, s, &v_tmp);
		mtx_addup(v_ynew, &v_tmp, v_yinterp);

		MTX_SET_FREE(v_tmp);
	}else{	/* cumprod collapses vectors */
		m_matrix_t	v_k, v_s, v_ktmp, v_tmp, m_dif_tmp;

		memset(&v_k, 0, sizeof(m_matrix_t));
		memset(&v_s, 0, sizeof(m_matrix_t));
		memset(&v_ktmp, 0, sizeof(m_matrix_t));
		memset(&v_tmp, 0, sizeof(m_matrix_t));
		memset(&m_dif_tmp, 0, sizeof(m_matrix_t));

		MTX_SET_ALLOC(v_k, k, 1);
		MTX_SET_ALLOC(v_s, k, 1);
		MTX_SET_ALLOC(v_ktmp, k, 1);
		MTX_SET_ALLOC(v_tmp, m_dif->row, 1);
		MTX_SET_ALLOC(m_dif_tmp, m_dif->row, k);
		if( v_k.elem == NULL || v_s.elem == NULL || v_ktmp.elem == NULL
		 || v_tmp.elem == NULL || m_dif_tmp.elem == NULL ){
			MTX_SET_FREE(v_k);
			MTX_SET_FREE(v_s);
			MTX_SET_FREE(v_ktmp);
			MTX_SET_FREE(v_tmp);
			MTX_SET_FREE(m_dif_tmp);
			goto err;
		}

		/* K = (1:k)'
		 * kI = K(:,ones(length(tinterp),1))
		 * ȤꤢK == kI
		 */
		for( i = 0 ; i < k ; i++ ){
			v_k.elem[i] = i + 1;
		}
		/*
		 * yinterp = ynew(:,ones(length(tinterp),1)) + ...
		 *	dif(:,K) * cumprod((s(ones(k,1),:)+kI-1)./kI);
		 */
		if( mtx_get_submatrix(m_dif, &m_dif_tmp, 0, 0) < 0 ){
			MTX_SET_FREE(v_k);
			MTX_SET_FREE(v_s);
			MTX_SET_FREE(v_ktmp);
			MTX_SET_FREE(v_tmp);
			MTX_SET_FREE(m_dif_tmp);
			goto err;
		}
		mtx_unique(s, &v_s);
		mtx_addup(&v_s, &v_k, &v_ktmp);
		mtx_scalar_addup(&v_ktmp, -1, &v_ktmp);
		mtx_elem_divide(&v_ktmp, &v_k, &v_ktmp);
		mtx_cumprod(&v_ktmp, 2, &v_ktmp);
		mtx_multiply(&m_dif_tmp, &v_ktmp, &v_tmp);
		mtx_addup(&v_tmp, v_ynew, v_yinterp);

		MTX_SET_FREE(v_k);
		MTX_SET_FREE(v_s);
		MTX_SET_FREE(v_ktmp);
		MTX_SET_FREE(v_tmp);
		MTX_SET_FREE(m_dif_tmp);
	}

	return 0;
err:
	log_sys("ERROR: _interpolate()\n");
	return -1;
}

/*
 * ׻̵ϿѤΥ塼롣
 */
static que_result_t *_alloc_que(double t, m_matrix_t *v_y)
{
	que_result_t	*que = NULL;
	int		size;

	que = (que_result_t *) malloc(sizeof(que_result_t));
	if( que == NULL ){
		log_sys("malloc()\n");
		goto err;
	}
	size = v_y->row * sizeof(double);
	que->y = (double *) malloc(size);
	if( que->y == NULL ){
		free(que);
		log_sys("malloc()\n");
		goto err;
	}
	que->t = t;
	memcpy(que->y, v_y->elem, size);

	return que;
err:
	log_sys("ERROR: _alloc_que()\n");
	return NULL;
}
/*
 * ׻̵ϿѤΥ塼롣
 */
static void _free_que(que_result_t *que)
{
	if(que){
		if( que->y )	free(que->y);
		free(que);
	}
}

/*
 * t + h βͽ¬
 * pred = y + sum(dif(:,K), 2)
 */
static int _predict_y(ndf_input_t *in, int order_k, m_matrix_t *m_dif, m_matrix_t *v_y, m_matrix_t *v_pred)
{
	m_matrix_t	v_tmp, m_tmp, m_tmp_dif;

	memset(&v_tmp, 0, sizeof(m_matrix_t));
	memset(&m_tmp, 0, sizeof(m_matrix_t));
	memset(&m_tmp_dif, 0, sizeof(m_matrix_t));

	MTX_SET_ALLOC(v_tmp, in->y0num, 1);
	if( v_tmp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(m_tmp, in->y0num, order_k);
	if( m_tmp.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}
	MTX_SET_ALLOC(m_tmp_dif, m_dif->row, order_k);
	if( m_tmp_dif.elem == NULL ){
		log_sys("MTX_SET_ALLOC()\n");
		goto err;
	}

	/* pred = y + sum(dif(:,K), 2) */
	mtx_get_submatrix(m_dif, &m_tmp_dif, 0, 0);
	mtx_sum(MTX_ROW, &m_tmp_dif, &v_tmp);
	mtx_addup(v_y, &v_tmp, v_pred);

	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(m_tmp);
	MTX_SET_FREE(m_tmp_dif);
	return 0;
err:
	MTX_SET_FREE(v_tmp);
	MTX_SET_FREE(m_tmp);
	MTX_SET_FREE(m_tmp_dif);
	log_sys("ERROR: _predict_y()\n");
	return -1;
}

