#include <stdio.h>
#include <stdlib.h>

#include "life.h"
#include "rs_io.h"
#include "utl_alloc.h"

typedef struct searchindex {
	int	myid;
	int search_no;
	int total_no;	/* Excluding a master node of MPI search_no = total_no */ 
} searchIndex_t;


/***** STATIC VARIABLES *****/
static double **outfileList;		/* ¸ΰ */
static searchIndex_t *sortedIndex;	/* Index */


/***** STATIC FUNCTIONS *****/

/* ॹʸ */
static char *getCurrentTimeStr();

/* Ŭ٤˱Index򥽡Ȥ */
static void sort_index(int num_search);


/***** PUBLIC FUNCTIONS *****/

/* ¸ΰȥIndexν */
int InitOutfileList(int num_search, int dim, int myid)
{
	int i;
	int rv = -1;

#ifdef DBG
	DBGW(10, ("InitOutfileList enter.\n"));
#endif /* DBG */
		
	outfileList = DoubleMatrixAlloc(num_search, dim + 1);
	sortedIndex = (searchIndex_t *)malloc(num_search * sizeof(searchIndex_t));

	if(outfileList == NULL || sortedIndex == NULL){
		goto out;
	}

	for(i = 0; i < num_search; i++){
		sortedIndex[i].myid = myid;
		sortedIndex[i].search_no = i;
		sortedIndex[i].total_no = i;
	}

	rv = 0;
out:

#ifdef DBG
	DBGW(10, ("InitOutfileList exit(%d)\n"));
#endif /* DBG */

	return rv;
}
					
/*----------------WriteInputData------------------------------------------
  *
  * ϥե˥ȥǡѥ᡼ϥǡʬ񤭽Ф
  *
  *	I:y, t_start, t_end, h, dt_save
  *	O:ʤ
  *	꥿͡ʤ
  *
  */
void WriteInputData(double y[], double t_start, double t_end, double h, double dt_save)
{
	int i, j;
	char *cdum;
#ifdef DBG
	DBGW(10, ("WriteInputData enter.\n"));
#endif /* DBG */

	fprintf(Fp_out, "##### %s #####\n\n", getCurrentTimeStr());
	fprintf(Fp_out, "%-20s:%10d\n", "Solver No.", SolverNo);
	fprintf(Fp_out, "%-20s:%10d\n", "Param Survey", F_PSurvey);
	if (Param.n_varNR > 0) {	 /* 򤯾 */
		fprintf(Fp_out, "%-20s:%10d\n", "NR_TRIAL", Nr_trial);
		fprintf(Fp_out, "%-20s:%10.3E\n", "NR_TOL_F", Nr_tol_f);
		fprintf(Fp_out, "%-20s:%10.3E\n", "NR_TOL_X", Nr_tol_x);
		fprintf(Fp_out, "%-20s:%10.5f\n", "NR_RATIO", Nr_ratio);
		fprintf(Fp_out, "%-20s:%10.5f\n", "NR_SENS_CW", Nr_sens_cw);
	}
	if (SolverNo<10) {	 /* ȯŸ׻ξ */
		fprintf(Fp_out, "%-20s:%10.3E\n", "Start Time", t_start);
		fprintf(Fp_out, "%-20s:%10.3E\n", "End Time", t_end);
		fprintf(Fp_out, "%-20s:%10.3E\n", "(initial)Time Step", h);
		fprintf(Fp_out, "%-20s:%10.3E\n", "Monitoring interval", dt_save);
		if (SolverNo == 2) {	 /* Ŭξ */
			fprintf(Fp_out, "%-20s:%10.3E\n", "RK_EPS", Rk_eps);
		}
		if (SolverNo == 3) {	 /* NDFξ */
			fprintf(Fp_out, "%-20s:%10.3E\n", "NDF_RTOL", Ndf_rtol);
			fprintf(Fp_out, "%-20s:%10.3E\n", "NDF_ATOL", Ndf_atol);
		}
	}
	fprintf(Fp_out, "%-20s:%10.3E\n", "G-Value", GeneConc);
	fprintf(Fp_out, "\n####### Starting Y Values ######\n");
	for(i=1; i<=Param.n_var; i++)
		fprintf(Fp_out, "y[%2d]=%e	#%s\n",
				i, y[i]/GeneConc, getVarTag(i));
	fprintf(Fp_out, "\n####### Parameters ######\n");
	for(i = 1 ; i <= getParamNum() ; i++ ){
		for( j = 1 ; j <= ParamT.n_val[i] ; j++ ){
			fprintf(Fp_out, "%s[%2d]    =%e	#%s\n",
				Pno2Name(i), j,
				ParamT.val[i][j], getParamTag(i, j));
		}
	}

	 /* ।٥ȥǡν */
	if (SolverNo<10 && EventsNum>0) {	 /* ȯŸ׻ǥ।٥Ȥ */
		fprintf(Fp_out, "\n####### Time Events ######\n");
		for (i=0; i<EventsNum; i++) {
			if( isPno(TIME_START, Events[i].p_no)
			 || isPno(TIME_END, Events[i].p_no) ){
				continue;
			}
			cdum = Pno2Name(Events[i].p_no);
			if( cdum == NULL ){
				log_sys("## Unknown Param No. : %3d\n",
							Events[i].p_no);
				exit(-1);
			}
			fprintf(Fp_out, "at %10.3E:%10s[%3d] => %10.3E\n",
				Events[i].time, cdum,
				Events[i].index, Events[i].value);
		}
	}
	fprintf(Fp_out, "\n\n$$ Results\n\n");


#ifdef DBG
	DBGW(10, ("WriteInputData exit\n"));
#endif /* DBG */

}


/* Print search results per a search */
void PrintResultFile(double *random_vector, double fitness, int myid, int search_no, int ls_ret)
{
	int		i, j;

#ifdef DBG
	DBGW(10, ("PrintResultFile enter.\n"));
#endif /* DBG */
	
	fprintf(Fp_out, "\n\n$$ Random Search No%d-%d\n", myid, search_no);
	fprintf(Fp_out, "fitness");
	for(i = 0; i < P_GaNum; i++){
		fprintf(Fp_out, ", %s[%d]", Pno2Name(P_Ga[i].p_no), P_Ga[i].index);
	}
	fprintf(Fp_out, "\n");
	fprintf(Fp_out, "%g", fitness);
	for(i = 0; i < P_GaNum; i++){
		fprintf(Fp_out, ", %g", random_vector[i]);
	}
	fprintf(Fp_out, "\n");
	
	if(StepNum != 0){
		fprintf(Fp_out, "Time");
		for(i = 0; i < Param.n_var; i++){
			fprintf(Fp_out, ", y[%d]", i + 1);
		}
		fprintf(Fp_out, "\n");

		for(i = 0; i < StepNum; i++){
			for(j = 0; j < Param.n_var + 1; j++){
				fprintf(Fp_out, "%10.3E", SolutionMatrix[i][j]);
				if(j != Param.n_var) fprintf(Fp_out, ", ");
			}
			fprintf(Fp_out, "\n");
		}
	}
	if(ls_ret == -1) fprintf(Fp_out, "Solution not found.\n");

	outfileList[search_no][0] = fitness;
	for(i = 0; i < P_GaNum; i++){
		outfileList[search_no][i+1] = random_vector[i];
	}

#ifdef DBG
	DBGW(10, ("printResultFile exit\n"));
#endif /* DBG */
}

/* Print list */
int PrintList(FILE *fp, int num_search, int flag)
{
	int i, j;

#ifdef DBG
	DBGW(10, ("PrintList enter.\n"));
#endif /* DBG */

	if(flag == F_LIST_SORT){
		for(i = 0; i < num_search; i++){
			sortedIndex[i].total_no = i;
		}
		sort_index(num_search);
	} else if(flag != F_LIST_UNSORT){
		fprintf(stderr, "Warning: Unknown flag. Print unsorted list!\n");
	}
	
	fprintf(fp, "#No, fitness");
	for(i = 0; i < P_GaNum; i++){
		fprintf(fp, ", %s[%d]", Pno2Name(P_Ga[i].p_no), P_Ga[i].index);
	}
	fprintf(fp, "\n");

	for(i = 0; i < num_search; i++){
		fprintf(fp, "No%d-%d", sortedIndex[i].myid, sortedIndex[i].search_no);
		for(j = 0; j < P_GaNum + 1; j++){
			fprintf(fp, ", %g", outfileList[sortedIndex[i].total_no][j]);
		}
		fprintf(fp, "\n");
	}

#ifdef DBG
	DBGW(10, ("PrintList exit(0).\n"));
#endif /* DBG */

	return 0;
}

/***** MPI FUNCTIONS *****/
#ifdef MPI
#include <mpi.h>


/*  */
int SendList(int list_size, int dim, int dest)
{
	int send_size;
	int rv = -1;

#ifdef DBG
	DBGW(10, ("SendList enter.\n"));
#endif /* DBG */

	send_size = list_size;

	if(MPI_Send(&send_size, 1, MPI_INT, dest,
							0, MPI_COMM_WORLD) != MPI_SUCCESS) {
		log_sys(" ## Error: MPI_Send(In SendList1).\n");
		goto out;
	}
	
	
	if(MPI_Send(sortedIndex, list_size * 3, MPI_INT, dest,
							0, MPI_COMM_WORLD) != MPI_SUCCESS) {
		log_sys(" ## Error: MPI_Send(In SendList1).\n");
		goto out;
	}
	
	if(MPI_Send(&(outfileList[0][0]), list_size * (dim + 1), MPI_DOUBLE, dest,
							0, MPI_COMM_WORLD) != MPI_SUCCESS) {
		log_sys(" ## Error: MPI_Send(In SendList2).\n");
		goto out;
	}

	rv = 0;
out:
#ifdef DBG
	DBGW(10, ("SendList exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}

/* μ */
int RecvList(int write_head, int dim)
{
	int rv = -1;
	int recv_size = -1;
	MPI_Status stat;

#ifdef DBG
	DBGW(10, ("RecvList enter.\n"));
#endif /* DBG */

	if(MPI_Recv(&recv_size, 1, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG,
							MPI_COMM_WORLD, &stat) != MPI_SUCCESS){
		log_sys(" ## Error: MPI_Recv(In RecvList1).\n");
		goto out;
	}
	
	if(MPI_Recv(&(sortedIndex[write_head]), recv_size * 3, MPI_INT,
		MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &stat) != MPI_SUCCESS) {	
		log_sys(" ## Error: MPI_Recv(In RecvList2).\n");
		goto out;
	}

	if(MPI_Recv(&(outfileList[write_head][0]), recv_size * (dim+1), MPI_DOUBLE,
		MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &stat) != MPI_SUCCESS) {	
		log_sys(" ## Error: MPI_Recv(In RecvList3).\n");
		goto out;
	}

	rv = recv_size;
out:
#ifdef DBG
	DBGW(10, ("RecvList exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}
	
#endif /* MPI */



		

/***** STATIC FUNCTIONS *****/

/* qsortΤӴؿ */
static int comp_func(const void *index1, const void *index2)
{
	searchIndex_t *sindex1;
	searchIndex_t *sindex2;
	int rv;

#ifdef DBG
	DBGW(10, ("comp_func enter.\n"));
#endif /* DBG */

	sindex1 = (searchIndex_t *)index1;
	sindex2 = (searchIndex_t *)index2;
	
	if(outfileList[sindex1->total_no][0] 
			< outfileList[sindex2->total_no][0]){
		rv = 1;
	} else if(outfileList[sindex1->total_no][0] 
			== outfileList[sindex2->total_no][0]){
		rv = 0;
	} else {
		rv = -1;
	}

#ifdef DBG
	DBGW(10, ("com_func exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}
	
static void sort_index(int num_search)
{
#ifdef DBG
	DBGW(10, ("sort_index enter.\n"));
#endif /* DBG */

	qsort(sortedIndex, num_search, sizeof(searchIndex_t), &comp_func);

#ifdef DBG
	DBGW(10, ("sort_index exit(0)\n"));
#endif /* DBG */
}

/*----------------getCurrentTimeStr------------------------------------------
 *
 * ॹʸ
 *
 *      I:ʤ
 *      O:ʤ
 *      ꥿͡߻ʸ
 *
 */
static char *getCurrentTimeStr()
{
        long    t;
        static char buf[32];

#ifdef DBG
	DBGW(10, ("getCurrentTimeStr enter.\n"));
#endif /* DBG */

        (void)time(&t);
        (void)strcpy(buf, ctime(&t));
        buf[strlen(buf) - 1] = '\0';

#ifdef DBG
	DBGW(10, ("getCurrentTimeStr exit\n"));
#endif /* DBG */

        return buf;
}


