#include <stdio.h>
#include <stdlib.h>

#include "life.h"
#include "rs_utl.h"
#include "utl_alloc.h"
#include "utl_rand.h"

/* ΰ */
int AllocSolutionMatrix(double t_start, double t_end, double dt_save)
{
	int	num_row, num_col;
	int	rv = -1;
#ifdef DBG
	DBGW(10, ("allocSolutionMatrix enter.\n"));
#endif /* DBG */
	
	num_row = (int)((t_end - t_start)/dt_save) + MAX_EVENT_NUM;
	num_col = Param.n_var + 1;

	if((SolutionMatrix = DoubleMatrixAlloc(num_row, num_col)) == NULL){
		  goto out;
	}

	rv = 0;
out:
#ifdef DBG
	DBGW(10, ("allocSolutionMatrix exit(%d)\n", rv));
#endif /* DBG */

	return rv;
}	

/* õѥ᡼ͤꤹ */
void SetRandomParam(double *random_vector)
{
	int		i;
#ifdef DBG
	DBGW(10, ("SetRandomParam enter.\n"));
#endif /* DBG */

	for(i = 0; i < P_GaNum; i++){
		random_vector[i] = GenUniformRand(P_Ga[i].min_value, P_Ga[i].max_value);
		ParamT.val[P_Ga[i].p_no][P_Ga[i].index]	= random_vector[i];
	}
#ifdef DBG
	DBGW(10, ("SetRandomParam exit\n"));
#endif /* DBG */
}

#ifdef MPI
#include <mpi.h>

/* ɤƥ졼֤ */
int DistributeSlaveSeed(int myid, int numprocs)
{
	int i, j;
	int	rv = -1;
	unsigned long	*slave_seed = NULL;

#ifdef DBG
	DBGW(10, ("DistributeSlaveSeed enter.\n"));
#endif /* DBG */

	if(myid == 0){
		if((slave_seed = 
			(unsigned long *)malloc((numprocs - 1)*sizeof(unsigned long))) == NULL){
			goto out;
		}

		SetSeedByTime();
		for(i = 0; i < numprocs - 1;){
			unsigned long candidate;
			candidate = GenIntRandMax();
			for(j = 0; j < i; j++){
				if(slave_seed[j] == candidate) continue;
			}
			slave_seed[i] = candidate;
			i++;
		}
		for(i = 0; i < numprocs - 1; i++){
			if(MPI_Send(&(slave_seed[i]), 1, MPI_UNSIGNED_LONG,
								i + 1, 0, MPI_COMM_WORLD) != MPI_SUCCESS){
				log_sys(" ## Error: MPI_Send(In DistributeSlaveSeed).\n");
				goto out;
			}
		}
	} else {
		unsigned long	my_seed;
		MPI_Status		stat;
		
		if(MPI_Recv(&my_seed, 1, MPI_UNSIGNED_LONG, MPI_ANY_SOURCE,
						MPI_ANY_TAG, MPI_COMM_WORLD, &stat) != MPI_SUCCESS){
				log_sys(" ## Error: MPI_Recv(In DistributeSlaveSeed).\n");
				goto out;
		}
		SetRandSeed(my_seed);
	}

	rv = 0;
out:
	if(slave_seed != NULL) free(slave_seed);

#ifdef DBG
	DBGW(10, ("DistributeSlaveSeed exit(%d)\n", rv));
#endif /* DBG */
	
	return rv;
}

#endif /* MPI */

