/* *****************************************************************************
 * ΥեmainؿޤǤ롣
 * ʴؿθƤӽФطϰʲοޤ̤ꡣ
 *
 *
 * ˡ
 *	[mpirun -np num_of_cpu] LifeSimulator -Iinputfile_name -Ooutputfile_name
 *
 */

#define _HAVE_MAIN_ROUTINE_
#include "life.h"
#ifdef MPI
	#include "mpi.h"
#endif

#include "nr_lu.h"
#include "nr_mutil.h"
#include "eigenvalue.h"

/* եstaticؿΥץȥ */
static void setSurveyValue(int cnt);
static void survey_pre(int *n_survey, int *start_survey, int *end_survey,
			   										int numprocs, int myid);
static int survey_post(int *f_psuccess, int *psuccess, int n_survey, int numprocs,
			int myid, double *y, double t_start, double t_end, double h, double dt_save,
			char **fname);
static int my_init(int argc, char *argv[], int myid, char *fname[]);
static void writeInitData(double y[], double t_start, double t_end, double h,
			   	double dt_save);
static void writeInputData(double y[], double t_start, double t_end, double h,
			   	double dt_save);
static void my_final();
static int makeFinalFile(int numprocs, int n_survey, char **fname);
static char *getCurrentTimeStr();

static int calc_eigenvalues(double *y, FILE *fp_out, int f_isResult);

/*------------------ main --------------------------------------
 *
 *
 */
int main(int argc, char *argv[])
{
    int myid=0, numprocs=1, f_psuccess=0, psuccess;
	int i, i_save=0, ret;
	int i_survey, n_survey=1;
	int start_survey, end_survey; 
    char *fname[FILE_NUM];
	double t_start, t_end, h;
	double dt_save;
	static double y[MAX_VAR_NUM], y_pre[MAX_VAR_NUM], y_start[MAX_VAR_NUM];
	Parameter param_start;
#ifdef MPI
    int namelen;
    char processor_name[MAX_LINE];

    MPI_Init(&argc,&argv);
    MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
    MPI_Comm_rank(MPI_COMM_WORLD,&myid);
    MPI_Get_processor_name(processor_name,&namelen);
#endif

	/* եΥץϥǡɤ߹ */
	if (my_init(argc, argv, myid, fname) < 0) {
		goto err_end;
	}
	if (Set_data(y, &t_start, &t_end, &h, &dt_save, fname) < 0) {
		goto err_end;
	}

	/* ϥե˷׻ѥ᡼ͤ񤭽Ф */
	if (myid == 0) writeInputData(y, t_start, t_end, h, dt_save);

	ParamB = param_start = Param;
	for (i=1;i<=Param.n_var;i++) y_start[i] = y_pre[i] = y[i];

#ifdef MPI
    fprintf(stdout,"##### CPU %d/%d (%s)\n", myid+1, numprocs, processor_name);
#endif

	/* ѥ᡼٥ԤΥ٥η׻Ѥ */
	survey_pre(&n_survey, &start_survey, &end_survey, numprocs, myid);

	/* ѥ᡼٥Υ롼 */
	for (i_survey=start_survey; i_survey<end_survey; i_survey++) {
		if (F_PSurvey) {
			setSurveyValue(i_survey);	/* βΥѥ᡼򥻥åȤ */
			log_sys("\nP_Survey No.%d\n", i_survey+1);
			Param = ParamB;
		}

		ret = DoSolver(param_start, h, t_start, t_end, dt_save, y_start, y, y_pre);

		if( ret < 0 ){
			if( F_PSurvey ){
				goto next_survey;
			} else {
				goto err_end;
			}
		}


#ifndef WIN32
		log_sys("Elapse time : %12.6f\n", log_getElapseTime());
#endif
	
		f_psuccess = 1;
		fprintf(Fp_out, "\n$$ Finish Successfully!!\n");
		log_sys("Finish Successfully!!\n");
		fflush(Fp_out);

		/* ʬΥǤʤϸͭͤ׻ƽ񤭽Ф */
		if (F_PSurvey==0 && (Param.n_varNR==0 || Param.n_varNR ==Param.n_var)
					   	&& myid==0) calc_eigenvalues(y, Fp_out, 1);

next_survey:	/* ѥ᡼٥Υ顼λ */
		i_save = 0;
		if( ret < 0 && F_PSurvey ){
			fprintf(Fp_out, "\n$$ Failed!!\n");
			log_sys("Failed!!\n");
		}

	}	/* end of Parameter Survey loop. */

	
	/*
	 * ѥ᡼٥θ
	 * ѥ᡼٥ʳΤȤϺǽ̤򼡲Ѥ˽񤭽Ф
	 */
	if (survey_post(&f_psuccess, &psuccess, n_survey, numprocs,
						myid, y, t_start, t_end, h, dt_save, fname)) return -1;

	my_final();
#ifdef MPI
	MPI_Finalize();
#endif
	return 0;

err_end:	/* ̾Υ顼λ */
	
	fprintf(Fp_out, "\n$$ ABORT!!\n");
	log_sys("\nABORT!!\n");

	my_final();
#ifdef MPI
	MPI_Finalize();
#endif
	return -1;
}

/*------------------ survey_pre() --------------------------------------
 *
 * ѥ᡼٥
 *
 */
static void
survey_pre(int *n_survey, int *start_survey, int *end_survey, int numprocs, int myid)
{
	int i, i_tmp, n_par_cpu, n_rest;

	if (F_PSurvey) {
		for (i=0; i<P_SurveyNum; i++) *n_survey *= P_Survey[i].steps;
		fprintf(Fp_out, "##### Parameter Survey : %d data sets.\n", *n_survey);
		log_sys("\n##### Parameter Survey : %d data sets.\n", *n_survey);
	
#ifdef MPI
		if (F_Parallel && (*n_survey >= numprocs)) {
			n_par_cpu = *n_survey/numprocs;
			n_rest = *n_survey - numprocs*n_par_cpu;
			i_tmp = n_rest + myid - numprocs;
			i_tmp = (i_tmp > 0) ? i_tmp : 0;
			*start_survey = myid*n_par_cpu + i_tmp;
			i_tmp = (numprocs - myid <= n_rest) ? 1 : 0;
			*end_survey = *start_survey + n_par_cpu + i_tmp;
		} else {
			*start_survey = 0;
			*end_survey = *n_survey;
		}
#else
		*start_survey = 0;
		*end_survey = *n_survey;
#endif
	} else {
		*start_survey = 0;
		*end_survey = 1;
	}
}

/*------------------ survey_post() --------------------------------------
 *
 * ѥ᡼٥θ
 *
 */
static int
survey_post(int *f_psuccess, int *psuccess, int n_survey, int numprocs, int myid,
				double *y, double t_start, double t_end, double h, double dt_save,
				char **fname)
{
#ifdef MPI
	if (!F_PSurvey && (myid==0)) writeInitData(y, t_start, t_end, h, dt_save);

	MPI_Reduce(f_psuccess, psuccess, 1, MPI_INT, MPI_SUM, 0, MPI_COMM_WORLD);
	
	if (myid == 0) *f_psuccess = *psuccess;
	if (*f_psuccess) {
		fprintf(Fp_out, "\n$$ END ALL!!\n");
		fclose(Fp_out); Fp_out=NULL;
		if (myid == 0) { /* ޥϷ̥եĤˤޤȤ롣 */
			if (makeFinalFile(numprocs, n_survey, fname)) goto err;
		}
		log_sys("\nEND ALL!!\n\n\n");
	} else {
		fprintf(Fp_out,
			"\n$$ END ALL, BUT THERE IS NO SUCCESS CASE!!\n");
		log_sys("\n ## END ALL, BUT THERE IS NO SUCCESS CASE!!\n\n\n");
		goto err;
	}
#else
	if (!F_PSurvey) writeInitData(y, t_start, t_end, h, dt_save);
	if (*f_psuccess) {
		fprintf(Fp_out, "\n$$ END ALL!!\n");
		fclose(Fp_out); Fp_out=NULL;
		if (makeFinalFile(1, n_survey, fname)) goto err;
		log_sys("END ALL!!\n");
	} else {
		fprintf(Fp_out,
			"\n$$ END ALL, BUT THERE IS NO SUCCESS CASE!!\n");
		log_sys("\n ## END ALL, BUT THERE IS NO SUCCESS CASE!!\n");
		goto err;
	}
#endif

	return 0;

err:
	return -1;
}

/*---------------- my_init------------------------------------------
 *
 * ץβϡϥեΥץ
 *
 * I:argc, argv
 * O:y, t_start, t_end, h, dt_save
 *
 */
static int
my_init(int argc, char *argv[], int myid, char *fname[])
{
	int i, fcount = 0;
	char myid_file[FILENAME_MAX];
	
#ifdef DBG
	if(log_dbginit(argc, argv)) {
		printf("Cannot open dbgLog file!!");
		goto	err;
	}
#endif

	if(log_sysinit(argc, argv)) {
		printf("Cannot open sysLog file!!");
		goto	err;
	}


#ifdef DBG
	DBGW(10, ("===> initialize enter\n"));
#endif

	/* ץβ -Iϥե̾-Oϥե̾[-dǥХå٥] */
	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
			case 'I':
				fcount++;
				fname[FILE_IN] = strdup(&argv[i][2]);
				break;
			case 'O':
				fcount++;
				fname[FILE_OUT] = strdup(&argv[i][2]);
				break;
			case 'd':
				break;
			case 'p':
				break;
			default:
				log_sys("initialize: invalid argument !!\n");
				goto	err;
			}
		}
	}
	if (fcount != 2) {
		log_sys("initialize: invarid file name!!\n");
		goto	err;
	}

	/* ϥե򥪡ץ󤹤 */
	sprintf(myid_file, "%s_%d", fname[FILE_OUT], myid);
	if((Fp_out = fopen(myid_file, "w")) == NULL){
			log_sys("initialize: outputfile[%s] open err!!\n", myid_file);
			goto	err;
	}
	
	return 0;
err:;
	return -1;
}

/*----------------setSurveyValue------------------------------------------
 *
 * ѥ᡼٥ֹ(ܤɽ͡cnt)˱ParamTͤ򥻥åȤ
 *
 *	I:cnt (ХP_Surveyͤ)
 *	O:ʤ (ХParamTͤ򥻥å)
 *	꥿͡ʤ
 */
static void setSurveyValue(int cnt)
{
	int i, index, steps, step, my_cnt, nsame=1;
	double value;
	char	*name = NULL;
	static int f_direction[MAX_P_SURVEY], last_cnt[MAX_P_SURVEY];

	/* ϥե˲ܤΥ٥񤭽Ф */
	fprintf(Fp_out, "\n$$ PARAMETER SURVEY No. %d : ", cnt+1);

	/* ˱ѥ᡼ͤ׻ */
	for (i=0; i<P_SurveyNum; i++) {
		index = P_Survey[i].index;
		steps = P_Survey[i].steps;
		my_cnt = (cnt/nsame) % steps;

		/* 򥽥СξѹĿǾˤʤ褦ꤹ */
		if (SolverNo > 10) {	/* 򥽥Сξ */
			if (last_cnt[i] != my_cnt && my_cnt == 0) {
				f_direction[i] = (f_direction[i] == 0) ? 1 : 0;
			}
			if (f_direction[i]) {
				step = (steps - 1) -my_cnt;
			} else {
				step = my_cnt;
			}
			last_cnt[i] = my_cnt;
		} else {	/* ȯŸСξ */
			step = my_cnt;
		}

		/* ͤη׻ */
		if (P_Survey[i].f_isArithmetic == true) {	/*  */
			value = P_Survey[i].start_value
				+ P_Survey[i].change_value * step;
		} else {			/*  */
			value = P_Survey[i].start_value
				* pow(P_Survey[i].change_value, step);
		}
		
		/* ParamTͤ򥻥åȡϥեؤͤν񤭽Ф */
		ParamB.val[P_Survey[i].p_no][index] = value;
		name = Pno2Name(P_Survey[i].p_no);
		if( name == NULL ){
			log_sys("## Fatal Error!! in setSurveyValue\n");
			exit(-1);
		}
		if (strcmp(name, CONSTANT_PLAYER) == 0) value /= GeneConc;
		fprintf(Fp_out, "%s[%d] = %10.3E  ", name, index, value);
		nsame *= steps;
	}
	fprintf(Fp_out, "\n\n");
}

 /*----------------writeInitData------------------------------------------
 *
 * ѥ᡼ϥǡʬȺǽ̤Ѥ˽񤭽Ф
 *
 *	I:y, t_start, t_end, h, dt_save
 *	O:ʤ
 *	꥿͡ʤ
 */
static void writeInitData(double y[], double t_start, double t_end, double h, double dt_save)
{
	int i, j;
	FILE *fp_init;

	/* ϥե򥪡ץ󤹤 */
	if((fp_init = fopen(INIT_FILE, "w")) == NULL){
		log_sys("Cannot open initial-data file!!\n");
		exit(-1);
	}

	fprintf(fp_init, "##### %s #####\n\n", getCurrentTimeStr());
	fprintf(fp_init, "%20s:%10d\n", "Solver No.", SolverNo);
	if (Param.n_varNR > 0) {	/* 򤯾 */
		fprintf(fp_init, "%20s:%10d\n", "NR_TRIAL", Nr_trial);
		fprintf(fp_init, "%20s:%10.3E\n", "NR_TOL_F", Nr_tol_f);
		fprintf(fp_init, "%20s:%10.3E\n", "NR_TOL_X", Nr_tol_x);
		fprintf(fp_init, "%20s:%10.5f\n", "NR_RATIO", Nr_ratio);
	}
	if (SolverNo<10) {	/* ȯŸ׻ξ */
		fprintf(fp_init, "%20s:%10.3E\n", "Start Time", t_start);
		fprintf(fp_init, "%20s:%10.3E\n", "End Time", t_end);
		fprintf(fp_init, "%20s:%10.3E\n", "(initial)Time Step", h);
		fprintf(fp_init, "%20s:%10.3E\n", "Monitoring interval", dt_save);
		if (SolverNo == 2) {	/* Ŭξ */
			fprintf(fp_init, "%20s:%10.3E\n", "RK_EPS", Rk_eps);
		}
	}
	fprintf(fp_init, "%20s:%10.3E\n", "G-Value", GeneConc);
	fprintf(fp_init, "\n####### Parameters ######\n");
	for( i = 1 ; i <= getParamNum() ; i++ ){
		for( j = 1 ; j <= ParamT.n_val[i] ; j++ ){
			fprintf(fp_init, "%s[%2d]    =%12.4e # %s\n",
					Pno2Name(i), j, ParamT.val[i][j], getParamTag(i, j));
		}
	}

	/* ʬΥǤʤϸͭͤ׻ƽ񤭽Ф */
	if (F_PSurvey==0 && (Param.n_varNR==0 || Param.n_varNR ==Param.n_var)) {
		calc_eigenvalues(y, fp_init, 0);
	}

	fprintf(fp_init, "\n####### Last Y Values ######\n");
	for(i=1; i<=Param.n_var; i++)
		fprintf(fp_init, "Y_START	;\t%2d;\t%12.4e; %s\t#\n",
							i, y[i]/GeneConc, getVarTag(i));

	/* ϥե򥯥 */
	fclose(fp_init); fp_init = NULL;
}

 /*----------------writeInputData------------------------------------------
  *
  * ϥե˥ȥǡѥ᡼ϥǡʬ񤭽Ф
  *
  *	I:y, t_start, t_end, h, dt_save
  *	O:ʤ
  *	꥿͡ʤ
  *
  */
static void
writeInputData(double y[], double t_start, double t_end, double h, double dt_save)
{
	int i, j;
	char *cdum;

	fprintf(Fp_out, "##### %s #####\n\n", getCurrentTimeStr());
	fprintf(Fp_out, "%-20s:%10d\n", "Solver No.", SolverNo);
	fprintf(Fp_out, "%-20s:%10d\n", "Param Survey", F_PSurvey);
	if (Param.n_varNR > 0 && SolverNo != 12) {	 /* 򤯾 */
		fprintf(Fp_out, "%-20s:%10d\n", "NR_TRIAL", Nr_trial);
		fprintf(Fp_out, "%-20s:%10.3E\n", "NR_TOL_F", Nr_tol_f);
		fprintf(Fp_out, "%-20s:%10.3E\n", "NR_TOL_X", Nr_tol_x);
		fprintf(Fp_out, "%-20s:%10.5f\n", "NR_RATIO", Nr_ratio);
		fprintf(Fp_out, "%-20s:%10.5f\n", "NR_SENS_CW", Nr_sens_cw);
	}
	if (SolverNo<10) {	 /* ȯŸ׻ξ */
		fprintf(Fp_out, "%-20s:%10.3E\n", "Start Time", t_start);
		fprintf(Fp_out, "%-20s:%10.3E\n", "End Time", t_end);
		fprintf(Fp_out, "%-20s:%10.3E\n", "(initial)Time Step", h);
		fprintf(Fp_out, "%-20s:%10.3E\n", "Monitoring interval", dt_save);
		if (SolverNo == 2) {	 /* Ŭξ */
			fprintf(Fp_out, "%-20s:%10.3E\n", "RK_EPS", Rk_eps);
		}
		if (SolverNo == 3) {	 /* NDFξ */
			fprintf(Fp_out, "%-20s:%10.3E\n", "NDF_RTOL", Ndf_rtol);
			fprintf(Fp_out, "%-20s:%10.3E\n", "NDF_ATOL", Ndf_atol);
		}
	}
	fprintf(Fp_out, "%-20s:%10.3E\n", "G-Value", GeneConc);
	if (SolverNo != 12) {
		fprintf(Fp_out, "\n####### Starting Y Values ######\n");
		for(i=1; i<=Param.n_var; i++)
			fprintf(Fp_out, "y[%2d]=%12.4e	#%s\n",
					i, y[i]/GeneConc, getVarTag(i));
	}
	fprintf(Fp_out, "\n####### Parameters ######\n");
	for(i = 1 ; i <= getParamNum() ; i++ ){
		for( j = 1 ; j <= ParamT.n_val[i] ; j++ ){
			fprintf(Fp_out, "%s[%2d]    =%12.4e	#%s\n",
				Pno2Name(i), j,
				ParamT.val[i][j], getParamTag(i, j));
		}
	}

	 /* ।٥ȥǡν */
	if (SolverNo<10 && EventsNum>0) {	 /* ȯŸ׻ǥ।٥Ȥ */
		fprintf(Fp_out, "\n####### Time Events ######\n");
		for (i=0; i<EventsNum; i++) {
			if( isPno(TIME_START, Events[i].p_no)
			 || isPno(TIME_END, Events[i].p_no) ){
				continue;
			}
			cdum = Pno2Name(Events[i].p_no);
			if( cdum == NULL ){
				log_sys("## Unknown Param No. : %3d\n",
							Events[i].p_no);
				exit(-1);
			}
			fprintf(Fp_out, "at %10.3E:%10s[%3d] => %10.3E\n",
				Events[i].time, cdum,
				Events[i].index, Events[i].value);
		}
	}
	fprintf(Fp_out, "\n\n$$ Results\n\n");
}

/*---------------- makeFinalFile------------------------------------------
 *
 * ׻λCPUȤ˽񤭽Ф̥ե礹롣
 *
 *	I:numprocsfname_out
 *	O:ʤ
 *	꥿͡0/-1 (/)
 *
 */
int makeFinalFile(int numprocs, int n_survey, char **fname)
{
	int i;
	char tline[MAX_LINE], fname_out[MAX_LINE];
	FILE *fp_in, *fp_out;
	
	strcpy(fname_out, fname[FILE_OUT]);

	if (n_survey < numprocs) {	 /* ٥Ρɿ꾮node0Ƿ׻ */
		sprintf(tline, "mv -f %s_0 %s", fname[FILE_OUT], fname[FILE_OUT]);
		system(tline);
	} else {	 /* ΡɤȤ˷׻ͤĤˤޤȤ */
		sprintf(tline, "%s_0", fname_out);	 /* ǽnode0Υǡɤ */
		if((fp_in = fopen(tline, "r")) == NULL){
				log_sys("Cannot open %s!!\n", tline);
				goto	err;
		}
		if((fp_out = fopen(fname_out, "w")) == NULL){
				log_sys("Cannot open %s!!\n", fname_out);
				goto	err;
		}
		 /* node0ΥǡǸιԤϥե˽񤭽Ф */
		while(fgets(tline, sizeof tline, fp_in)) {
			if (strncmp(tline, "$$ END", 5) == 0) break;
			fprintf(fp_out, "%s", tline);
		}
		fclose(fp_in); fp_in = NULL;
		for (i=1; i<numprocs; i++) { /* node1ʹߤΥեΥǡɤ߹ǽ񤭽Ф */
			sprintf(tline, "%s_%d", fname_out, i);
			if((fp_in = fopen(tline, "r")) == NULL){
					log_sys("Cannot open %s!!\n", tline);
					goto	err;
			}
			fgets(tline, sizeof tline, fp_in);
			while(fgets(tline, sizeof tline, fp_in) != NULL) {
				if (strncmp(tline, "$$ END", 5) == 0) break;
				fprintf(fp_out, "%s", tline);
			}
			fclose(fp_in); fp_in = NULL;
		}
		fprintf(fp_out, "\n$$ END ALL!!\n");
		fclose(fp_out); fp_out = NULL;
	}

	return 0;

err:;

	return -1;
}

static void my_final()
{
	/*
	 * ѥ᡼γǼΰγ
	 */
	freeParamTag();

	/*
	 * ϥեĤ
	 */
	if( Fp_out ){
		fclose(Fp_out); Fp_out = NULL;
	}

	/*
	 * λ
	 */
	log_sysfinalize();
	log_dbgfinalize();
}

/*----------------getCurrentTimeStr------------------------------------------
 *
 * ॹʸ
 *
 *	I:ʤ
 *	O:ʤ
 *	꥿͡߻ʸ
 *
 */
static char *getCurrentTimeStr()
{
	long	t;
	static char buf[32];

	(void)time(&t);
	(void)strcpy(buf, ctime(&t));
	buf[strlen(buf) - 1] = '\0';
	return buf;
}

/*---------------- calc_eigenvalues ------------------------------------------
 *	ͭͤ׻ѿ͡Flux ȶ˽Ϥ롣
 */
static int
calc_eigenvalues(double *y, FILE *fp_out, int f_isResult)
{
	int i, j, n_var = Param.n_var;
	double **Sd=NULL, *real=NULL, *imaginary=NULL;
	double fjac[MAX_VAR_NUM][MAX_VAR_NUM];

	Sd = nr_alloc_matrix(1, n_var, 1, n_var);
	if (Sd == NULL) goto err;
	real = nr_alloc_vector(1, n_var);
	if (real == NULL) goto err;
	imaginary = nr_alloc_vector(1, n_var);
	if (imaginary == NULL) goto err;

	for (i=1; i<=n_var; i++) {
		for (j=1; j<=n_var; j++) {
			fjac[i][j] = 0.0;
		}
	}

	usr_fjac(fjac, y, GeneConc, &Param);

	for (i=1; i<=n_var; i++) {
		for (j=1; j<=n_var; j++) {
			Sd[i][j] =fjac[i][j];
		}
	}
	
#ifdef DBG
	nr_dump_matrix("Sd before", Sd, n_var, n_var);
#endif


	GetEigenvalues(Sd, real, imaginary, n_var);
	
#ifdef DBG
	nr_dump_matrix("Sd after", Sd, n_var, n_var);
#endif

	if (f_isResult) {
		fprintf(fp_out, "\n$$ Eigenvalues");
		if (SolverNo < 10) {
			fprintf(fp_out, " (calc. with last y values)\n");
		} else {
			fprintf(fp_out, "\n");
		}
		fprintf(fp_out, "\n  No.,        Real,   Imaginary\n");
	} else {
		fprintf(fp_out, "\n##### Eigenvalues #####\n");
		fprintf(fp_out, "\n  No.        Real     Imaginary\n");
	}
	for(i=1; i<=n_var; i++) {
		if (f_isResult) {
			fprintf(fp_out, "%5d,%12.3e,%12.3e\n", i, real[i], imaginary[i]);
		} else {
			fprintf(fp_out, "%5d %12.3e %12.3e\n", i, real[i], imaginary[i]);
		}
	}

	if (Sd) nr_free_matrix(Sd, 1, n_var, 1, n_var);
	if (real) nr_free_vector(real, 1, n_var);
	if (imaginary) nr_free_vector(imaginary, 1, n_var);

	return 0;

err:;
	log_sys("## Error in calc_eigenvalues.\n");
	if (Sd) nr_free_matrix(Sd, 1, n_var, 1, n_var);
	if (real) nr_free_vector(real, 1, n_var);
	if (imaginary) nr_free_vector(imaginary, 1, n_var);
	return -1;
}

#undef _HAVE_MAIN_ROUTINE_
