#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#endif
#include <errno.h>
#include "logUtil.h"

#define	DBGOPT		"-d"
#define	DBG_PREFIX	"dbg_log"
#define	SYS_PREFIX	"sys_log"
#define	LOG_DIR		"."		//"LOGs"



char	*Dbg_filename;
int		Dbg_line;

int Dbg_Level = -1;
static	FILE *dbg_fp = NULL;
static	FILE *Sys_fp = NULL;

#ifndef WIN32
static	struct timeval start_time;

static char * log_getCurrentTimeStr()
{
	long	t;
	static char buf[32];

	(void)time(&t);
	(void)strcpy(buf, ctime(&t));
	buf[strlen(buf) - 1] = '\0';
	return buf;
}

double log_getElapseTime()
{
	struct timeval current_time;

	gettimeofday(&current_time, 0);
	return (double)(current_time.tv_sec  - start_time.tv_sec)
		 + (double)(current_time.tv_usec - start_time.tv_usec)/1e6;
}
#endif
/*************** dbg_log **********************/
int log_dbginit(int argc, char *argv[])
{
	char *procname;
	register int i;
	unsigned int optlen = strlen(DBGOPT);
#ifndef LOGtoSTD
	char dbglogfile[FNAME_SIZE];
#endif

	if ((procname = strrchr(argv[0], '/')))
		procname++;
	else
		procname = argv[0];

	for (i = 1; i < argc; i++) {
		if (!strncmp(argv[i], DBGOPT, optlen)) {
			if (strlen(argv[i]) > optlen)
				Dbg_Level = atoi(&argv[i][optlen]);
			else if (argv[i+1])
				Dbg_Level = atoi(argv[i+1]);
			else
				Dbg_Level = 0;
		}
	}

#ifdef LOGtoSTD
	dbg_fp = stdout;
#else
#ifdef WIN32
	(void)sprintf(dbglogfile, "%s.txt", DBG_PREFIX);
#else
	(void)sprintf(dbglogfile, "%s/%s.%s.%d"
			, LOG_DIR, DBG_PREFIX, procname, getpid());
#endif
	if (!(dbg_fp = fopen(dbglogfile, "w"))){
		perror("Error!! in log_dbginit");
		return -1;
	}
	setbuf(dbg_fp, (char *)NULL);
#endif

#ifndef WIN32
	(void)fprintf(dbg_fp, "%s : Start!! : %s\n", log_getCurrentTimeStr(), procname);

	if (! start_time.tv_sec) {
		if (gettimeofday(&start_time, 0)) {
			perror("Error!! in log_dbginit");
			return -1;
		}
	}
#endif

	return 0;
}

void log_dbgfinalize(void)
{
	if (dbg_fp != NULL) fclose(dbg_fp);
}

int log_dbg(char *format, ...)
{
	int	err = errno;
	va_list ap;

	va_start(ap,format);
	(void)vfprintf(dbg_fp, format, ap);
	va_end(ap);
	errno = err;
	return 0;
}

int log_dbg_l(char *format, ...)
{
	int	err = errno;
	va_list ap;

	va_start(ap,format);
	(void)fprintf(dbg_fp, "%-16s %4d : ", Dbg_filename, Dbg_line);
	(void)vfprintf(dbg_fp, format, ap);
	va_end(ap);
	errno = err;
	return 0;
}

#ifndef WIN32
int log_dbg_t(char *format, ... )
{
	int err = errno;
	va_list ap;

	va_start(ap,format);
	(void)fprintf(dbg_fp,"%s ===> time:%10.6f ===> "
					, log_getCurrentTimeStr()
					, log_getElapseTime());
	(void)vfprintf(dbg_fp, format, ap);
	va_end(ap);
	errno = err;
	return 0;
}
#endif

/********** sys_log ************************/
int log_sysinit(int argc, char *argv[])
{
	char *procname;
#ifndef LOGtoSTD
	char syslogfile[FNAME_SIZE];
#endif

	if ((procname = strrchr(argv[0], '/')))
		procname++;
	else
		procname = argv[0];

#ifdef LOGtoSTD
	Sys_fp = stdout;
#else
#ifdef WIN32
	(void)sprintf(syslogfile, "%s.txt", SYS_PREFIX);
#else
	(void)sprintf(syslogfile, "%s/%s.%s.%d"
			, LOG_DIR, SYS_PREFIX, procname, getpid());
#endif
	if (!(Sys_fp = fopen(syslogfile, "w"))){
		perror("Error!! in log_sysinit");
		return -1;
	}
	setbuf(Sys_fp, (char *)NULL);
#endif

#ifndef WIN32
	(void)fprintf(Sys_fp, "%s : Start!! : %s\n", log_getCurrentTimeStr(), procname);

	if (! start_time.tv_sec) {
		if (gettimeofday(&start_time, 0)) {
			perror("Error!! in log_sysinit");
			return -1;
		}
	}
#endif

	return 0;
}

void log_sysfinalize(void)
{
	if (Sys_fp != NULL) fclose(Sys_fp);
}

void log_sys(char *format, ...)
{
	int	err = errno;
	va_list ap;

	va_start(ap,format);
	//(void)fprintf(Sys_fp, "%-16s %4d : ", Dbg_filename, Dbg_line);
	(void)vfprintf(Sys_fp, format, ap);
	va_end(ap);
	errno = err;
	return;
}

#ifndef WIN32
void log_sys_t(char *format, ... )
{
	int err = errno;
	va_list ap;

	va_start(ap,format);
	(void)fprintf(Sys_fp,"%s ===> time:%10.6f ===> "
					, log_getCurrentTimeStr()
					, log_getElapseTime());
	(void)vfprintf(Sys_fp, format, ap);
	va_end(ap);
	errno = err;
	return;
}
#endif

#ifdef _TEST_
/*   
*	 Main routine for testing this file.
*
*	COMPILE:
*		gcc -D_TEST_ [ -DDBG ] [ -D_LODtoSTD_ ] [ -o object_file_name ] log_util.c
*
*	USAGE:
*		a.out [-d?]	( ? : 0 to 9 )
*
*/

int main(int argc, char *argv[])
{
	int i, j;
	double dum = 2.0;

	if(log_sysinit(argc, argv)) {
		printf("Cannot open sys_log file!!\n");
		exit (-1);
	}

	if(log_dbginit(argc, argv)) {
		printf("Cannot open dbg_log file!!\n");
		exit (-1);
	}

	for (i=0; i<10; i++) {
		log_sys("main sys:%10.6f\n", log_getElapseTime());
		log_sys_t("main sys_t\n");
		log_dbg("in TEST_main dbg\n");
		DBGW(1,("in TEST_main dbg\n"));
		DBGW(3,("in TEST_main dbg%2d\n", i));
		DBGW_T(1,("in TEST_main DBGW_T dum=%e\n", dum));
		DBGW_L(1,("in TEST_main DBGW_L dum=%e\n", dum));
		//sleep(1);
		//for (j=0; j<100000; j++) dum=pow(dum, 1.0/j);
	}

	log_dbgfinalize();
	log_sysfinalize();

}
#endif
