package jp.co.suri.lifesim;
import java.lang.*;
import java.util.*;
import jp.co.suri.lifesim.*;
import org.w3c.dom.*;
import org.apache.xerces.parsers.*;

/**
 * InputXML
 *
 * SANAC ե򳫤Xerces ǥѡ
 * 줿ǡ JModelPack ֥Ȥ˳Ǽ֤
 */
public class InputXML {
	Document document = null;
	DOMParser parser = new DOMParser();

	private boolean debug;

	private JListOfCompartments listOfCompartments;
	private int[] specieID;
	private Vector refID;

	/**
	 * 󥹥ȥ饯
	 */
	public InputXML() {
		// ǥХåϥե饰
		debug = false;
		//debug = true;
	}

	/**
	 * parseXML
	 *
	 * XML եɤ߹ߡfiletype ˱ѡԤ
	 * 줿ǡ JModelPack ֥Ȥ˳Ǽ֤
	 *
	 * @param filename ɤ߹ XML ե̾
	 * @param filetype ɤ߹ XML եη
	 * @param modelPack ̤Ǽ JModelPack ֥
	 * @return ｪλʤ 0, ʳʤ 1
	 */
	public int parseXML (String filename, int filetype, JModelPack modelPack) {
		modelPack.result = JModelPack.PARSEDAE_OK;

		DOMParser parser;
		Document doc;

		try{
			parser = new DOMParser();
			parser.parse(filename);
			doc = parser.getDocument();
			Element root = doc.getDocumentElement();

			if (! root.getNodeName().equals("sanac")) {
				throw (new Exception("element SANAC not found."));
			}

			NodeList nodes = root.getChildNodes();
			Node modelNode = null;
			for (int i = 0; i < nodes.getLength(); i++) {
				Node node = nodes.item(i);
				if (node.getNodeName().equals("model")) {
					if (modelNode == null) {
						modelNode = node;
					} else {
						throw (new Exception("node 'model' found 2 times.\n"));
					}
				}
			}

			if (modelNode == null) {
				throw (new Exception("node 'model' not found.\n"));
			} else {
				modelPack.model = getModel(modelNode);
			}
		} catch (Exception e){
			modelPack.errmsg = e.getMessage();
			modelPack.result = JModelPack.PARSEDAE_NG;
		}

		return 0;
	}

	/**
	 * model Ρɤɤࡣ
	 */
	private JModel getModel (Node modelNode) throws Exception {
		String myname = "model";
		JModel model = new JModel();

		NamedNodeMap attrs = modelNode.getAttributes();
		model.name = getAttrStrByName(attrs, "name", true, myname);
		model.modelInfo = getAttrStrByName(attrs, "modelInfo");

		NodeList nodes = modelNode.getChildNodes();

		Node listOfCompartmentsNode = getNodeByName(nodes, "listOfCompartments");
		Node listOfSpeciesNode = getNodeByName(nodes, "listOfSpecies");
		Vector listOfReactionsNodes = getNodesByName(nodes, "listOfReactions");

		model.listOfCompartments = getListOfCompartments(listOfCompartmentsNode);
		this.listOfCompartments = model.listOfCompartments;
		model.listOfSpecies = getListOfSpecies(listOfSpeciesNode);

		int listOfReactions_n = listOfReactionsNodes.size();
		if (listOfReactions_n > 0) {
			JListOfReactions[] listOfReactions = new JListOfReactions[listOfReactions_n];
			for (int i = 0; i < listOfReactions_n; i++) {
				listOfReactions[i] = getListOfReactions((Node)listOfReactionsNodes.elementAt(i));
			}
			model.listOfReactions_n = listOfReactions_n;
			model.listOfReactions = listOfReactions;
		} else {
			throw (new Exception("node 'listOfReactions' not found.\n"));
		}

		return model;
	}

	/**
	 * listOfCompartments Ρɤɤࡣ
	 */
	private JListOfCompartments getListOfCompartments(Node listOfCompartmentsNode) throws Exception {
		String myname = "listOfCompartments";
		JListOfCompartments listOfCompartments = new JListOfCompartments();
		NamedNodeMap attrs = listOfCompartmentsNode.getAttributes();
		listOfCompartments.numberOfCompartments = getAttrIntByName(attrs, "numberOfCompartments", true, myname);

		NodeList nodes = listOfCompartmentsNode.getChildNodes();
		Vector compartmentNodes = getNodesByName(nodes, "compartment");

		int compartment_n = compartmentNodes.size();

		if (listOfCompartments.numberOfCompartments != compartment_n) {
			throw (new Exception("ListOfCompartments.numberOfCompartments is not equal to actual number of compartments.\n"));
		}

		if (compartment_n > 0) {
			JCompartment[] compartment = new JCompartment[compartment_n];
			for (int i = 0; i < compartment_n; i++) {
				compartment[i] = getCompartment((Node)compartmentNodes.elementAt(i), i+1);
			}
			listOfCompartments.compartment = compartment;
		} else {
			throw (new Exception("node 'compartment' not found.\n"));
		}

		return listOfCompartments;
	}

	/**
	 * compartment Ρɤɤࡣ
	 */
	private JCompartment getCompartment(Node compartmentNode, int compartmentID) throws Exception {
		String myname = "compartment";
		JCompartment compartment = new JCompartment();
		NamedNodeMap attrs = compartmentNode.getAttributes();
		compartment.compartmentID = compartmentID;
		compartment.name = getAttrStrByName(attrs, "name");
		compartment.volume = getAttrStrByName(attrs, "volume");

		return compartment;
	}

	/**
	 * ꤵ줿 ID  this.specieID[] ¸ߤ뤫Ĵ٤롣
	 */
	private void checkSpecieID (int id) throws Exception {
		int[] specieID = this.specieID;
		int i;

		for (i = 0; i < specieID.length; i++) {
			if (specieID[i] == id)
				break;
		}
		if (i == specieID.length) {
			throw (new Exception ("specieReference.specieID [" + id + "] is not defined in ListOfSpecies.\n"));
		}
	}

	/**
	 * listOfSpecies Ρɤɤࡣ
	 */
	private JListOfSpecies getListOfSpecies(Node listOfSpeciesNode) throws Exception {
		String myname = "listOfSpecies";
		JListOfSpecies listOfSpecies = new JListOfSpecies();
		NamedNodeMap attrs = listOfSpeciesNode.getAttributes();
		listOfSpecies.numberOfSpecies = getAttrIntByName(attrs, "numberOfSpecies", true, myname);

		NodeList nodes = listOfSpeciesNode.getChildNodes();
		Vector specieNodes = getNodesByName(nodes, "Specie");

		int specie_n = specieNodes.size();

		if (listOfSpecies.numberOfSpecies != specie_n) {
			throw (new Exception("ListOfSpecies.numberOfSpecies is not equal to actual number of species.\n"));
		}

		this.specieID = new int[specie_n];
		this.refID = new Vector();

		if (specie_n > 0) {
			JSpecie[] specie = new JSpecie[specie_n];
			for (int i = 0; i < specie_n; i++) {
				specie[i] = getSpecie((Node)specieNodes.elementAt(i));
				this.specieID[i] = specie[i].specieID;
			}
			for (int i = 0; i < refID.size(); i++) {
				Integer id = (Integer) refID.elementAt (i);
				checkSpecieID (id.intValue());
			}
			listOfSpecies.specie = specie;
		} else {
			throw (new Exception("node 'specie' not found.\n"));
		}
		this.refID = null;

		return listOfSpecies;
	}

	/**
	 * specie Ρɤɤࡣ
	 */
	private JSpecie getSpecie(Node specieNode) throws Exception {
		String myname = "specie";
		JSpecie specie = new JSpecie();

		NamedNodeMap attrs = specieNode.getAttributes();
		specie.specieID = getAttrIntByName(attrs, "specieID", true, myname);
		specie.name = getAttrStrByName(attrs, "name", true, myname);
		specie.compartment = getCompartmentNumberByName(attrs, "compartment", true, myname);

		NodeList nodes = specieNode.getChildNodes();

		Node specieAttributeNode = getNodeByName(nodes, "specieAttribute");
		Vector listOfElementsNodes = getNodesByName(nodes, "listOfElements");

		specie.specieAttribute = getSpecieAttribute(specieAttributeNode);

		int listOfElements_n = listOfElementsNodes.size();

		if (listOfElements_n == 1) {
			JListOfElements[] listOfElements = new JListOfElements[1];
			listOfElements[0] = getListOfElements((Node)listOfElementsNodes.elementAt(0));
			specie.listOfElements_n = listOfElements_n;
			specie.listOfElements = listOfElements;
		} else if (listOfElements_n == 0) {
			specie.listOfElements_n = 0;
			specie.listOfElements = null;
		} else {
			throw (new Exception("node 'listOfElements' found 2 or more times.\n"));
		}
		return specie;
	}

	/**
	 * specieAttribute Ρɤɤࡣ
	 */
	private JSpecieAttribute getSpecieAttribute(Node specieAttributeNode) throws Exception {
		String myname = "specieAttribute";
		JSpecieAttribute specieAttribute = new JSpecieAttribute();

		NamedNodeMap attrs = specieAttributeNode.getAttributes();
		specieAttribute.specieClass = getSpecieClassNumberByName(attrs, "specieClass", true, myname);
		specieAttribute.massBalance = getMassBalanceNumberByName(attrs, "massBalance", true, myname);
		specieAttribute.totalAmount = getTotalAmountNumberByName(attrs, "totalAmount", true, myname);
		specieAttribute.decomposition = getDecompositionNumberByName(attrs, "decomposition", true, myname);
		specieAttribute.initialAmount = getAttrStrByName(attrs, "initialAmount");
		if (specieAttribute.initialAmount == null) {
			specieAttribute.initialAmount_isValid = JModelPack.PARSEDAE_FALSE;
		} else {
			specieAttribute.initialAmount_isValid = JModelPack.PARSEDAE_TRUE;
		}

		return specieAttribute;
	}

	/**
	 * listOfElements Ρɤɤࡣ
	 */
	private JListOfElements getListOfElements(Node listOfElementsNode) throws Exception {
		String myname = "listOfElements";
		JListOfElements listOfElements = new JListOfElements();
		NamedNodeMap attrs = listOfElementsNode.getAttributes();
		listOfElements.numberOfElements = getAttrIntByName(attrs, "numberOfElements", true, myname);

		NodeList nodes = listOfElementsNode.getChildNodes();
		Vector specieReferenceNodes = getNodesByName(nodes, "specieReference");

		int specieReference_n = specieReferenceNodes.size();

		if (listOfElements.numberOfElements != specieReference_n) {
			throw (new Exception("ListOfElements.numberOfElements is not equal to actual number of elements.\n"));
		}

		if (specieReference_n > 0) {
			JSpecieReference[] specieReference = new JSpecieReference[specieReference_n];
			for (int i = 0; i < specieReference_n; i++) {
				specieReference[i] = getSpecieReference((Node)specieReferenceNodes.elementAt(i));
				refID.add (new Integer (specieReference[i].specieID));
			}
			listOfElements.specieReference = specieReference;
		} else {
			throw (new Exception("node 'specieReference' not found.\n"));
		}

		return listOfElements;
	}

	/**
	 * specieReference Ρɤɤࡣ
	 */
	private JSpecieReference getSpecieReference(Node specieReferenceNode) throws Exception {
		String myname = "specieReference";
		JSpecieReference specieReference = new JSpecieReference();

		NamedNodeMap attrs = specieReferenceNode.getAttributes();
		specieReference.specieID = getAttrIntByName(attrs, "specieID", true, myname);
		specieReference.name = getAttrStrByName(attrs, "name", true, myname);
		specieReference.compartment = getCompartmentNumberByName(attrs, "compartment", true, myname);
		specieReference.stoichiometry = getAttrStrByName(attrs, "stoichiometry", true, myname);

		return specieReference;
	}

	/**
	 * listOfReactions Ρɤɤࡣ
	 */
	private JListOfReactions getListOfReactions(Node listOfReactionsNode) throws Exception {
		String myname = "listOfReactions";
		JListOfReactions listOfReactions = new JListOfReactions();
		NamedNodeMap attrs = listOfReactionsNode.getAttributes();
		listOfReactions.numberOfReactions = getAttrIntByName(attrs, "numberOfReactions", true, myname);
		listOfReactions.layer = getLayerNumberByName(attrs, "layer", true, myname);

		NodeList nodes = listOfReactionsNode.getChildNodes();
		Vector reactionNodes = getNodesByName(nodes, "Reaction");

		int reaction_n = reactionNodes.size();

		if (listOfReactions.numberOfReactions != reaction_n) {
			throw (new Exception("ListOfReactions.numberOfReactions is not equal to actual number of reactions.\n"));
		}

		if (reaction_n > 0) {
			JReaction[] reaction = new JReaction[reaction_n];
			for (int i = 0; i < reaction_n; i++) {
				reaction[i] = getReaction((Node)reactionNodes.elementAt(i));
			}
			listOfReactions.reaction = reaction;
		} else {
			throw (new Exception("node 'reaction' not found.\n"));
		}

		return listOfReactions;
	}

	/**
	 * reaction Ρɤɤࡣ
	 */
	private JReaction getReaction(Node reactionNode) throws Exception {
		String myname = "reaction";
		JReaction reaction = new JReaction();

		NamedNodeMap attrs = reactionNode.getAttributes();
		reaction.reactionID = getAttrIntByName(attrs, "reactionID", true, myname);
		reaction.groupID = getAttrIntByName(attrs, "groupID", true, myname);
		reaction.reactionType = getReactionTypeNumberByName(attrs, "reactionType", true, myname);
		reaction.isConvertible = getAttrBoolByName(attrs, "isConvertible", true, myname);
		reaction.modifierReactionEquation = getAttrStrByName(attrs, "modifierReactionEquation", true, myname);

		NodeList nodes = reactionNode.getChildNodes();

		Vector listOfReactantsNodes = getNodesByName(nodes, "listOfReactants");
		Vector listOfProductsNodes = getNodesByName(nodes, "listOfProducts");
		Vector listOfModifiersNodes = getNodesByName(nodes, "listOfModifiers");
		Vector modifierComplexNodes = getNodesByName(nodes, "modifierComplex");

		int listOfReactants_n = listOfReactantsNodes.size();

		if (listOfReactants_n > 0) {
			JListOfReactants[] listOfReactants = new JListOfReactants[listOfReactants_n];
			for (int i = 0; i < listOfReactants_n; i++) {
				listOfReactants[i] = getListOfReactants((Node)listOfReactantsNodes.elementAt(i));
			}
			reaction.listOfReactants_n = listOfReactants_n;
			reaction.listOfReactants = listOfReactants;
		} else {
			reaction.listOfReactants_n = 0;
			reaction.listOfReactants = null;
		}

		int listOfProducts_n = listOfProductsNodes.size();
		if (listOfProducts_n == 1) {
			JListOfProducts[] listOfProducts = new JListOfProducts[1];
			listOfProducts[0] = getListOfProducts((Node)listOfProductsNodes.elementAt(0));
			reaction.listOfProducts_n = listOfProducts_n;
			reaction.listOfProducts = listOfProducts;
		} else if (listOfProducts_n == 0) {
			reaction.listOfProducts_n = 0;
			reaction.listOfProducts = null;
		} else {
			throw (new Exception("node 'listOfProducts' found 2 or more times.\n"));
		}

		int listOfModifiers_n = listOfModifiersNodes.size();
		if (listOfModifiers_n == 1) {
			JListOfModifiers[] listOfModifiers = new JListOfModifiers[1];
			listOfModifiers[0] = getListOfModifiers((Node)listOfModifiersNodes.elementAt(0));
			reaction.listOfModifiers_n = listOfModifiers_n;
			reaction.listOfModifiers = listOfModifiers;
		} else if (listOfModifiers_n == 0) {
			reaction.listOfModifiers_n = 0;
			reaction.listOfModifiers = null;
		} else {
			throw (new Exception("node 'listOfModifiers' found 2 or more times.\n"));
		}

		int modifierComplex_n = modifierComplexNodes.size();
		if (modifierComplex_n == 1) {
			JModifierComplex[] modifierComplex = new JModifierComplex[1];
			modifierComplex[0] = getModifierComplex((Node)modifierComplexNodes.elementAt(0));
			reaction.modifierComplex_n = modifierComplex_n;
			reaction.modifierComplex = modifierComplex;
		} else if (modifierComplex_n == 0) {
			reaction.modifierComplex_n = 0;
			reaction.modifierComplex = null;
		} else {
			throw (new Exception("node 'modifierComplex' found 2 or more times.\n"));
		}

		return reaction;
	}

	/**
	 * listOfReactants Ρɤɤࡣ
	 */
	private JListOfReactants getListOfReactants(Node listOfReactantsNode) throws Exception {
		String myname = "listOfReactants";
		JListOfReactants listOfReactants = new JListOfReactants();
		NamedNodeMap attrs = listOfReactantsNode.getAttributes();
		listOfReactants.numberOfReactants = getAttrIntByName(attrs, "numberOfReactants", true, myname);

		NodeList nodes = listOfReactantsNode.getChildNodes();
		Vector specieReferenceNodes = getNodesByName(nodes, "specieReference");

		int specieReference_n = specieReferenceNodes.size();

		if (listOfReactants.numberOfReactants != specieReference_n) {
			throw (new Exception("ListOfReactants.numberOfReactants is not equal to actual number of reactants.\n"));
		}

		if (specieReference_n > 0) {
			JSpecieReference[] specieReference = new JSpecieReference[specieReference_n];
			for (int i = 0; i < specieReference_n; i++) {
				specieReference[i] = getSpecieReference((Node)specieReferenceNodes.elementAt(i));
				checkSpecieID (specieReference[i].specieID);
			}
			listOfReactants.specieReference = specieReference;
		} else {
			throw (new Exception("node 'specieReference' not found.\n"));
		}

		return listOfReactants;
	}

	/**
	 * listOfProducts Ρɤɤࡣ
	 */
	private JListOfProducts getListOfProducts(Node listOfProductsNode) throws Exception {
		String myname = "listOfProducts";
		JListOfProducts listOfProducts = new JListOfProducts();
		NamedNodeMap attrs = listOfProductsNode.getAttributes();
		listOfProducts.numberOfProducts = getAttrIntByName(attrs, "numberOfProducts", true, myname);

		NodeList nodes = listOfProductsNode.getChildNodes();
		Vector specieReferenceNodes = getNodesByName(nodes, "specieReference");

		int specieReference_n = specieReferenceNodes.size();

		if (listOfProducts.numberOfProducts != specieReference_n) {
			throw (new Exception("ListOfProducts.numberOfProducts is not equal to actual number of products.\n"));
		}

		if (specieReference_n > 0) {
			JSpecieReference[] specieReference = new JSpecieReference[specieReference_n];
			for (int i = 0; i < specieReference_n; i++) {
				specieReference[i] = getSpecieReference((Node)specieReferenceNodes.elementAt(i));
				checkSpecieID (specieReference[i].specieID);
			}
			listOfProducts.specieReference = specieReference;
		} else {
			throw (new Exception("node 'specieReference' not found.\n"));
		}

		return listOfProducts;
	}

	/**
	 * listOfModifiers Ρɤɤࡣ
	 */
	private JListOfModifiers getListOfModifiers(Node listOfModifiersNode) throws Exception {
		String myname = "listOfModifiers";
		JListOfModifiers listOfModifiers = new JListOfModifiers();
		NamedNodeMap attrs = listOfModifiersNode.getAttributes();
		listOfModifiers.numberOfModifiers = getAttrIntByName(attrs, "numberOfModifiers", true, myname);

		NodeList nodes = listOfModifiersNode.getChildNodes();
		Vector modifierSpecieReferenceNodes = getNodesByName(nodes, "modifierSpecieReference");

		int modifierSpecieReference_n = modifierSpecieReferenceNodes.size();

		if (listOfModifiers.numberOfModifiers != modifierSpecieReference_n) {
			throw (new Exception("ListOfModifiers.numberOfModifiers is not equal to actual number of modifiers.\n"));
		}

		if (modifierSpecieReference_n > 0) {
			JModifierSpecieReference[] modifierSpecieReference = new JModifierSpecieReference[modifierSpecieReference_n];
			for (int i = 0; i < modifierSpecieReference_n; i++) {
				modifierSpecieReference[i] = getModifierSpecieReference((Node)modifierSpecieReferenceNodes.elementAt(i));
				checkSpecieID (modifierSpecieReference[i].specieID);
			}
			listOfModifiers.modifierSpecieReference = modifierSpecieReference;
		} else {
			throw (new Exception("node 'modifierSpecieReference' not found.\n"));
		}

		return listOfModifiers;
	}

	/**
	 * modifierSpecieReference Ρɤɤࡣ
	 */
	private JModifierSpecieReference getModifierSpecieReference(Node modifierSpecieReferenceNode) throws Exception {
		String myname = "modifierSpecieReference";
		JModifierSpecieReference modifierSpecieReference = new JModifierSpecieReference();

		NamedNodeMap attrs = modifierSpecieReferenceNode.getAttributes();
		modifierSpecieReference.specieID = getAttrIntByName(attrs, "specieID", true, myname);
		modifierSpecieReference.name = getAttrStrByName(attrs, "name", true, myname);
		modifierSpecieReference.compartment = getCompartmentNumberByName(attrs, "compartment", true, myname);
		modifierSpecieReference.stoichiometry = getAttrStrByName(attrs, "stoichiometry", true, myname);
		modifierSpecieReference.modifierType = getModifierTypeNumberByName(attrs, "modifierType", true, myname);
		modifierSpecieReference.isTotal = getAttrBoolByName(attrs, "isTotal", true, myname);

		return modifierSpecieReference;
	}

	/**
	 * modifierComplex Ρɤɤࡣ
	 */
	private JModifierComplex getModifierComplex(Node modifierComplexNode) throws Exception {
		String myname = "modifierComplex";
		JModifierComplex modifierComplex = new JModifierComplex();

		NodeList nodes = modifierComplexNode.getChildNodes();
		Vector specieReferenceNodes = getNodesByName(nodes, "specieReference");
		Node specieReferenceNode = getNodeByName(nodes, "specieReference");
		modifierComplex.specieReference = getSpecieReference(specieReferenceNode);
		checkSpecieID (modifierComplex.specieReference.specieID);

		return modifierComplex;
	}


/**********************************************************************
   Ū XML ᥽å
 **********************************************************************/

	/**
	 * Ρɥꥹ椫Ρ̾ǻꤷΡɤȤ֤
	 */
	private Vector getNodesByName (NodeList nodes, String name) throws Exception {
		Vector v = new Vector();
		if (nodes == null || name == null) {
			return null;
		}
		for (int i = 0; i < nodes.getLength(); i++) {
			Node node = nodes.item(i);
			if (name.equals(node.getNodeName())) {
				v.add(node);
			}
		}
		return v;
	}

	/**
	 * Ρɥꥹ椫Ρ̾ǻꤷñΡɤ֤
	 */
	private Node getNodeByName (NodeList nodes, String name) throws Exception {
		Vector v = getNodesByName(nodes, name);
		if (v == null) {
			return null;
		}
		int n = v.size();
		if (n < 1) {
			throw (new Exception("node '" + name + "' not found.\n"));
		} else if (n > 1) {
			throw (new Exception("node '" + name + "' found 2 or more times.\n"));
		}
		return (Node)v.elementAt(0);
	}

	/**
	 * °ꥹ椫°̾ǻꤷ°ͤʸ֤
	 */
	private String getAttrStrByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		if (attrs == null || name == null) {
			return null;
		}
		Node node = attrs.getNamedItem(name);
		if (node == null) {
			if (isRequired && parentName != null) {
				throw (new Exception("attribute '" + name + "' in '" + parentName + "' not found.\n"));
			} else {
				return null;
			}
		}
		return node.getNodeValue();
	}

	/**
	 * °ꥹ椫°̾ǻꤷ°ͤʸ֤
	 */
	private String getAttrStrByName(NamedNodeMap attrs, String name) throws Exception {
		return getAttrStrByName(attrs, name, false, null);
	}

	/**
	 * °ꥹ椫°̾ǻꤷ°֤ͤ
	 */
	private int getAttrIntByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		return Integer.valueOf(getAttrStrByName(attrs, name, isRequired, parentName)).intValue();
	}

/**********************************************************************
   °ͤʸ󢪿Ѵ
 **********************************************************************/

	/**
	 * specieAttribute  specieClass ° (int) 롣
	 */
	private int getSpecieClassNumberByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		String s = getAttrStrByName(attrs, name, isRequired, parentName);
		int ret = 0;
		if (s.equals("DNA_gene")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_DNA_GENE;
		} else if (s.equals("DNA_promoter")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_DNA_PROMOTER;
		} else if (s.equals("DNA_enhancer")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_DNA_ENHANCER;
		} else if (s.equals("DNA_others")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_DNA_OTHERS;
		} else if (s.equals("RNA")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_RNA;
		} else if (s.equals("protein")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_PROTEIN;
		} else if (s.equals("metabolite")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_METABOLITE;
		} else if (s.equals("environmental_factor")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_ENVIRONMENTAL_FACTOR;
		} else if (s.equals("ion_signal")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_ION_SIGNAL;
		} else if (s.equals("complex")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_COMPLEX;
		} else if (s.equals("modifier_complex")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_COMPLEX;
		} else if (s.equals("modified")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_MODIFIED;
		} else if (s.equals("small_molecule")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_SMALL_MOLECULE;
		} else if (s.equals("text_option")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_TEXT_OPTION;
		} else if (s.equals("others")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_SPECIECLASS_OTHERS;
		} else {
			throw (new Exception("invalid specieClass value '" + s + "'.\n"));
		}
		return ret;
	}

	/**
	 * specieAttribute  massBalance ° (int) 롣
	 */
	private int getMassBalanceNumberByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		String s = getAttrStrByName(attrs, name, isRequired, parentName);
		int ret = 0;
		if (s.equals("on")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_MASSBALANCE_ON;
		} else if (s.equals("off")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_MASSBALANCE_OFF;
		} else {
			throw (new Exception("invalid massBalance value '" + s + "'.\n"));
		}
		return ret;
	}

	/**
	 * specieAttribute  totalAmount ° (int) 롣
	 */
	private int getTotalAmountNumberByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		String s = getAttrStrByName(attrs, name, isRequired, parentName);
		int ret = 0;
		if (s.equals("constant")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_TOTALAMOUNT_CONSTANT;
		} else if (s.equals("variable")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_TOTALAMOUNT_VARIABLE;
		} else {
			throw (new Exception("invalid totalAmount value '" + s + "'.\n"));
		}
		return ret;
	}

	/**
	 * specieAttribute  decomposition ° (int) 롣
	 */
	private int getDecompositionNumberByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		String s = getAttrStrByName(attrs, name, isRequired, parentName);
		int ret = 0;
		if (s.equals("on")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_DECOMPOSITION_ON;
		} else if (s.equals("off")) {
			ret = JSpecieAttribute.SPECIEATTRIBUTE_DECOMPOSITION_OFF;
		} else {
			throw (new Exception("invalid decomposition value '" + s + "'.\n"));
		}
		return ret;
	}

	/**
	 * specieAttribute  layer ° (int) 롣
	 */
	private int getLayerNumberByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		String s = getAttrStrByName(attrs, name, isRequired, parentName);
		int ret = 0;
		if (s.equals("gene0")) {
			ret = JListOfReactions.LISTOFREACTIONS_LAYER_GENE;
		} else if (s.equals("gene1")) {
			ret = JListOfReactions.LISTOFREACTIONS_LAYER_GENE;
		} else if (s.equals("gene_other")) {
			ret = JListOfReactions.LISTOFREACTIONS_LAYER_GENE;
		} else if (s.equals("gene")) {
			ret = JListOfReactions.LISTOFREACTIONS_LAYER_GENE;
		} else if (s.equals("protein")) {
			ret = JListOfReactions.LISTOFREACTIONS_LAYER_PROTEIN;
		} else if (s.equals("metabolic")) {
			ret = JListOfReactions.LISTOFREACTIONS_LAYER_METABOLIC;
		} else {
			throw (new Exception("invalid layer value '" + s + "'.\n"));
		}
		return ret;
	}

	/**
	 * Reaction  reactionType ° (int) 롣
	 */
	private int getReactionTypeNumberByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		String s = getAttrStrByName(attrs, name, isRequired, parentName);
		int ret = 0;
		if (s.equals("binding")) {
			ret = JReaction.REACTION_REACTIONTYPE_BINDING;
		} else if (s.equals("binding_with_stoichiometric_changes")) {
			ret = JReaction.REACTION_REACTIONTYPE_BINDING_WITH_STOICHIOMETRIC_CHANGES;
		} else if (s.equals("homo_association_or_modification")) {
			ret = JReaction.REACTION_REACTIONTYPE_HOMO_ASSOCIATION_OR_MODIFICATION;
		} else if (s.equals("homo_association_or_modification_with_stoichiometric_changes")) {
			ret = JReaction.REACTION_REACTIONTYPE_HOMO_ASSOCIATION_OR_MODIFICATION_WITH_STOICHIOMETRIC_CHANGES;
		} else if (s.equals("elimination")) {
			ret = JReaction.REACTION_REACTIONTYPE_ELIMINATION;
		} else if (s.equals("elimination_with_stoichiometric_changes")) {
			ret = JReaction.REACTION_REACTIONTYPE_ELIMINATION_WITH_STOICHIOMETRIC_CHANGES;
		} else if (s.equals("reversible_conversion")) {
			ret = JReaction.REACTION_REACTIONTYPE_REVERSIBLE_CONVERSION;
		} else if (s.equals("irreversible_conversion")) {
			ret = JReaction.REACTION_REACTIONTYPE_IRREVERSIBLE_CONVERSION;
		} else if (s.equals("reversible_conversion_regarding_multicomponent")) {
			ret = JReaction.REACTION_REACTIONTYPE_REVERSIBLE_CONVERSION_REGARDING_MULTICOMPONENT;
		} else if (s.equals("irreversible_conversion_regarding_multicomponent")) {
			ret = JReaction.REACTION_REACTIONTYPE_IRREVERSIBLE_CONVERSION_REGARDING_MULTICOMPONENT;
		} else if (s.equals("transport")) {
			ret = JReaction.REACTION_REACTIONTYPE_TRANSPORT;
		} else if (s.equals("option_transport")) {
			ret = JReaction.REACTION_REACTIONTYPE_OPTION_TRANSPORT;
		} else if (s.equals("transcription")) {
			ret = JReaction.REACTION_REACTIONTYPE_TRANSCRIPTION;
		} else if (s.equals("translation")) {
			ret = JReaction.REACTION_REACTIONTYPE_TRANSLATION;
		} else if (s.equals("degradation")) {
			ret = JReaction.REACTION_REACTIONTYPE_DEGRADATION;
		} else {
			throw (new Exception("invalid reactionType value '" + s + "'.\n"));
		}
		return ret;
	}

	/**
	 * modifierSpecieReference  modifierType ° (int) 롣
	 */
	private int getModifierTypeNumberByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		String s = getAttrStrByName(attrs, name, isRequired, parentName);
		int ret = 0;
		if (s.equals("enzyme")) {
			ret = JModifierSpecieReference.MODIFIERSPECIEREFERENCE_MODIFIERTYPE_ENZYME;
		} else if (s.equals("activator")) {
			ret = JModifierSpecieReference.MODIFIERSPECIEREFERENCE_MODIFIERTYPE_ACTIVATOR;
		} else if (s.equals("inhibitor")) {
			ret = JModifierSpecieReference.MODIFIERSPECIEREFERENCE_MODIFIERTYPE_INHIBITOR;
		} else {
			throw (new Exception("invalid modifierType value '" + s + "'.\n"));
		}
		return ret;
	}

	/**
	 * °ꥹ椫°̾ǻꤷ°ͤб true / false ֤ͤ
	 * jutil  boolean ѤJModelPack  int ͤȤ
	 */
	private int getAttrBoolByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		String s = getAttrStrByName(attrs, name, isRequired, parentName);
		int ret = JModelPack.PARSEDAE_FALSE;
		if (s.equals("true")) {
			ret = JModelPack.PARSEDAE_TRUE;
		} else if (s.equals("false")) {
			ret = JModelPack.PARSEDAE_FALSE;
		} else {
			throw (new Exception("invalid " + name + " value '" + s + "'.\n"));
		}
		return ret;
	}

	/**
	 * Compartment ̾ compartmentID (int) 롣
	 *
	 * listOfCompartment  compartment  name פΤõ
	 *  compartment  compartmentID ֤
	 * פΤʤХ顼Ȥ롣
	 */
	private int getCompartmentNumberByName(NamedNodeMap attrs, String name, boolean isRequired, String parentName) throws Exception {
		String s = getAttrStrByName(attrs, name, isRequired, parentName);
		int ret = 0;

		JCompartment[] compartment = this.listOfCompartments.compartment;

		for (int i = 0; i < compartment.length; i++) {
			if (s.equals (compartment[i].name)) {
				ret = compartment[i].compartmentID;
				break;
			}
		}

		if (ret == 0) {
			throw (new Exception("invalid compartment value '" + s + "'.\n"));
		}

		return ret;
	}


/**********************************************************************
   ¾
 **********************************************************************/

	/**
	 * ǥХåѲ̽
	 */
	private void debugPrint (String s) {
		if (debug) {
			System.out.println (s);
		}
	}
}
